@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -37,26 +37,26 @@
 #define PRINTF(fmt, ...) do { if (!getenv("STARPU_SSILENT")) {printf(fmt, ## __VA_ARGS__); }} while(0)
 #define NMAXBLOCKS	128
 
-#define TAG11(k)	((starpu_tag_t)( (1ULL<<60) | (unsigned long long)(k)))
-#define TAG21(k,j)	((starpu_tag_t)(((3ULL<<60) | (((unsigned long long)(k))<<32)	\
-					| (unsigned long long)(j))))
-#define TAG22(k,i,j)	((starpu_tag_t)(((4ULL<<60) | ((unsigned long long)(k)<<32) 	\
-					| ((unsigned long long)(i)<<16)	\
+#define TAG_POTRF(k)	((starpu_tag_t)((1ULL<<60) | (unsigned long long)(k)))
+#define TAG_TRSM(k,j)	((starpu_tag_t)(((3ULL<<60) | (((unsigned long long)(k))<<32)	\
 					| (unsigned long long)(j))))
+#define TAG_GEMM(k,i,j)	((starpu_tag_t)(((4ULL<<60) | ((unsigned long long)(k)<<32) 	\
+					 | ((unsigned long long)(i)<<16) \
+					 | (unsigned long long)(j))))
 
-#define TAG11_AUX(k, prefix)	((starpu_tag_t)( (((unsigned long long)(prefix))<<60)  |  (1ULL<<56) | (unsigned long long)(k)))
-#define TAG21_AUX(k,j, prefix)	((starpu_tag_t)( (((unsigned long long)(prefix))<<60)  			\
-					|  ((3ULL<<56) | (((unsigned long long)(k))<<32)	\
-					| (unsigned long long)(j))))
-#define TAG22_AUX(k,i,j, prefix)    ((starpu_tag_t)(  (((unsigned long long)(prefix))<<60)	\
-					|  ((4ULL<<56) | ((unsigned long long)(k)<<32)  	\
-					| ((unsigned long long)(i)<<16) 			\
-					| (unsigned long long)(j))))
+#define TAG_POTRF_AUX(k, prefix)	((starpu_tag_t)((((unsigned long long)(prefix))<<60)  |  (1ULL<<56) | (unsigned long long)(k)))
+#define TAG_TRSM_AUX(k,j, prefix)	((starpu_tag_t)((((unsigned long long)(prefix))<<60) \
+							 |  ((3ULL<<56) | (((unsigned long long)(k))<<32) \
+							     | (unsigned long long)(j))))
+#define TAG_GEMM_AUX(k,i,j, prefix)    ((starpu_tag_t)((((unsigned long long)(prefix))<<60) \
+							 |  ((4ULL<<56) | ((unsigned long long)(k)<<32)	\
+							     | ((unsigned long long)(i)<<16) \
+							     | (unsigned long long)(j))))
 
 #define BLOCKSIZE	(size_p/nblocks_p)
 
 #define BLAS3_FLOP(n1,n2,n3)    \
-        (2*((uint64_t)n1)*((uint64_t)n2)*((uint64_t)n3))
+	(2*((uint64_t)n1)*((uint64_t)n2)*((uint64_t)n3))
 
 /* This is from magma
 
@@ -93,26 +93,32 @@
   */
 
 #define FMULS_POTRF(__n) ((double)(__n) * (((1. / 6.) * (double)(__n) + 0.5) * (double)(__n) + (1. / 3.)))
-#define FADDS_POTRF(__n) ((double)(__n) * (((1. / 6.) * (double)(__n)      ) * (double)(__n) - (1. / 6.)))
+#define FADDS_POTRF(__n) ((double)(__n) * (((1. / 6.) * (double)(__n)) * (double)(__n) - (1. / 6.)))
 
-#define FLOPS_SPOTRF(__n) (     FMULS_POTRF((__n)) +       FADDS_POTRF((__n)) )
+#define FLOPS_SPOTRF(__n) (FMULS_POTRF((__n)) + FADDS_POTRF((__n)))
 
 #define FMULS_TRMM_2(__m, __n) (0.5 * (double)(__n) * (double)(__m) * ((double)(__m)+1.))
 #define FADDS_TRMM_2(__m, __n) (0.5 * (double)(__n) * (double)(__m) * ((double)(__m)-1.))
 
-#define FMULS_TRMM(__m, __n) ( /*( (__side) == PlasmaLeft ) ? FMULS_TRMM_2((__m), (__n)) :*/ FMULS_TRMM_2((__n), (__m)) )
-#define FADDS_TRMM(__m, __n) ( /*( (__side) == PlasmaLeft ) ? FADDS_TRMM_2((__m), (__n)) :*/ FADDS_TRMM_2((__n), (__m)) )
+#define FMULS_TRMM(__m, __n) (/*((__side) == PlasmaLeft) ? FMULS_TRMM_2((__m), (__n)) :*/ FMULS_TRMM_2((__n), (__m)))
+#define FADDS_TRMM(__m, __n) (/*((__side) == PlasmaLeft) ? FADDS_TRMM_2((__m), (__n)) :*/ FADDS_TRMM_2((__n), (__m)))
 
 #define FMULS_TRSM FMULS_TRMM
 #define FADDS_TRSM FMULS_TRMM
 
-#define FLOPS_STRSM(__m, __n) (     FMULS_TRSM((__m), (__n)) +       FADDS_TRSM((__m), (__n)) )
+#define FLOPS_STRSM(__m, __n) (FMULS_TRSM((__m), (__n)) + FADDS_TRSM((__m), (__n)))
+
+
+#define FMULS_SYRK(__k, __n) (0.5 * (double)(__k) * (double)(__n) * ((double)(__n)+1.))
+#define FADDS_SYRK(__k, __n) (0.5 * (double)(__k) * (double)(__n) * ((double)(__n)+1.))
+
+#define FLOPS_SSYRK(__k, __n) (FMULS_SYRK((__k), (__n)) + FADDS_SYRK((__k), (__n)))
 
 
 #define FMULS_GEMM(__m, __n, __k) ((double)(__m) * (double)(__n) * (double)(__k))
 #define FADDS_GEMM(__m, __n, __k) ((double)(__m) * (double)(__n) * (double)(__k))
 
-#define FLOPS_SGEMM(__m, __n, __k) (     FMULS_GEMM((__m), (__n), (__k)) +       FADDS_GEMM((__m), (__n), (__k)) )
+#define FLOPS_SGEMM(__m, __n, __k) (FMULS_GEMM((__m), (__n), (__k)) + FADDS_GEMM((__m), (__n), (__k)))
 
 /* End of magma code */
 
@@ -160,36 +166,49 @@ static unsigned with_noctxs_p = 0;
 static unsigned chole1_p = 0;
 static unsigned chole2_p = 0;
 
-extern struct starpu_perfmodel chol_model_11;
-extern struct starpu_perfmodel chol_model_21;
-extern struct starpu_perfmodel chol_model_22;
+extern struct starpu_perfmodel chol_model_potrf;
+extern struct starpu_perfmodel chol_model_trsm;
+extern struct starpu_perfmodel chol_model_syrk;
+extern struct starpu_perfmodel chol_model_gemm;
 
-extern struct starpu_codelet cl11;
-extern struct starpu_codelet cl21;
-extern struct starpu_codelet cl22;
-extern struct starpu_codelet cl11_gpu;
-extern struct starpu_codelet cl21_gpu;
-extern struct starpu_codelet cl22_gpu;
-extern struct starpu_codelet cl11_cpu;
-extern struct starpu_codelet cl21_cpu;
-extern struct starpu_codelet cl22_cpu;
+extern struct starpu_codelet cl_potrf;
+extern struct starpu_codelet cl_trsm;
+extern struct starpu_codelet cl_syrk;
+extern struct starpu_codelet cl_gemm;
+extern struct starpu_codelet cl_potrf_gpu;
+extern struct starpu_codelet cl_trsm_gpu;
+extern struct starpu_codelet cl_syrk_gpu;
+extern struct starpu_codelet cl_gemm_gpu;
+extern struct starpu_codelet cl_potrf_cpu;
+extern struct starpu_codelet cl_trsm_cpu;
+extern struct starpu_codelet cl_syrk_cpu;
+extern struct starpu_codelet cl_gemm_cpu;
 
-void chol_cpu_codelet_update_u11(void **, void *);
-void chol_cpu_codelet_update_u21(void **, void *);
-void chol_cpu_codelet_update_u22(void **, void *);
+extern starpu_data_handle_t scratch;
 
-double cpu_chol_task_11_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
-double cpu_chol_task_21_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
-double cpu_chol_task_22_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+void cholesky_kernel_init(int nb);
+void cholesky_kernel_fini(void);
+
+void chol_cpu_codelet_update_potrf(void **, void *);
+void chol_cpu_codelet_update_trsm(void **, void *);
+void chol_cpu_codelet_update_syrk(void **, void *);
+void chol_cpu_codelet_update_gemm(void **, void *);
+
+double cpu_chol_task_potrf_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cpu_chol_task_trsm_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cpu_chol_task_syrk_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cpu_chol_task_gemm_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
 
 #ifdef STARPU_USE_CUDA
-void chol_cublas_codelet_update_u11(void *descr[], void *_args);
-void chol_cublas_codelet_update_u21(void *descr[], void *_args);
-void chol_cublas_codelet_update_u22(void *descr[], void *_args);
+void chol_cublas_codelet_update_potrf(void *descr[], void *_args);
+void chol_cublas_codelet_update_trsm(void *descr[], void *_args);
+void chol_cublas_codelet_update_syrk(void *descr[], void *_args);
+void chol_cublas_codelet_update_gemm(void *descr[], void *_args);
 
-double cuda_chol_task_11_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
-double cuda_chol_task_21_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
-double cuda_chol_task_22_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cuda_chol_task_potrf_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cuda_chol_task_trsm_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cuda_chol_task_syrk_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cuda_chol_task_gemm_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
 #endif
 
 void initialize_chol_model(struct starpu_perfmodel* model, char* symbol,
@@ -246,7 +265,8 @@ static void parse_args(int argc, char **argv)
 		}
 		else if (strcmp(argv[i], "-commute") == 0)
 		{
-			cl22.modes[2] |= STARPU_COMMUTE;
+			cl_syrk.modes[1] |= STARPU_COMMUTE;
+			cl_gemm.modes[2] |= STARPU_COMMUTE;
 		}
 		else if (strcmp(argv[i], "-bound") == 0)
 		{
