@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,33 +21,33 @@
 #include "blas.h"
 
 /*
-    This files contains BLAS wrappers for the different BLAS implementations
-  (eg. REFBLAS, ATLAS, GOTOBLAS ...). We assume a Fortran orientation as most
-  libraries do not supply C-based ordering.
+ * This files contains BLAS wrappers for the different BLAS implementations
+ * (eg. REFBLAS, ATLAS, GOTOBLAS ...). We assume a Fortran orientation as most
+ * libraries do not supply C-based ordering.
  */
 
 #ifdef STARPU_ATLAS
 
-inline void STARPU_SGEMM(char *transa, char *transb, int M, int N, int K, 
-			float alpha, const float *A, int lda, const float *B, int ldb, 
-			float beta, float *C, int ldc)
+inline void STARPU_SGEMM(char *transa, char *transb, int M, int N, int K,
+			 float alpha, const float *A, int lda, const float *B, int ldb,
+			 float beta, float *C, int ldc)
 {
 	enum CBLAS_TRANSPOSE ta = (toupper(transa[0]) == 'N')?CblasNoTrans:CblasTrans;
 	enum CBLAS_TRANSPOSE tb = (toupper(transb[0]) == 'N')?CblasNoTrans:CblasTrans;
 
 	cblas_sgemm(CblasColMajor, ta, tb,
-			M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);				
+		    M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);
 }
 
-inline void STARPU_DGEMM(char *transa, char *transb, int M, int N, int K, 
-			double alpha, double *A, int lda, double *B, int ldb, 
-			double beta, double *C, int ldc)
+inline void STARPU_DGEMM(char *transa, char *transb, int M, int N, int K,
+			 double alpha, double *A, int lda, double *B, int ldb,
+			 double beta, double *C, int ldc)
 {
 	enum CBLAS_TRANSPOSE ta = (toupper(transa[0]) == 'N')?CblasNoTrans:CblasTrans;
 	enum CBLAS_TRANSPOSE tb = (toupper(transb[0]) == 'N')?CblasNoTrans:CblasTrans;
 
 	cblas_dgemm(CblasColMajor, ta, tb,
-			M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);				
+		    M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);
 }
 
 inline void STARPU_SGEMV(char *transa, int M, int N, float alpha, float *A, int lda, float *X, int incX, float beta, float *Y, int incY)
@@ -55,7 +55,7 @@ inline void STARPU_SGEMV(char *transa, int M, int N, float alpha, float *A, int
 	enum CBLAS_TRANSPOSE ta = (toupper(transa[0]) == 'N')?CblasNoTrans:CblasTrans;
 
 	cblas_sgemv(CblasColMajor, ta, M, N, alpha, A, lda,
-					X, incX, beta, Y, incY);
+		    X, incX, beta, Y, incY);
 }
 
 inline void STARPU_DGEMV(char *transa, int M, int N, double alpha, double *A, int lda, double *X, int incX, double beta, double *Y, int incY)
@@ -63,7 +63,7 @@ inline void STARPU_DGEMV(char *transa, int M, int N, double alpha, double *A, in
 	enum CBLAS_TRANSPOSE ta = (toupper(transa[0]) == 'N')?CblasNoTrans:CblasTrans;
 
 	cblas_dgemv(CblasColMajor, ta, M, N, alpha, A, lda,
-					X, incX, beta, Y, incY);
+		    X, incX, beta, Y, incY);
 }
 
 inline float STARPU_SASUM(int N, float *X, int incX)
@@ -87,9 +87,9 @@ void STARPU_DSCAL(int N, double alpha, double *X, int incX)
 }
 
 void STARPU_STRSM (const char *side, const char *uplo, const char *transa,
-                   const char *diag, const int m, const int n,
-                   const float alpha, const float *A, const int lda,
-                   float *B, const int ldb)
+		   const char *diag, const int m, const int n,
+		   const float alpha, const float *A, const int lda,
+		   float *B, const int ldb)
 {
 	enum CBLAS_SIDE side_ = (toupper(side[0]) == 'L')?CblasLeft:CblasRight;
 	enum CBLAS_UPLO uplo_ = (toupper(uplo[0]) == 'U')?CblasUpper:CblasLower;
@@ -100,9 +100,9 @@ void STARPU_STRSM (const char *side, const char *uplo, const char *transa,
 }
 
 void STARPU_DTRSM (const char *side, const char *uplo, const char *transa,
-                   const char *diag, const int m, const int n,
-                   const double alpha, const double *A, const int lda,
-                   double *B, const int ldb)
+		   const char *diag, const int m, const int n,
+		   const double alpha, const double *A, const int lda,
+		   double *B, const int ldb)
 {
 	enum CBLAS_SIDE side_ = (toupper(side[0]) == 'L')?CblasLeft:CblasRight;
 	enum CBLAS_UPLO uplo_ = (toupper(uplo[0]) == 'U')?CblasUpper:CblasLower;
@@ -113,41 +113,41 @@ void STARPU_DTRSM (const char *side, const char *uplo, const char *transa,
 }
 
 void STARPU_SSYR (const char *uplo, const int n, const float alpha,
-                  const float *x, const int incx, float *A, const int lda)
+		  const float *x, const int incx, float *A, const int lda)
 {
 	enum CBLAS_UPLO uplo_ = (toupper(uplo[0]) == 'U')?CblasUpper:CblasLower;
 
-	cblas_ssyr(CblasColMajor, uplo_, n, alpha, x, incx, A, lda); 
+	cblas_ssyr(CblasColMajor, uplo_, n, alpha, x, incx, A, lda);
 }
 
 void STARPU_SSYRK (const char *uplo, const char *trans, const int n,
-                   const int k, const float alpha, const float *A,
-                   const int lda, const float beta, float *C,
-                   const int ldc)
+		   const int k, const float alpha, const float *A,
+		   const int lda, const float beta, float *C,
+		   const int ldc)
 {
 	enum CBLAS_UPLO uplo_ = (toupper(uplo[0]) == 'U')?CblasUpper:CblasLower;
 	enum CBLAS_TRANSPOSE trans_ = (toupper(trans[0]) == 'N')?CblasNoTrans:CblasTrans;
-	
-	cblas_ssyrk(CblasColMajor, uplo_, trans_, n, k, alpha, A, lda, beta, C, ldc); 
+
+	cblas_ssyrk(CblasColMajor, uplo_, trans_, n, k, alpha, A, lda, beta, C, ldc);
 }
 
 void STARPU_SGER(const int m, const int n, const float alpha,
-                  const float *x, const int incx, const float *y,
-                  const int incy, float *A, const int lda)
+		  const float *x, const int incx, const float *y,
+		  const int incy, float *A, const int lda)
 {
 	cblas_sger(CblasColMajor, m, n, alpha, x, incx, y, incy, A, lda);
 }
 
 void STARPU_DGER(const int m, const int n, const double alpha,
-                  const double *x, const int incx, const double *y,
-                  const int incy, double *A, const int lda)
+		  const double *x, const int incx, const double *y,
+		  const int incy, double *A, const int lda)
 {
 	cblas_dger(CblasColMajor, m, n, alpha, x, incx, y, incy, A, lda);
 }
 
-void STARPU_STRSV (const char *uplo, const char *trans, const char *diag, 
-                   const int n, const float *A, const int lda, float *x, 
-                   const int incx)
+void STARPU_STRSV (const char *uplo, const char *trans, const char *diag,
+		   const int n, const float *A, const int lda, float *x,
+		   const int incx)
 {
 	enum CBLAS_UPLO uplo_ = (toupper(uplo[0]) == 'U')?CblasUpper:CblasLower;
 	enum CBLAS_TRANSPOSE trans_ = (toupper(trans[0]) == 'N')?CblasNoTrans:CblasTrans;
@@ -157,9 +157,9 @@ void STARPU_STRSV (const char *uplo, const char *trans, const char *diag,
 }
 
 void STARPU_STRMM(const char *side, const char *uplo, const char *transA,
-                 const char *diag, const int m, const int n,
-                 const float alpha, const float *A, const int lda,
-                 float *B, const int ldb)
+		 const char *diag, const int m, const int n,
+		 const float alpha, const float *A, const int lda,
+		 float *B, const int ldb)
 {
 	enum CBLAS_SIDE side_ = (toupper(side[0]) == 'L')?CblasLeft:CblasRight;
 	enum CBLAS_UPLO uplo_ = (toupper(uplo[0]) == 'U')?CblasUpper:CblasLower;
@@ -170,9 +170,9 @@ void STARPU_STRMM(const char *side, const char *uplo, const char *transA,
 }
 
 void STARPU_DTRMM(const char *side, const char *uplo, const char *transA,
-                 const char *diag, const int m, const int n,
-                 const double alpha, const double *A, const int lda,
-                 double *B, const int ldb)
+		 const char *diag, const int m, const int n,
+		 const double alpha, const double *A, const int lda,
+		 double *B, const int ldb)
 {
 	enum CBLAS_SIDE side_ = (toupper(side[0]) == 'L')?CblasLeft:CblasRight;
 	enum CBLAS_UPLO uplo_ = (toupper(uplo[0]) == 'U')?CblasUpper:CblasLower;
@@ -183,8 +183,8 @@ void STARPU_DTRMM(const char *side, const char *uplo, const char *transA,
 }
 
 void STARPU_STRMV(const char *uplo, const char *transA, const char *diag,
-                 const int n, const float *A, const int lda, float *X,
-                 const int incX)
+		 const int n, const float *A, const int lda, float *X,
+		 const int incX)
 {
 	enum CBLAS_UPLO uplo_ = (toupper(uplo[0]) == 'U')?CblasUpper:CblasLower;
 	enum CBLAS_TRANSPOSE transA_ = (toupper(transA[0]) == 'N')?CblasNoTrans:CblasTrans;
@@ -205,16 +205,16 @@ void STARPU_DAXPY(const int n, const double alpha, double *X, const int incX, do
 
 int STARPU_ISAMAX (const int n, float *X, const int incX)
 {
-    int retVal;
-    retVal = cblas_isamax(n, X, incX);
-    return retVal;
+	int retVal;
+	retVal = cblas_isamax(n, X, incX);
+	return retVal;
 }
 
 int STARPU_IDAMAX (const int n, double *X, const int incX)
 {
-    int retVal;
-    retVal = cblas_idamax(n, X, incX);
-    return retVal;
+	int retVal;
+	retVal = cblas_idamax(n, X, incX);
+	return retVal;
 }
 
 float STARPU_SDOT(const int n, const float *x, const int incx, const float *y, const int incy)
@@ -239,22 +239,22 @@ void STARPU_DSWAP(const int n, double *x, const int incx, double *y, const int i
 
 #elif defined(STARPU_GOTO) || defined(STARPU_OPENBLAS) || defined(STARPU_SYSTEM_BLAS) || defined(STARPU_MKL) || defined(STARPU_ARMPL)
 
-inline void STARPU_SGEMM(char *transa, char *transb, int M, int N, int K, 
-			float alpha, const float *A, int lda, const float *B, int ldb, 
+inline void STARPU_SGEMM(char *transa, char *transb, int M, int N, int K,
+			float alpha, const float *A, int lda, const float *B, int ldb,
 			float beta, float *C, int ldc)
 {
 	sgemm_(transa, transb, &M, &N, &K, &alpha,
 			 A, &lda, B, &ldb,
-			 &beta, C, &ldc);	
+			 &beta, C, &ldc);
 }
 
-inline void STARPU_DGEMM(char *transa, char *transb, int M, int N, int K, 
-			double alpha, double *A, int lda, double *B, int ldb, 
+inline void STARPU_DGEMM(char *transa, char *transb, int M, int N, int K,
+			double alpha, double *A, int lda, double *B, int ldb,
 			double beta, double *C, int ldc)
 {
 	dgemm_(transa, transb, &M, &N, &K, &alpha,
 			 A, &lda, B, &ldb,
-			 &beta, C, &ldc);	
+			 &beta, C, &ldc);
 }
 
 
@@ -291,75 +291,75 @@ void STARPU_DSCAL(int N, double alpha, double *X, int incX)
 }
 
 void STARPU_STRSM (const char *side, const char *uplo, const char *transa,
-                   const char *diag, const int m, const int n,
-                   const float alpha, const float *A, const int lda,
-                   float *B, const int ldb)
+		   const char *diag, const int m, const int n,
+		   const float alpha, const float *A, const int lda,
+		   float *B, const int ldb)
 {
 	strsm_(side, uplo, transa, diag, &m, &n, &alpha, A, &lda, B, &ldb);
 }
 
 void STARPU_DTRSM (const char *side, const char *uplo, const char *transa,
-                   const char *diag, const int m, const int n,
-                   const double alpha, const double *A, const int lda,
-                   double *B, const int ldb)
+		   const char *diag, const int m, const int n,
+		   const double alpha, const double *A, const int lda,
+		   double *B, const int ldb)
 {
 	dtrsm_(side, uplo, transa, diag, &m, &n, &alpha, A, &lda, B, &ldb);
 }
 
 void STARPU_SSYR (const char *uplo, const int n, const float alpha,
-                  const float *x, const int incx, float *A, const int lda)
+		  const float *x, const int incx, float *A, const int lda)
 {
-	ssyr_(uplo, &n, &alpha, x, &incx, A, &lda); 
+	ssyr_(uplo, &n, &alpha, x, &incx, A, &lda);
 }
 
 void STARPU_SSYRK (const char *uplo, const char *trans, const int n,
-                   const int k, const float alpha, const float *A,
-                   const int lda, const float beta, float *C,
-                   const int ldc)
+		   const int k, const float alpha, const float *A,
+		   const int lda, const float beta, float *C,
+		   const int ldc)
 {
-	ssyrk_(uplo, trans, &n, &k, &alpha, A, &lda, &beta, C, &ldc); 
+	ssyrk_(uplo, trans, &n, &k, &alpha, A, &lda, &beta, C, &ldc);
 }
 
 void STARPU_SGER(const int m, const int n, const float alpha,
-                  const float *x, const int incx, const float *y,
-                  const int incy, float *A, const int lda)
+		  const float *x, const int incx, const float *y,
+		  const int incy, float *A, const int lda)
 {
 	sger_(&m, &n, &alpha, x, &incx, y, &incy, A, &lda);
 }
 
 void STARPU_DGER(const int m, const int n, const double alpha,
-                  const double *x, const int incx, const double *y,
-                  const int incy, double *A, const int lda)
+		  const double *x, const int incx, const double *y,
+		  const int incy, double *A, const int lda)
 {
 	dger_(&m, &n, &alpha, x, &incx, y, &incy, A, &lda);
 }
 
-void STARPU_STRSV (const char *uplo, const char *trans, const char *diag, 
-                   const int n, const float *A, const int lda, float *x, 
-                   const int incx)
+void STARPU_STRSV (const char *uplo, const char *trans, const char *diag,
+		   const int n, const float *A, const int lda, float *x,
+		   const int incx)
 {
 	strsv_(uplo, trans, diag, &n, A, &lda, x, &incx);
 }
 
 void STARPU_STRMM(const char *side, const char *uplo, const char *transA,
-                 const char *diag, const int m, const int n,
-                 const float alpha, const float *A, const int lda,
-                 float *B, const int ldb)
+		 const char *diag, const int m, const int n,
+		 const float alpha, const float *A, const int lda,
+		 float *B, const int ldb)
 {
 	strmm_(side, uplo, transA, diag, &m, &n, &alpha, A, &lda, B, &ldb);
 }
 
 void STARPU_DTRMM(const char *side, const char *uplo, const char *transA,
-                 const char *diag, const int m, const int n,
-                 const double alpha, const double *A, const int lda,
-                 double *B, const int ldb)
+		 const char *diag, const int m, const int n,
+		 const double alpha, const double *A, const int lda,
+		 double *B, const int ldb)
 {
 	dtrmm_(side, uplo, transA, diag, &m, &n, &alpha, A, &lda, B, &ldb);
 }
 
 void STARPU_STRMV(const char *uplo, const char *transA, const char *diag,
-                 const int n, const float *A, const int lda, float *X,
-                 const int incX)
+		 const int n, const float *A, const int lda, float *X,
+		 const int incX)
 {
 	strmv_(uplo, transA, diag, &n, A, &lda, X, &incX);
 }
@@ -376,16 +376,16 @@ void STARPU_DAXPY(const int n, const double alpha, double *X, const int incX, do
 
 int STARPU_ISAMAX (const int n, float *X, const int incX)
 {
-    int retVal;
-    retVal = isamax_ (&n, X, &incX);
-    return retVal;
+	int retVal;
+	retVal = isamax_ (&n, X, &incX);
+	return retVal;
 }
 
 int STARPU_IDAMAX (const int n, double *X, const int incX)
 {
-    int retVal;
-    retVal = idamax_ (&n, X, &incX);
-    return retVal;
+	int retVal;
+	retVal = idamax_ (&n, X, &incX);
+	return retVal;
 }
 
 float STARPU_SDOT(const int n, const float *x, const int incx, const float *y, const int incy)
@@ -428,12 +428,12 @@ void STARPU_DPOTRF(const char*uplo, const int n, double *a, const int lda)
 #endif
 
 #elif defined(STARPU_SIMGRID)
-inline void STARPU_SGEMM(char *transa, char *transb, int M, int N, int K, 
-			float alpha, const float *A, int lda, const float *B, int ldb, 
+inline void STARPU_SGEMM(char *transa, char *transb, int M, int N, int K,
+			float alpha, const float *A, int lda, const float *B, int ldb,
 			float beta, float *C, int ldc) { }
 
-inline void STARPU_DGEMM(char *transa, char *transb, int M, int N, int K, 
-			double alpha, double *A, int lda, double *B, int ldb, 
+inline void STARPU_DGEMM(char *transa, char *transb, int M, int N, int K,
+			double alpha, double *A, int lda, double *B, int ldb,
 			double beta, double *C, int ldc) { }
 
 inline void STARPU_SGEMV(char *transa, int M, int N, float alpha, float *A, int lda,
@@ -451,48 +451,48 @@ void STARPU_SSCAL(int N, float alpha, float *X, int incX) { }
 void STARPU_DSCAL(int N, double alpha, double *X, int incX) { }
 
 void STARPU_STRSM (const char *side, const char *uplo, const char *transa,
-                   const char *diag, const int m, const int n,
-                   const float alpha, const float *A, const int lda,
-                   float *B, const int ldb) { }
+		   const char *diag, const int m, const int n,
+		   const float alpha, const float *A, const int lda,
+		   float *B, const int ldb) { }
 
 void STARPU_DTRSM (const char *side, const char *uplo, const char *transa,
-                   const char *diag, const int m, const int n,
-                   const double alpha, const double *A, const int lda,
-                   double *B, const int ldb) { }
+		   const char *diag, const int m, const int n,
+		   const double alpha, const double *A, const int lda,
+		   double *B, const int ldb) { }
 
 void STARPU_SSYR (const char *uplo, const int n, const float alpha,
-                  const float *x, const int incx, float *A, const int lda) { }
+		  const float *x, const int incx, float *A, const int lda) { }
 
 void STARPU_SSYRK (const char *uplo, const char *trans, const int n,
-                   const int k, const float alpha, const float *A,
-                   const int lda, const float beta, float *C,
-                   const int ldc) { }
+		   const int k, const float alpha, const float *A,
+		   const int lda, const float beta, float *C,
+		   const int ldc) { }
 
 void STARPU_SGER(const int m, const int n, const float alpha,
-                  const float *x, const int incx, const float *y,
-                  const int incy, float *A, const int lda) { }
+		  const float *x, const int incx, const float *y,
+		  const int incy, float *A, const int lda) { }
 
 void STARPU_DGER(const int m, const int n, const double alpha,
-                  const double *x, const int incx, const double *y,
-                  const int incy, double *A, const int lda) { }
+		  const double *x, const int incx, const double *y,
+		  const int incy, double *A, const int lda) { }
 
-void STARPU_STRSV (const char *uplo, const char *trans, const char *diag, 
-                   const int n, const float *A, const int lda, float *x, 
-                   const int incx) { }
+void STARPU_STRSV (const char *uplo, const char *trans, const char *diag,
+		   const int n, const float *A, const int lda, float *x,
+		   const int incx) { }
 
 void STARPU_STRMM(const char *side, const char *uplo, const char *transA,
-                 const char *diag, const int m, const int n,
-                 const float alpha, const float *A, const int lda,
-                 float *B, const int ldb) { }
+		 const char *diag, const int m, const int n,
+		 const float alpha, const float *A, const int lda,
+		 float *B, const int ldb) { }
 
 void STARPU_DTRMM(const char *side, const char *uplo, const char *transA,
-                 const char *diag, const int m, const int n,
-                 const double alpha, const double *A, const int lda,
-                 double *B, const int ldb) { }
+		 const char *diag, const int m, const int n,
+		 const double alpha, const double *A, const int lda,
+		 double *B, const int ldb) { }
 
 void STARPU_STRMV(const char *uplo, const char *transA, const char *diag,
-                 const int n, const float *A, const int lda, float *X,
-                 const int incX) { }
+		 const int n, const float *A, const int lda, float *X,
+		 const int incX) { }
 
 void STARPU_SAXPY(const int n, const float alpha, float *X, const int incX, float *Y, const int incY) { }
 
