@@ -78,7 +78,7 @@ void callback_func(void *foo)
 	float i = 1.;
 	while (i > 0)
 	{
-		usleep(100000);
+		starpu_usleep(100000);
 		display(i);
 		i -= 0.1;
 	}
@@ -122,9 +122,8 @@ int main(int argc, char **argv)
 
 	/* Enable OpenGL interoperability */
 	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
 	conf.ncuda = 1;
-	conf.ncpus = 0;
-	conf.nopencl = 0;
 	conf.cuda_opengl_interoperability = cuda_devices;
 	conf.n_cuda_opengl_interoperability = sizeof(cuda_devices) / sizeof(*cuda_devices);
 	conf.not_launched_drivers = drivers;
