@@ -1,7 +1,6 @@
 ! StarPU --- Runtime system for heterogeneous multicore architectures.
 !
-! Copyright (C) 2017                                     CNRS
-! Copyright (C) 2016                                     Inria
+! Copyright (C) 2016-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 !
 ! StarPU is free software; you can redistribute it and/or modify
 ! it under the terms of the GNU Lesser General Public License as published by
@@ -94,7 +93,7 @@ program nf_partition
                 !     . . .
                 !     C_NULL_PTR
                 !   )/
-                call fstarpu_insert_task((/ cl_partition, FSTARPU_RW, dh_sub, C_NULL_PTR /))
+                call fstarpu_task_insert((/ cl_partition, FSTARPU_RW, dh_sub, C_NULL_PTR /))
         end do
         end do
 
