@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015  CNRS
- * Copyright (C) 2010-2013, 2015  Université de Bordeaux
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,9 +21,9 @@
  */
 
 /* gcc build:
-
-   gcc -fopenmp -O2 -g vector_scal.c -o vector_scal $(pkg-config --cflags starpu-1.0) $(pkg-config --libs starpu-1.0)
-
+ *
+ *  gcc -fopenmp -O2 -g vector_scal.c -o vector_scal $(pkg-config --cflags starpu-1.0) $(pkg-config --libs starpu-1.0)
+ *
  */
 
 #include <starpu.h>
@@ -77,21 +76,13 @@ static struct starpu_codelet cl =
 	.model = &vector_scal_model,
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
 	struct starpu_conf conf;
 	float *vector;
 	unsigned i;
 	int ret;
 
-	vector = malloc(NX*sizeof(*vector));
-
-	for (i = 0; i < NX; i++)
-                vector[i] = (i+1.0f);
-
-	FPRINTF(stderr, "BEFORE: First element was %f\n", vector[0]);
-	FPRINTF(stderr, "BEFORE: Last element was %f\n", vector[NX-1]);
-
 	starpu_conf_init(&conf);
 
 	/* Most OpenMP implementations do not support concurrent parallel
@@ -103,6 +94,14 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
+	starpu_malloc((void **)&vector, NX*sizeof(float));
+
+	for (i = 0; i < NX; i++)
+		vector[i] = (i+1.0f);
+
+	FPRINTF(stderr, "BEFORE: First element was %f\n", vector[0]);
+	FPRINTF(stderr, "BEFORE: Last element was %f\n", vector[NX-1]);
+
 	starpu_data_handle_t vector_handle;
 	starpu_vector_data_register(&vector_handle, STARPU_MAIN_RAM, (uintptr_t)vector, NX, sizeof(vector[0]));
 
@@ -125,18 +124,19 @@ int main(int argc, char **argv)
 
 	starpu_data_unregister(vector_handle);
 
-	/* terminate StarPU, no task can be submitted after */
-	starpu_shutdown();
-
 	FPRINTF(stderr, "AFTER: First element is %f\n", vector[0]);
 	FPRINTF(stderr, "AFTER: Last element is %f\n", vector[NX-1]);
 
-	free(vector);
+	starpu_free_noflag(vector, NX*sizeof(float));
+
+	/* terminate StarPU, no task can be submitted after */
+	starpu_shutdown();
+
 	return 0;
 
 enodev:
 	starpu_data_unregister(vector_handle);
-	free(vector);
+	starpu_free_noflag(vector, NX*sizeof(float));
 	starpu_shutdown();
 	return 77;
 }
