@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,9 +21,9 @@
  */
 
 /* gcc build:
-
-   gcc -O2 -g vector_scal.c -o vector_scal $(pkg-config --cflags starpu-1.0) $(pkg-config --libs starpu-1.0)
-
+ *
+ *  gcc -O2 -g vector_scal.c -o vector_scal $(pkg-config --cflags starpu-1.0) $(pkg-config --libs starpu-1.0)
+ *
  */
 
 #include <starpu.h>
@@ -66,7 +66,7 @@ void scal_cpu_func(void *buffers[], void *_args)
 	if (rank == 0)
 		FPRINTF(stderr, "running task with %d CPUs.\n", starpu_combined_worker_get_size());
 
-	/* We add 1 to the (nel_total % nb_workers) first workers, thus we get an evenly splitted data. */
+	/* We add 1 to the (nel_total % nb_workers) first workers, thus we get an evenly split data. */
 	int nel_worker = (n / nb_workers) + ((rank < (n % nb_workers)) ? 1 : 0);
 
 	int begin = get_first_element_rank(n, rank, nb_workers);
@@ -112,11 +112,20 @@ int main(void)
 	conf.single_combined_worker = 1;
 	conf.sched_policy_name = "pheft";
 
+	{
+		ret = starpu_init(NULL);
+		if (ret == -ENODEV) return 77;
+		conf.ncpus = starpu_cpu_worker_get_count();
+		conf.ncpus /= 2;
+		starpu_shutdown();
+	}
+
+
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	vector = malloc(NX*sizeof(*vector));
+	starpu_malloc((void **)&vector, NX*sizeof(float));
 	for (i = 0; i < NX; i++)
 		vector[i] = (i+1.0f);
 
@@ -149,13 +158,13 @@ int main(void)
 
 	starpu_data_unregister(vector_handle);
 
-	/* terminate StarPU, no task can be submitted after */
-	starpu_shutdown();
-
 	FPRINTF(stderr, "AFTER: First element is %f\n", vector[0]);
 	FPRINTF(stderr, "AFTER: Last element is %f\n", vector[NX-1]);
 
-	free(vector);
+	starpu_free_noflag(vector, NX*sizeof(float));
+
+	/* terminate StarPU, no task can be submitted after */
+	starpu_shutdown();
 
 	return ret;
 }
