@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,24 +17,23 @@
 #ifndef __STARPU_CUBLAS_V2_H__
 #define __STARPU_CUBLAS_V2_H__
 
-#if defined STARPU_USE_CUDA && !defined STARPU_DONT_INCLUDE_CUDA_HEADERS
+#ifdef STARPU_USE_CUDA
 
 #include <cublas_v2.h>
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
    @ingroup API_CUDA_Extensions
    @{
- */
+*/
 
 /**
-   Return the CUSPARSE handle to be used to queue CUSPARSE kernels. It
+   Return the CUBLAS handle to be used to queue CUBLAS kernels. It
    is properly initialized and configured for multistream by
-   starpu_cusparse_init().
+   starpu_cublas_init(). See \ref CUDA-specificOptimizations for more details.
 */
 cublasHandle_t starpu_cublas_get_local_handle(void);
 
