@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2013       Thibaut Lambert
  *
@@ -23,8 +23,7 @@
 #include <stdio.h>
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -35,16 +34,14 @@ extern "C"
 struct starpu_task;
 struct starpu_data_descr;
 
-#define STARPU_NARCH STARPU_ANY_WORKER
-
 /**
    todo
 */
 struct starpu_perfmodel_device
 {
 	enum starpu_worker_archtype type; /**< type of the device */
-	int devid;                        /**< identifier of the precise device */
-	int ncores;                       /**< number of execution in parallel, minus 1 */
+	int devid;			  /**< identifier of the precise device */
+	int ncores;			  /**< number of execution in parallel, minus 1 */
 };
 
 /**
@@ -52,28 +49,33 @@ struct starpu_perfmodel_device
 */
 struct starpu_perfmodel_arch
 {
-	int ndevices;                            /**< number of the devices for the given arch */
+	int ndevices;				 /**< number of the devices for the given arch */
 	struct starpu_perfmodel_device *devices; /**< list of the devices for the given arch */
 };
 
-
+/**
+   todo
+*/
 struct starpu_perfmodel_history_entry
 {
-	double mean;        /**< mean_n = 1/n sum */
-	double deviation;   /**< n dev_n = sum2 - 1/n (sum)^2 */
-	double sum;         /**< sum of samples (in µs) */
-	double sum2;        /**< sum of samples^2 */
-	unsigned nsample;   /**< number of samples */
+	double mean;	  /**< mean_n = 1/n sum */
+	double deviation; /**< n dev_n = sum2 - 1/n (sum)^2 */
+	double sum;	  /**< sum of samples (in µs) */
+	double sum2;	  /**< sum of samples^2 */
+	unsigned nsample; /**< number of samples */
 	unsigned nerror;
 	uint32_t footprint; /**< data footprint */
-	size_t size;        /**< in bytes */
-	double flops;       /**< Provided by the application */
+	size_t size;	    /**< in bytes */
+	double flops;	    /**< Provided by the application */
 
 	double duration;
 	starpu_tag_t tag;
 	double *parameters;
 };
 
+/**
+   todo
+*/
 struct starpu_perfmodel_history_list
 {
 	struct starpu_perfmodel_history_list *next;
@@ -85,38 +87,38 @@ struct starpu_perfmodel_history_list
 */
 struct starpu_perfmodel_regression_model
 {
-	double sumlny;          /**< sum of ln(measured) */
+	double sumlny; /**< sum of ln(measured) */
 
-	double sumlnx;          /**< sum of ln(size) */
-	double sumlnx2;         /**< sum of ln(size)^2 */
+	double sumlnx;	/**< sum of ln(size) */
+	double sumlnx2; /**< sum of ln(size)^2 */
 
-	unsigned long minx;     /**< minimum size */
-	unsigned long maxx;     /**< maximum size */
+	unsigned long minx; /**< minimum size */
+	unsigned long maxx; /**< maximum size */
 
-	double sumlnxlny;       /**< sum of ln(size)*ln(measured) */
+	double sumlnxlny; /**< sum of ln(size)*ln(measured) */
 
-	double alpha;           /**< estimated = alpha * size ^ beta */
-	double beta;            /**< estimated = alpha * size ^ beta */
-	unsigned valid;         /**< whether the linear regression model is valid (i.e. enough measures) */
+	double alpha;	/**< estimated = alpha * size ^ beta */
+	double beta;	/**< estimated = alpha * size ^ beta */
+	unsigned valid; /**< whether the linear regression model is valid (i.e. enough measures) */
 
-	double a;               /**< estimated = a size ^b + c */
-	double b;               /**< estimated = a size ^b + c */
-	double c;               /**< estimated = a size ^b + c */
-	unsigned nl_valid;      /**< whether the non-linear regression model is valid (i.e. enough measures) */
+	double a;	   /**< estimated = a size ^b + c */
+	double b;	   /**< estimated = a size ^b + c */
+	double c;	   /**< estimated = a size ^b + c */
+	unsigned nl_valid; /**< whether the non-linear regression model is valid (i.e. enough measures) */
 
-	unsigned nsample;       /**< number of sample values for non-linear regression */
+	unsigned nsample; /**< number of sample values for non-linear regression */
 
-	double *coeff;          /**< list of computed coefficients for multiple linear regression model */
-	unsigned ncoeff;        /**< number of coefficients for multiple linear regression model */
-	unsigned multi_valid;   /**< whether the multiple linear regression model is valid */
+	double *coeff;	      /**< list of computed coefficients for multiple linear regression model */
+	unsigned ncoeff;      /**< number of coefficients for multiple linear regression model */
+	unsigned multi_valid; /**< whether the multiple linear regression model is valid */
 };
 
 struct starpu_perfmodel_history_table;
 
 #define starpu_per_arch_perfmodel starpu_perfmodel_per_arch STARPU_DEPRECATED
 
-typedef double (*starpu_perfmodel_per_arch_cost_function)(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
-typedef size_t (*starpu_perfmodel_per_arch_size_base)(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+typedef double (*starpu_perfmodel_per_arch_cost_function)(struct starpu_task *task, struct starpu_perfmodel_arch *arch, unsigned nimpl);
+typedef size_t (*starpu_perfmodel_per_arch_size_base)(struct starpu_task *task, struct starpu_perfmodel_arch *arch, unsigned nimpl);
 
 /**
    information about the performance model of a given arch.
@@ -164,18 +166,19 @@ struct starpu_perfmodel_per_arch
 */
 enum starpu_perfmodel_type
 {
-        STARPU_PERFMODEL_INVALID=0,
-	STARPU_PER_ARCH,                  /**< Application-provided per-arch cost model function */
-	STARPU_COMMON,                    /**< Application-provided common cost model function, with per-arch factor */
-	STARPU_HISTORY_BASED,             /**< Automatic history-based cost model */
-	STARPU_REGRESSION_BASED,          /**< Automatic linear regression-based cost model  (alpha * size ^ beta) */
-	STARPU_NL_REGRESSION_BASED,       /**< Automatic non-linear regression-based cost model (a * size ^ b + c) */
-	STARPU_MULTIPLE_REGRESSION_BASED  /**< Automatic multiple linear regression-based cost model. Application
+	STARPU_PERFMODEL_INVALID = 0,
+	STARPU_PER_WORKER,		 /**< Application-provided per-worker cost model function */
+	STARPU_PER_ARCH,		 /**< Application-provided per-arch cost model function */
+	STARPU_COMMON,			 /**< Application-provided common cost model function, with per-arch factor */
+	STARPU_HISTORY_BASED,		 /**< Automatic history-based cost model */
+	STARPU_REGRESSION_BASED,	 /**< Automatic linear regression-based cost model  (alpha * size ^ beta) */
+	STARPU_NL_REGRESSION_BASED,	 /**< Automatic non-linear regression-based cost model (a * size ^ b + c) */
+	STARPU_MULTIPLE_REGRESSION_BASED /**< Automatic multiple linear regression-based cost model. Application
 					     provides parameters, their combinations and exponents. */
 };
 
 struct _starpu_perfmodel_state;
-typedef struct _starpu_perfmodel_state* starpu_perfmodel_state_t;
+typedef struct _starpu_perfmodel_state *starpu_perfmodel_state_t;
 
 /**
    Contain all information about a performance model. At least the
@@ -226,11 +229,17 @@ struct starpu_perfmodel
 	*/
 	double (*cost_function)(struct starpu_task *, unsigned nimpl);
 	/**
-	   Used by ::STARPU_COMMON. Take a task, an arch and implementation
+	   Used by ::STARPU_PER_ARCH. Take a task, an arch and implementation
 	   number, and must return a task duration estimation in
 	   micro-seconds on that arch.
 	*/
-	double (*arch_cost_function)(struct starpu_task *, struct starpu_perfmodel_arch * arch, unsigned nimpl);
+	double (*arch_cost_function)(struct starpu_task *, struct starpu_perfmodel_arch *arch, unsigned nimpl);
+	/**
+	   Used by ::STARPU_PER_WORKER. Take a task, a worker id and implementation
+	   number, and must return a task duration estimation in
+	   micro-seconds on that worker.
+	*/
+	double (*worker_cost_function)(struct starpu_task *, unsigned workerid, unsigned nimpl);
 
 	/**
 	   Used by ::STARPU_HISTORY_BASED, ::STARPU_REGRESSION_BASED and
@@ -254,6 +263,12 @@ struct starpu_perfmodel
 	*/
 	const char *symbol;
 
+	/**
+	   name of the file storing the performance model. It is non
+	   NULL if the model has been loaded or stored in a file.
+	 */
+	char *path;
+
 	/**
 	   \private
 	   Whether the performance model is already loaded from the disk.
@@ -268,7 +283,7 @@ struct starpu_perfmodel
 	*/
 	unsigned is_init;
 
-	void (*parameters)(struct starpu_task * task, double *parameters);
+	void (*parameters)(struct starpu_task *task, double *parameters);
 	/**
 	   \private
 	   Names of parameters used for multiple linear regression models (M,
@@ -304,6 +319,37 @@ struct starpu_perfmodel
 */
 void starpu_perfmodel_init(struct starpu_perfmodel *model);
 
+/**
+   Deinitialize the \p model performance model structure. You need to call this
+   before deallocating the structure. You will probably want to call
+   starpu_perfmodel_unload_model() before calling this function, to save the perfmodel.
+*/
+int starpu_perfmodel_deinit(struct starpu_perfmodel *model);
+
+/**
+   starpu_energy_start - start counting hardware events in an event set
+
+   - \p workerid is the worker on which calibration is to be performed (in the case of GPUs, use -1 for CPUs)
+   - \p archi is the type of architecture on which calibration will be run
+
+   See \ref MeasuringEnergyandPower for more details.
+*/
+int starpu_energy_start(int workerid, enum starpu_worker_archtype archi);
+
+/**
+   starpu_energy_stop - stop counting hardware events in an event set
+
+   - \p model is the energy performance model to be filled with the result
+   - \p task is a task specimen, so the performance model folds the result according to the parameter sizes of the task.
+   - \p nimpl is the implementation number run during calibration
+   - \p ntasks is the number of tasks run during calibration
+   - \p workerid is the worker on which calibration was performed (in the case of GPUs, use -1 for CPUs)
+   - \p archi is the type of architecture on which calibration was run
+
+   See \ref MeasuringEnergyandPower for more details.
+*/
+int starpu_energy_stop(struct starpu_perfmodel *model, struct starpu_task *task, unsigned nimpl, unsigned ntasks, int workerid, enum starpu_worker_archtype archi);
+
 /**
    Load the performance model found in the file named \p filename. \p model has to be
    completely zero, and will be filled with the information stored in the given file.
@@ -325,6 +371,11 @@ int starpu_perfmodel_load_symbol(const char *symbol, struct starpu_perfmodel *mo
 */
 int starpu_perfmodel_unload_model(struct starpu_perfmodel *model);
 
+/**
+	Save the performance model in its file.
+*/
+void starpu_save_history_based_model(struct starpu_perfmodel *model);
+
 /**
   Fills \p path (supposed to be \p maxlen long) with the full path to the
   performance model file for symbol \p symbol.  This path can later on be used
@@ -334,6 +385,7 @@ void starpu_perfmodel_get_model_path(const char *symbol, char *path, size_t maxl
 
 /**
   Dump performance model \p model to output stream \p output, in XML format.
+  See \ref PerformanceModelExample for more details.
 */
 void starpu_perfmodel_dump_xml(FILE *output, struct starpu_perfmodel *model);
 
@@ -351,7 +403,7 @@ void starpu_perfmodel_free_sampling(void);
 struct starpu_perfmodel_arch *starpu_worker_get_perf_archtype(int workerid, unsigned sched_ctx_id);
 
 int starpu_perfmodel_get_narch_combs(void);
-int starpu_perfmodel_arch_comb_add(int ndevices, struct starpu_perfmodel_device* devices);
+int starpu_perfmodel_arch_comb_add(int ndevices, struct starpu_perfmodel_device *devices);
 int starpu_perfmodel_arch_comb_get(int ndevices, struct starpu_perfmodel_device *devices);
 struct starpu_perfmodel_arch *starpu_perfmodel_arch_comb_fetch(int comb);
 
@@ -366,7 +418,7 @@ int starpu_perfmodel_set_per_devices_size_base(struct starpu_perfmodel *model, i
 */
 void starpu_perfmodel_debugfilepath(struct starpu_perfmodel *model, struct starpu_perfmodel_arch *arch, char *path, size_t maxlen, unsigned nimpl);
 
-char* starpu_perfmodel_get_archtype_name(enum starpu_worker_archtype archtype);
+const char *starpu_perfmodel_get_archtype_name(enum starpu_worker_archtype archtype);
 
 /**
    Return the architecture name for \p arch
@@ -374,9 +426,9 @@ char* starpu_perfmodel_get_archtype_name(enum starpu_worker_archtype archtype);
 void starpu_perfmodel_get_arch_name(struct starpu_perfmodel_arch *arch, char *archname, size_t maxlen, unsigned nimpl);
 
 /**
-   Return the estimated time of a task with the given model and the given footprint.
+   Return the estimated time in µs of a task with the given model and the given footprint.
 */
-double starpu_perfmodel_history_based_expected_perf(struct starpu_perfmodel *model, struct starpu_perfmodel_arch* arch, uint32_t footprint);
+double starpu_perfmodel_history_based_expected_perf(struct starpu_perfmodel *model, struct starpu_perfmodel_arch *arch, uint32_t footprint);
 
 /**
    If starpu_init() is not used, starpu_perfmodel_initialize() should be used called calling starpu_perfmodel_* functions.
@@ -395,15 +447,33 @@ int starpu_perfmodel_print_estimations(struct starpu_perfmodel *model, uint32_t
 int starpu_perfmodel_list_combs(FILE *output, struct starpu_perfmodel *model);
 
 /**
-   Feed the performance model model with an explicit
-   measurement measured (in µs), in addition to measurements done by StarPU
+   Feed the performance model \p model with one explicit
+   measurement (in µs or J), in addition to measurements done by StarPU
    itself. This can be useful when the application already has an
    existing set of measurements done in good conditions, that StarPU
    could benefit from instead of doing on-line measurements. An example
    of use can be seen in \ref PerformanceModelExample.
+
+   Note that this records only one measurement, and StarPU would ignore
+   the first measurement (since it is usually disturbed by library loading
+   etc.). Make sure to call this function several times to record all your
+   measurements.
+
+   You can also call starpu_perfmodel_update_history_n() to directly provide an
+   average performed on several tasks.
+
+   See \ref PerformanceModelCalibration for more details.
 */
 void starpu_perfmodel_update_history(struct starpu_perfmodel *model, struct starpu_task *task, struct starpu_perfmodel_arch *arch, unsigned cpuid, unsigned nimpl, double measured);
 
+/**
+   Feed the performance model \p model with an explicit average measurement (in µs or J).
+
+   This is similar to starpu_perfmodel_update_history(), but records a batch of
+   \p number measurements provided as the average of the measurements \p average_measured.
+*/
+void starpu_perfmodel_update_history_n(struct starpu_perfmodel *model, struct starpu_task *task, struct starpu_perfmodel_arch *arch, unsigned cpuid, unsigned nimpl, double average_measured, unsigned number);
+
 /**
    Print the directory name storing performance models on \p output
 */
@@ -425,17 +495,20 @@ void starpu_bus_print_affinity(FILE *f);
 void starpu_bus_print_filenames(FILE *f);
 
 /**
-   Return the bandwidth of data transfer between two memory nodes
+   Return the bandwidth of data transfer between two memory nodes.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_transfer_bandwidth(unsigned src_node, unsigned dst_node);
 
 /**
-   Return the latency of data transfer between two memory nodes
+   Return the latency of data transfer between two memory nodes.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_transfer_latency(unsigned src_node, unsigned dst_node);
 
 /**
    Return the estimated time to transfer a given size between two memory nodes.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_transfer_predict(unsigned src_node, unsigned dst_node, size_t size);
 
