@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2020       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -22,9 +22,14 @@
 #include <errno.h>
 #include <time.h>
 
+#include <starpu_config.h>
+
+#ifdef STARPU_PAPI
+#include <papi.h>
+#endif
+
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -35,16 +40,16 @@ extern "C"
 /**
    Used when calling the function starpu_profiling_status_set() to disable profiling.
 */
-#define STARPU_PROFILING_DISABLE	0
+#define STARPU_PROFILING_DISABLE 0
 /**
    Used when calling the function starpu_profiling_status_set() to enable profiling.
 */
-#define STARPU_PROFILING_ENABLE		1
+#define STARPU_PROFILING_ENABLE 1
 
 /**
    Information about the execution of a task. It is accessible from
    the field starpu_task::profiling_info if profiling was enabled.
- */
+*/
 struct starpu_profiling_task_info
 {
 	/** Date of task submission (relative to the initialization of StarPU). */
@@ -90,12 +95,23 @@ struct starpu_profiling_task_info
 	uint64_t stall_cycles;
 	/** Energy consumed by the task, in Joules */
 	double energy_consumed;
+
+#ifdef STARPU_PAPI
+	/** PAPI Events **/
+	long long int papi_values[PAPI_MAX_HWCTRS];
+	int papi_event_set;
+#endif
 };
 
 /**
    Profiling information associated to a worker. The timing is
    provided since the previous call to
-   starpu_profiling_worker_get_info()
+   starpu_profiling_worker_get_info().
+
+   The executing_time, callback_time, waiting_time, sleeping_time, and
+   scheduling_time are exclusive to each other, i.e. they can be added up, their
+   sum is smaller than total_time. The difference between total_time and the sum
+   is the uncategorized runtime overhead.
 */
 struct starpu_profiling_worker_info
 {
@@ -103,10 +119,41 @@ struct starpu_profiling_worker_info
 	struct timespec start_time;
 	/** Duration of the profiling measurement interval. */
 	struct timespec total_time;
+
 	/** Time spent by the worker to execute tasks during the profiling measurement interval. */
 	struct timespec executing_time;
-	/** Time spent idling by the worker during the profiling measurement interval. */
+	/** Time spent by the worker to execute callbacks, while not executing a
+	 * task, during the profiling measurement interval. */
+	struct timespec callback_time;
+	/** Time spent by the worker waiting for a data transfer to finish,
+	 * while not executing a task or a callback, during the profiling
+	 * measurement interval. */
+	struct timespec waiting_time;
+	/** Time spent idling by the worker because no task were available, and
+	 * not executing a task or a callback or waiting for a data transfer to
+	 * finish, during the profiling measurement interval. */
 	struct timespec sleeping_time;
+	/** Time spent by the worker scheduling tasks, while not executing a
+	 * task or a callback or waiting for a data transfer to finish, and there
+	 * are tasks to be scheduled, during the profiling measurement interval. */
+	struct timespec scheduling_time;
+
+	/** Time spent by the worker to execute tasks during the profiling measurement interval.
+	 * Normally always equal to executing_time. */
+	struct timespec all_executing_time;
+	/** Time spent by the worker to execute callbacks during the profiling measurement interval.
+	 * Normally always greater than callback_time. */
+	struct timespec all_callback_time;
+	/** Time spent by the worker waiting for a data transfer to finish during the profiling measurement interval.
+	 * Normally always greater than waiting_time. */
+	struct timespec all_waiting_time;
+	/** Time spent idling by the worker because no task were available during the profiling measurement interval.
+	 * Normally always greater than sleeping_time. */
+	struct timespec all_sleeping_time;
+	/** Time spent by the worker scheduling tasks during the profiling measurement interval.
+	 * Normally always greater than scheduling_time. */
+	struct timespec all_scheduling_time;
+
 	/** Number of tasks executed by the worker during the profiling measurement interval. */
 	int executed_tasks;
 
@@ -117,9 +164,14 @@ struct starpu_profiling_worker_info
 	/** Energy consumed by the worker, in Joules */
 	double energy_consumed;
 
+	/* TODO: add wasted time due to failed tasks */
+
 	double flops;
 };
 
+/**
+   todo
+*/
 struct starpu_profiling_bus_info
 {
 	/** Time of bus profiling startup. */
@@ -135,11 +187,13 @@ struct starpu_profiling_bus_info
 /**
    Reset performance counters and enable profiling if the
    environment variable \ref STARPU_PROFILING is set to a positive value.
+   See \ref EnablingOn-linePerformanceMonitoring for more details.
 */
 void starpu_profiling_init(void);
 
 /**
    Set the ID used for profiling trace filename. Has to be called before starpu_init().
+   See \ref TraceMpi for more details.
 */
 void starpu_profiling_set_id(int new_id);
 
@@ -152,12 +206,14 @@ void starpu_profiling_set_id(int new_id);
    starpu_profiling_task_info containing information about the execution
    of the task. Negative return values indicate an error, otherwise the
    previous status is returned.
+   See \ref EnablingOn-linePerformanceMonitoring for more details.
 */
 int starpu_profiling_status_set(int status);
 
 /**
    Return the current profiling status or a negative value in case
    there was an error.
+   See \ref EnablingOn-linePerformanceMonitoring for more details.
 */
 int starpu_profiling_status_get(void);
 
@@ -165,13 +221,14 @@ int starpu_profiling_status_get(void);
 #include <common/utils.h>
 #ifdef __GNUC__
 extern int _starpu_profiling;
-#define starpu_profiling_status_get() ({ \
-	int __ret; \
-	ANNOTATE_HAPPENS_AFTER(&_starpu_profiling); \
-	__ret = _starpu_profiling; \
-	ANNOTATE_HAPPENS_BEFORE(&_starpu_profiling); \
-	__ret; \
-})
+#define starpu_profiling_status_get() (                      \
+	{                                                    \
+		int __ret;                                   \
+		ANNOTATE_HAPPENS_AFTER(&_starpu_profiling);  \
+		__ret = _starpu_profiling;                   \
+		ANNOTATE_HAPPENS_BEFORE(&_starpu_profiling); \
+		__ret;                                       \
+	})
 #endif
 #endif
 
@@ -181,36 +238,54 @@ extern int _starpu_profiling;
    worker_info is <c>NULL</c>, only reset the counters associated to worker
    \p workerid. Upon successful completion, this function returns 0.
    Otherwise, a negative value is returned.
+   See \ref Per-workerFeedback for more details.
 */
 int starpu_profiling_worker_get_info(int workerid, struct starpu_profiling_worker_info *worker_info);
 
 /**
-   Return the number of buses in the machine
+   Return the number of buses in the machine.
+   See \ref HardwareTopology for more details.
 */
 int starpu_bus_get_count(void);
 
 /**
-   Return the identifier of the bus between \p src and \p dst
+   Return the identifier of the bus between \p src and \p dst.
+   See \ref HardwareTopology for more details.
 */
 int starpu_bus_get_id(int src, int dst);
 
 /**
-   Return the source point of bus \p busid
+   Return the source point of bus \p busid.
+   See \ref HardwareTopology for more details.
 */
 int starpu_bus_get_src(int busid);
 
 /**
-   Return the destination point of bus \p busid
+   Return the destination point of bus \p busid.
+   See \ref HardwareTopology for more details.
 */
 int starpu_bus_get_dst(int busid);
+/**
+   See \ref HardwareTopology for more details.
+*/
 void starpu_bus_set_direct(int busid, int direct);
+/**
+   See \ref HardwareTopology for more details.
+*/
 int starpu_bus_get_direct(int busid);
+/**
+   See \ref HardwareTopology for more details.
+*/
 void starpu_bus_set_ngpus(int busid, int ngpus);
+/**
+   See \ref HardwareTopology for more details.
+*/
 int starpu_bus_get_ngpus(int busid);
 
 /**
    See _starpu_profiling_bus_helper_display_summary in src/profiling/profiling_helpers.c for a usage example.
    Note that calling starpu_bus_get_profiling_info() resets the counters to zero.
+   See \ref FeedBackFigures for more details.
 */
 int starpu_bus_get_profiling_info(int busid, struct starpu_profiling_bus_info *bus_info);
 
@@ -218,7 +293,7 @@ int starpu_bus_get_profiling_info(int busid, struct starpu_profiling_bus_info *b
 /* Reset timespec */
 static __starpu_inline void starpu_timespec_clear(struct timespec *tsp)
 {
-	tsp->tv_sec = 0;
+	tsp->tv_sec  = 0;
 	tsp->tv_nsec = 0;
 }
 
@@ -229,7 +304,7 @@ static __starpu_inline void starpu_timespec_add(struct timespec *a,
 						struct timespec *b,
 						struct timespec *result)
 {
-	result->tv_sec = a->tv_sec + b->tv_sec;
+	result->tv_sec	= a->tv_sec + b->tv_sec;
 	result->tv_nsec = a->tv_nsec + b->tv_nsec;
 
 	if (result->tv_nsec >= STARPU_NS_PER_S)
@@ -258,7 +333,7 @@ static __starpu_inline void starpu_timespec_sub(const struct timespec *a,
 						const struct timespec *b,
 						struct timespec *result)
 {
-	result->tv_sec = a->tv_sec - b->tv_sec;
+	result->tv_sec	= a->tv_sec - b->tv_sec;
 	result->tv_nsec = a->tv_nsec - b->tv_nsec;
 
 	if ((result)->tv_nsec < 0)
@@ -268,16 +343,18 @@ static __starpu_inline void starpu_timespec_sub(const struct timespec *a,
 	}
 }
 
-#define starpu_timespec_cmp(a, b, CMP)                          \
-	(((a)->tv_sec == (b)->tv_sec) ? ((a)->tv_nsec CMP (b)->tv_nsec) : ((a)->tv_sec CMP (b)->tv_sec))
+#define starpu_timespec_cmp(a, b, CMP) \
+	(((a)->tv_sec == (b)->tv_sec) ? ((a)->tv_nsec CMP(b)->tv_nsec) : ((a)->tv_sec CMP(b)->tv_sec))
 
 /**
    Return the time elapsed between \p start and \p end in microseconds.
+   See \ref Per-taskFeedback for more details.
 */
 double starpu_timing_timespec_delay_us(struct timespec *start, struct timespec *end);
 
 /**
-   Convert the given timespec \p ts into microseconds
+   Convert the given timespec \p ts into microseconds.
+   See \ref Per-taskFeedback for more details.
 */
 double starpu_timing_timespec_to_us(struct timespec *ts);
 
@@ -285,6 +362,7 @@ double starpu_timing_timespec_to_us(struct timespec *ts);
    Display statistics about the bus on \c stderr. if the environment
    variable \ref STARPU_BUS_STATS is defined. The function is called
    automatically by starpu_shutdown().
+   See \ref DataStatistics for more details.
 */
 void starpu_profiling_bus_helper_display_summary(void);
 
@@ -292,6 +370,7 @@ void starpu_profiling_bus_helper_display_summary(void);
    Display statistic about the workers on \c stderr if the
    environment variable \ref STARPU_WORKER_STATS is defined. The function is
    called automatically by starpu_shutdown().
+   See \ref DataStatistics for more details.
 */
 void starpu_profiling_worker_helper_display_summary(void);
 
@@ -300,8 +379,9 @@ void starpu_profiling_worker_helper_display_summary(void);
    within StarPU. StarPU must have been configured with the configure
    option \ref enable-memory-stats "--enable-memory-stats" (see \ref
    MemoryFeedback).
+   See \ref MemoryFeedback for more details.
 */
-void starpu_data_display_memory_stats();
+void starpu_data_display_memory_stats(void);
 
 /** @} */
 
