@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2016       Uppsala University
  *
@@ -33,7 +33,7 @@ extern "C" {
 #endif
 
 /**
-   @defgroup API_Workers_Properties Workers’ Properties
+   @defgroup API_Workers Workers
    @{
 */
 
@@ -152,7 +152,7 @@ struct starpu_worker_collection
 	*/
 	void (*init)(struct starpu_worker_collection *workers);
 	/**
-	   Deinitialize the colection
+	   Deinitialize the collection
 	*/
 	void (*deinit)(struct starpu_worker_collection *workers);
 	/**
@@ -169,7 +169,7 @@ extern struct starpu_worker_collection starpu_worker_tree;
    Wait for all workers to be initialised. Calling this function is
    normally not necessary. It is called for example in
    <c>tools/starpu_machine_display</c> to make sure all workers
-   information are correctly set before printing their informations.
+   information are correctly set before printing their information.
    See \ref PauseResume for more details.
 */
 void starpu_worker_wait_for_initialisation(void);
@@ -701,6 +701,10 @@ int starpu_combined_worker_assign_workerid(int nworkers, int workerid_array[]);
 /**
    Get the description of a combined worker.
    See \ref SchedulingHelpers for more details.
+
+   \p workerid is the requested combined worker id,
+   \p worker_size returns the number of workers in the combined worker,
+   \p combined_workerid returns the list for worker ids in the combined worker.
 */
 int starpu_combined_worker_get_description(int workerid, int *worker_size, int **combined_workerid);
 
