@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -33,6 +33,10 @@
 #include <numaif.h>
 #endif
 
+#ifdef STARPU_HAVE_VALGRIND_H
+#include <valgrind/valgrind.h>
+#endif
+
 static unsigned long size = 4096;
 static unsigned nblocks = 16;
 static size_t blocksize;
@@ -41,12 +45,15 @@ static int p = -1;
 static int q = -1;
 static unsigned display = 0;
 static unsigned no_prio = 0;
-static char *path = "./starpu-ooc-files";
 
 #ifdef STARPU_HAVE_LIBNUMA
 static unsigned numa = 0;
 #endif
 
+unsigned bound = 0;
+unsigned bounddeps = 0;
+unsigned boundprio = 0;
+
 static size_t allocated_memory = 0;
 
 static starpu_data_handle_t *dataA_handles;
@@ -109,9 +116,25 @@ static void parse_args(int argc, char **argv)
 			path = argv[++i];
 		}
 
+		if (strcmp(argv[i], "-bound") == 0)
+		{
+			bound = 1;
+		}
+		if (strcmp(argv[i], "-bounddeps") == 0)
+		{
+			bound = 1;
+			bounddeps = 1;
+		}
+		if (strcmp(argv[i], "-bounddepsprio") == 0)
+		{
+			bound = 1;
+			bounddeps = 1;
+			boundprio = 1;
+		}
+
 		if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "-help") == 0 || strcmp(argv[i], "--help") == 0)
 		{
-			fprintf(stderr,"usage: %s [-size n] [-nblocks b] [-check] [-display] [-numa] [-p p] [-q q] [-path PATH]\n", argv[0]);
+			fprintf(stderr,"usage: %s [-size n] [-nblocks b] [-check] [-display] [-numa] [-p p] [-q q] [-path PATH] [-bound] [-bounddeps] [-bounddepsprio]\n", argv[0]);
 			fprintf(stderr,"\np * q must be equal to the number of MPI nodes\n");
 			exit(0);
 		}
@@ -257,6 +280,32 @@ static void init_matrix(int rank)
 	//display_all_blocks(nblocks, size/nblocks);
 }
 
+static void destroy_matrix(int rank)
+{
+	char *filename;
+	unsigned filename_length = strlen(path) + 1 + sizeof(nblocks)*3 + 1 + sizeof(nblocks)*3 + 1;
+	unsigned i,j;
+
+	filename = malloc(filename_length);
+
+	for (j = 0; j < nblocks; j++)
+	{
+		for (i = 0; i < nblocks; i++)
+		{
+			int block_rank = get_block_rank(i, j);
+			if (block_rank == rank)
+			{
+				snprintf(filename, filename_length, "%s/%u,%u", path, i, j);
+				unlink(filename);
+			}
+		}
+	}
+
+	free(filename);
+
+	rmdir(path);
+}
+
 TYPE *STARPU_PLU(get_block)(unsigned i, unsigned j)
 {
 	(void)i;
@@ -366,8 +415,14 @@ int main(int argc, char **argv)
 		free(y);
 	}
 
+	if (bound)
+		starpu_bound_start(bounddeps, boundprio);
+
 	double timing = STARPU_PLU(plu_main)(nblocks, rank, world_size, no_prio);
 
+	if (bound)
+		starpu_bound_stop();
+
 	/*
 	 * 	Report performance
 	 */
@@ -378,7 +433,18 @@ int main(int argc, char **argv)
 
 		unsigned n = size;
 		double flop = (2.0f*n*n*n)/3.0f;
-		fprintf(stderr, "Synthetic GFlops : %2.2f\n", (flop/timing/1000.0f));
+		printf("# size\tms\tGFlops"); fflush(stdout);
+		if (bound)
+			printf("\tTms\tTGFlops");
+		printf("\n");
+		printf("%u\t%.0f\t%2.2f", n, timing/1000, (flop/timing/1000.0f));
+		if (bound)
+		{
+			double min;
+			starpu_bound_compute(&min, NULL, 0);
+			printf("\t%.0f\t%.1f", min, flop/min/1000000.0f);
+		}
+		printf("\n");
 	}
 
 	/*
@@ -439,6 +505,8 @@ int main(int argc, char **argv)
 	free(dataA_handles);
 	free(disk_objs);
 
+	destroy_matrix(rank);
+
 	starpu_cublas_shutdown();
 	starpu_mpi_shutdown();
 
