@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2019,2021  Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -78,7 +78,8 @@ int _starpu_mpi_choose_node(starpu_data_handle_t handle, enum starpu_data_access
 
 		for (i = 0; i < STARPU_MAXNODES; i++)
 		{
-			if (starpu_node_get_kind(i) == STARPU_CPU_RAM)
+                       if (starpu_node_get_kind(i) == STARPU_CPU_RAM
+                               || (_starpu_mpi_has_cuda && starpu_node_get_kind(i) == STARPU_CUDA_RAM))
 			{
 				starpu_ssize_t size = starpu_memory_get_available(i);
 				if (size >= needed && size > maximum)
@@ -680,31 +681,40 @@ void starpu_mpi_comm_stats_enable()
 int _starpu_mpi_data_cpy(starpu_data_handle_t dst_handle, starpu_data_handle_t src_handle, MPI_Comm comm, int asynchronous, void (*callback_func)(void *), void *callback_arg, int priority)
 {
 	int src, dst;
+	int ret;
 
+	if (dst_handle == src_handle)
+	{
+		if (callback_func)
+			callback_func(callback_arg);
+		return 0;
+	}
+
+	ret = 0;
 	src = starpu_mpi_data_get_rank(src_handle);
 	dst = starpu_mpi_data_get_rank(dst_handle);
 
 	if (src == dst)
 		// Both data are on the same node, no need to transfer data
-		return starpu_data_cpy_priority(dst_handle, src_handle, asynchronous, callback_func, callback_arg, priority);
+		ret = starpu_data_cpy_priority(dst_handle, src_handle, asynchronous, callback_func, callback_arg, priority);
 	else
 	{
 		// We need to transfer data
 		int rank;
-		starpu_mpi_comm_rank(comm, &rank);
+		starpu_mpi_tag_t tag;
 
-		starpu_mpi_tag_t tag = starpu_mpi_data_get_tag(dst_handle);
+		tag = starpu_mpi_data_get_tag(dst_handle);
+		starpu_mpi_comm_rank(comm, &rank);
 
 		if (rank == src)
 		{
 			if (asynchronous == 1)
-				return starpu_mpi_isend_detached_prio(src_handle, dst, tag, priority, comm, NULL, NULL);
+				ret = starpu_mpi_isend_detached_prio(src_handle, dst, tag, priority, comm, NULL, NULL);
 			else
-				return starpu_mpi_send_prio(src_handle, dst, tag, priority, comm);
+				ret = starpu_mpi_send_prio(src_handle, dst, tag, priority, comm);
 		}
 		else if (rank == dst)
 		{
-			int ret;
 			if (asynchronous == 1)
 				ret = starpu_mpi_irecv_detached_prio(dst_handle, src, tag, priority, comm, callback_func, callback_arg);
 			else
@@ -713,12 +723,12 @@ int _starpu_mpi_data_cpy(starpu_data_handle_t dst_handle, starpu_data_handle_t s
 				if (callback_func)
 					callback_func(callback_arg);
 			}
-			starpu_mpi_cache_flush(comm, dst_handle);
 			return ret;
 
 		}
-		return 0;
 	}
+	starpu_mpi_cache_flush(comm, dst_handle);
+	return ret;
 }
 
 int starpu_mpi_data_cpy(starpu_data_handle_t dst_handle, starpu_data_handle_t src_handle, MPI_Comm comm, int asynchronous, void (*callback_func)(void *), void *callback_arg)
