@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -37,14 +37,21 @@ struct starpu_codelet mycodelet =
 	.model = &starpu_perfmodel_nop,
 };
 
-int test(int rank, int node, int *before, int *after, int task_insert, int data_array)
+int test(int rank, int node, starpu_mpi_tag_t initial_tag, int *before, int *after, int task_insert, int data_array)
 {
 	int ok, ret, i, x[2];
 	starpu_data_handle_t data_handles[2];
 	struct starpu_data_descr descrs[2];
 	int barrier_ret;
+	struct starpu_conf conf;
 
-	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, NULL);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, &conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
 	if (starpu_cpu_worker_get_count() == 0)
@@ -62,7 +69,7 @@ int test(int rank, int node, int *before, int *after, int task_insert, int data_
 		if (rank <= 1)
 		{
 			x[i] = before[rank*2+i];
-			FPRINTF_MPI(stderr, "before computation x[%d] = %d\n", i, x[i]);
+			//FPRINTF_MPI(stderr, "before computation x[%d] = %d\n", i, x[i]);
 		}
 		else
 			x[i] = rank*2+i;
@@ -70,7 +77,7 @@ int test(int rank, int node, int *before, int *after, int task_insert, int data_
 			starpu_variable_data_register(&data_handles[i], 0, (uintptr_t)&x[i], sizeof(int));
 		else
 			starpu_variable_data_register(&data_handles[i], -1, (uintptr_t)NULL, sizeof(int));
-		starpu_mpi_data_register(data_handles[i], i, i);
+		starpu_mpi_data_register(data_handles[i], initial_tag+i, i);
 		descrs[i].handle = data_handles[i];
 	}
 	descrs[0].mode = STARPU_RW;
@@ -154,6 +161,8 @@ int test(int rank, int node, int *before, int *after, int task_insert, int data_
 								     STARPU_RW, data_handles[0], STARPU_R, data_handles[1],
 								     STARPU_VALUE, &rank, sizeof(rank),
 								     STARPU_EXECUTE_ON_NODE, node, 0);
+					if (ret == -ENODEV)
+						goto enodev;
 					break;
 				}
 				case 1:
@@ -162,6 +171,8 @@ int test(int rank, int node, int *before, int *after, int task_insert, int data_
 								     STARPU_DATA_ARRAY, data_handles, 2,
 								     STARPU_VALUE, &rank, sizeof(rank),
 								     STARPU_EXECUTE_ON_NODE, node, 0);
+					if (ret == -ENODEV)
+						goto enodev;
 					break;
 				}
 				case 2:
@@ -170,6 +181,8 @@ int test(int rank, int node, int *before, int *after, int task_insert, int data_
 								     STARPU_DATA_MODE_ARRAY, descrs, 2,
 								     STARPU_VALUE, &rank, sizeof(rank),
 								     STARPU_EXECUTE_ON_NODE, node, 0);
+					if (ret == -ENODEV)
+						goto enodev;
 					break;
 				}
 			}
@@ -193,7 +206,8 @@ enodev:
 		for(i=0; i<2; i++)
 		{
 			ok = ok && (x[i] == after[rank*2+i]);
-			FPRINTF_MPI(stderr, "after computation x[%d] = %d, should be %d\n", i, x[i], after[rank*2+i]);
+			if (x[i] != after[rank*2+i])
+				FPRINTF_MPI(stderr, "after computation x[%d] = %d, should be %d\n", i, x[i], after[rank*2+i]);
 		}
 		FPRINTF_MPI(stderr, "result is %s\n", ok?"CORRECT":"NOT CORRECT");
 	}
@@ -214,9 +228,10 @@ int main(int argc, char **argv)
 	int before[4] = {10, 20, 11, 22};
 	int after_node[2][4] = {{220, 20, 11, 22}, {220, 20, 11, 22}};
 	int node, insert_task, data_array;
+	starpu_mpi_tag_t initial_tag = 0;
 
 	MPI_INIT_THREAD_real(&argc, &argv, MPI_THREAD_SERIALIZED);
-	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
+	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
 
 	global_ret = 0;
 	for(node=0 ; node<=1 ; node++)
@@ -225,7 +240,8 @@ int main(int argc, char **argv)
 		{
 			for(data_array=0 ; data_array<=2 ; data_array++)
 			{
-				ret = test(rank, node, before, after_node[node], insert_task, data_array);
+				ret = test(rank, node, initial_tag, before, after_node[node], insert_task, data_array);
+				initial_tag += 2;
 				if (ret == -ENODEV || ret)
 					global_ret = ret;
 			}
