@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +29,10 @@
 extern void increment_cuda(void *descr[], void *_args);
 #endif
 
+#ifdef STARPU_USE_HIP
+extern void increment_hip(void *descr[], void *_args);
+#endif
+
 void increment_cpu(void *descr[], void *_args)
 {
 	(void)_args;
@@ -40,6 +44,9 @@ static struct starpu_codelet increment_cl =
 {
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {increment_cuda},
+#endif
+#ifdef STARPU_USE_HIP
+	.hip_funcs = {increment_hip},
 #endif
 	.cpu_funcs = {increment_cpu},
 	.nbuffers = 1,
@@ -87,19 +94,19 @@ int main(int argc, char **argv)
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
 	starpu_mpi_comm_size(MPI_COMM_WORLD, &size);
 
-	if (size < 2 || (starpu_cpu_worker_get_count() + starpu_cuda_worker_get_count() == 0))
+	if (size < 2 || (starpu_cpu_worker_get_count() + starpu_cuda_worker_get_count() == 0) || (starpu_cpu_worker_get_count() + starpu_hip_worker_get_count() == 0))
 	{
 		if (rank == 0)
 		{
 			if (size < 2)
 				FPRINTF(stderr, "We need at least 2 processes.\n");
 			else
-				FPRINTF(stderr, "We need at least 1 CPU or CUDA worker.\n");
+				FPRINTF(stderr, "We need at least 1 CPU or CUDA or HIP worker.\n");
 		}
 		starpu_mpi_shutdown();
 		if (!mpi_init)
 			MPI_Finalize();
-		return STARPU_TEST_SKIPPED;
+		return rank == 0 ? STARPU_TEST_SKIPPED : 0;
 	}
 
 	starpu_vector_data_register(&token_handle, 0, (uintptr_t)&token, 1, sizeof(token));
@@ -124,7 +131,8 @@ int main(int argc, char **argv)
 		else
 		{
 			MPI_Status status;
-			starpu_mpi_recv(token_handle, (rank+size-1)%size, tag, MPI_COMM_WORLD, &status);
+			ret = starpu_mpi_recv(token_handle, (rank+size-1)%size, tag, MPI_COMM_WORLD, &status);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_recv");
 		}
 
 		increment_token(token_handle);
@@ -138,7 +146,8 @@ int main(int argc, char **argv)
 		else
 		{
 			int sent = 0;
-			starpu_mpi_issend_detached(token_handle, (rank+1)%size, tag+1, MPI_COMM_WORLD, callback, &sent);
+			ret = starpu_mpi_issend_detached(token_handle, (rank+1)%size, tag+1, MPI_COMM_WORLD, callback, &sent);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_issend_detached");
 
 			STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 			while (!sent)
@@ -148,9 +157,9 @@ int main(int argc, char **argv)
 	}
 
 	starpu_data_unregister(token_handle);
-	starpu_mpi_shutdown();
-
 	FPRINTF_MPI(stderr, "Final value for token %d\n", token);
+
+	starpu_mpi_shutdown();
 	if (!mpi_init)
 		MPI_Finalize();
 
