@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012,2014                           Inria
- * Copyright (C) 2016,2017                                CNRS
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,11 +27,11 @@
 unsigned val[3];
 starpu_pthread_mutex_t mut[3];
 
-/* Every implementation of a codelet must have this prototype, the first                                                                                                                                             * argument (buffers) describes the buffers/streams that are managed by the
+/* Every implementation of a codelet must have this prototype, the first
+ * argument (buffers) describes the buffers/streams that are managed by the
  * DSM; the second arguments references read-only data that is passed as an
  * argument of the codelet (task->cl_arg). Here, "buffers" is unused as there
  * are no data input/output managed by the DSM (cl.nbuffers = 0) */
-
 void cpu_func(__attribute__((unused))void *buffers[], void *cl_arg)
 {
 	unsigned sched_ctx = *((unsigned *) cl_arg);
@@ -82,8 +81,7 @@ int main()
 	int ret = starpu_init(NULL);
 
 	if (ret == -ENODEV)
-        return 77;
-
+		return 77;
 
 	/* create contexts */
 	unsigned sched_ctx1 = starpu_sched_ctx_create(NULL, 0, "sched_ctx1", STARPU_SCHED_CTX_POLICY_NAME, "dmda", STARPU_SCHED_CTX_HIERARCHY_LEVEL, 0, 0);
@@ -96,13 +94,13 @@ int main()
 	struct sc_hypervisor_policy policy;
 	policy.custom = 0;
 	/* indicate which strategy to use
-	   in this particular case we use app_driven which allows the user to resize 
-	   the ctxs dynamically at particular moments of the execution of the application */
+	* in this particular case we use app_driven which allows the user to resize
+	* the ctxs dynamically at particular moments of the execution of the application */
 	policy.name = "feft_lp";
 	void *perf_counters = sc_hypervisor_init(&policy);
 
-	/* let starpu know which performance counters should use 
-	   to inform the hypervisor how the application and the resources are executing */
+	/* let starpu know which performance counters should use
+	 * to inform the hypervisor how the application and the resources are executing */
 	starpu_sched_ctx_set_perf_counters(sched_ctx1, perf_counters);
 	starpu_sched_ctx_set_perf_counters(sched_ctx2, perf_counters);
 	starpu_sched_ctx_set_perf_counters(sched_ctx3, perf_counters);
@@ -111,7 +109,7 @@ int main()
 	double flops2 = NTASKS*NINCR*1000000000.0;
 	double flops3 = NTASKS*NINCR*1000000000.0;
 	/* register the contexts that should be managed by the hypervisor
-	   and indicate an approximate amount of workload if known;
+	 * and indicate an approximate amount of workload if known;
 	   in this case we don't know it and we put 0 */
 	sc_hypervisor_register_ctx(sched_ctx1, flops1);
 	sc_hypervisor_register_ctx(sched_ctx2, flops2);
@@ -131,8 +129,8 @@ int main()
 			  SC_HYPERVISOR_MAX_WORKERS, ncpus,
 			  SC_HYPERVISOR_NULL);
 
-        /* lp strategy allows sizing the contexts because we know the total number of flops
-	   to be executed */
+	/* lp strategy allows sizing the contexts because we know the total number of flops
+	 * to be executed */
 	sc_hypervisor_size_ctxs(NULL, -1, NULL, -1);
 
 	starpu_pthread_t tid[3];
