@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2016                                Inria
- * Copyright (C) 2013,2014,2016,2017                      CNRS
- * Copyright (C) 2013-2015,2017                           Université de Bordeaux
- * Copyright (C) 2013                                     Thibaut Lambert
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -191,8 +189,8 @@ int sc_hypervisor_get_movable_nworkers(struct sc_hypervisor_policy_config *confi
 	{
 		worker = workers->get_next(workers, &it);
 		enum starpu_worker_archtype curr_arch = starpu_worker_get_type(worker);
-                if(arch == STARPU_ANY_WORKER || curr_arch == arch)
-                {
+		if(arch == STARPU_ANY_WORKER || curr_arch == arch)
+		{
 			if(!config->fixed_workers[worker])
 				potential_workers++;
 		}
@@ -202,11 +200,11 @@ int sc_hypervisor_get_movable_nworkers(struct sc_hypervisor_policy_config *confi
 }
 
 /* compute the number of workers that should be moved depending:
-   - on the min/max number of workers in a context imposed by the user,
-   - on the resource granularity imposed by the user for the resizing process*/
+ * - on the min/max number of workers in a context imposed by the user,
+ * - on the resource granularity imposed by the user for the resizing process*/
 int sc_hypervisor_compute_nworkers_to_move(unsigned req_sched_ctx)
 {
-       	struct sc_hypervisor_policy_config *config = sc_hypervisor_get_config(req_sched_ctx);
+	struct sc_hypervisor_policy_config *config = sc_hypervisor_get_config(req_sched_ctx);
 	int nworkers = (int)starpu_sched_ctx_get_nworkers(req_sched_ctx);
 	int nworkers_to_move = 0;
 
@@ -321,7 +319,7 @@ double sc_hypervisor_get_slowest_ctx_exec_time(void)
 		if(elapsed_time > slowest_time)
 			slowest_time = elapsed_time;
 
-        }
+	}
 	return slowest_time;
 }
 
@@ -331,7 +329,7 @@ double sc_hypervisor_get_fastest_ctx_exec_time(void)
 	int nsched_ctxs = sc_hypervisor_get_nsched_ctxs();
 
 	double curr_time = starpu_timing_now();
- 	double fastest_time = curr_time;
+	double fastest_time = curr_time;
 
 	int s;
 	struct sc_hypervisor_wrapper* sc_w;
@@ -345,7 +343,7 @@ double sc_hypervisor_get_fastest_ctx_exec_time(void)
 		if(elapsed_time < fastest_time)
 			fastest_time = elapsed_time;
 
-        }
+	}
 
 	return fastest_time;
 }
@@ -409,30 +407,30 @@ unsigned sc_hypervisor_get_index_for_arch(enum starpu_worker_archtype arch, stru
 
 void sc_hypervisor_get_tasks_times(int nw, int nt, double times[nw][nt], int *workers, unsigned size_ctxs, struct sc_hypervisor_policy_task_pool *task_pools)
 {
-        struct sc_hypervisor_policy_task_pool *tp;
-        int w, t;
+	struct sc_hypervisor_policy_task_pool *tp;
+	int w, t;
 	for(w = 0; w < nw; w++)
 		for(t = 0; t < nt; t++)
 			times[w][t] = NAN;
-        for (w = 0; w < nw; w++)
-        {
-                for (t = 0, tp = task_pools; tp; t++, tp = tp->next)
-                {
+	for (w = 0; w < nw; w++)
+	{
+		for (t = 0, tp = task_pools; tp; t++, tp = tp->next)
+		{
 			int worker = workers == NULL ? w : workers[w];
-                        struct starpu_perfmodel_arch* arch = starpu_worker_get_perf_archtype(worker, STARPU_NMAX_SCHED_CTXS);
-                        double length = starpu_perfmodel_history_based_expected_perf(tp->cl->model, arch, tp->footprint);
+			struct starpu_perfmodel_arch* arch = starpu_worker_get_perf_archtype(worker, STARPU_NMAX_SCHED_CTXS);
+			double length = starpu_perfmodel_history_based_expected_perf(tp->cl->model, arch, tp->footprint);
 
-                        if (isnan(length))
-                                times[w][t] = NAN;
+			if (isnan(length))
+				times[w][t] = NAN;
 			else
 			{
-                                times[w][t] = (length / 1000.);
+				times[w][t] = (length / 1000.);
 				double transfer_time = 0.0;
 				unsigned worker_in_ctx = starpu_sched_ctx_contains_worker(worker, tp->sched_ctx_id);
-				enum starpu_worker_archtype arch = starpu_worker_get_type(worker);
+				enum starpu_worker_archtype warch = starpu_worker_get_type(worker);
 				if(!worker_in_ctx && !size_ctxs)
 				{
-					if(arch == STARPU_CUDA_WORKER)
+					if(warch == STARPU_CUDA_WORKER)
 					{
 						double transfer_speed = starpu_transfer_bandwidth(STARPU_MAIN_RAM, starpu_worker_get_memory_node(worker));
 						if(transfer_speed > 0.0)
@@ -442,9 +440,9 @@ void sc_hypervisor_get_tasks_times(int nw, int nt, double times[nw][nt], int *wo
 						transfer_time += latency/1000.;
 //						transfer_time *=4;
 					}
-					else if(arch == STARPU_CPU_WORKER)
+					else if (warch == STARPU_CPU_WORKER)
 					{
-						if(!starpu_sched_ctx_contains_type_of_worker(arch, tp->sched_ctx_id))
+						if(!starpu_sched_ctx_contains_type_of_worker(warch, tp->sched_ctx_id))
 						{
 							double transfer_speed = starpu_transfer_bandwidth(starpu_worker_get_memory_node(worker), STARPU_MAIN_RAM);
 							if(transfer_speed > 0.0)
@@ -460,8 +458,8 @@ void sc_hypervisor_get_tasks_times(int nw, int nt, double times[nw][nt], int *wo
 				times[w][t] += transfer_time;
 			}
 //			printf("sc%d w%d task %s nt %d times %lf s\n", tp->sched_ctx_id, w, tp->cl->model->symbol, tp->n, times[w][t]);
-                }
-        }
+		}
+	}
 }
 
 unsigned sc_hypervisor_check_idle(unsigned sched_ctx, int worker)
@@ -486,7 +484,7 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs(unsigned *sched_ctxs_in, int ns_
 	unsigned *sched_ctxs = sched_ctxs_in == NULL ? sc_hypervisor_get_sched_ctxs() : sched_ctxs_in;
 	int ns = ns_in == -1 ? sc_hypervisor_get_nsched_ctxs() : ns_in;
 	int *workers = workers_in;
-	int nworkers = nworkers_in == -1 ? starpu_worker_get_count() : nworkers_in;
+	int nworkers = nworkers_in == -1 ? (int)starpu_worker_get_count() : nworkers_in;
 	int i = 0, j = 0;
 	struct sc_hypervisor_wrapper* sc_w;
 	struct sc_hypervisor_wrapper* other_sc_w;
@@ -536,8 +534,8 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs(unsigned *sched_ctxs_in, int ns_
 		free(tw);
 	}
 
-/* if we have an optimal speed for each type of worker compare the monitored one with the
-   theoretical one */
+	/* if we have an optimal speed for each type of worker compare the monitored one with the
+	 * theoretical one */
 	if(has_opt_v)
 	{
 		for(i = 0; i < ns; i++)
@@ -576,8 +574,8 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs(unsigned *sched_ctxs_in, int ns_
 				{
 					if(sched_ctxs[i] != sched_ctxs[j])
 					{
-						unsigned nworkers = starpu_sched_ctx_get_nworkers(sched_ctxs[j]);
-						if(nworkers == 0)
+						unsigned snworkers = starpu_sched_ctx_get_nworkers(sched_ctxs[j]);
+						if(snworkers == 0)
 							return 1;
 
 						other_sc_w = sc_hypervisor_get_wrapper(sched_ctxs[j]);
@@ -602,7 +600,7 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs_on_level(int level, int *workers
 {
 	sc_hypervisor_get_ctxs_on_level(sched_ctxs, nsched_ctxs, level, father_sched_ctx_id);
 
-	if(*nsched_ctxs  > 0)
+	if(*nsched_ctxs	 > 0)
 		return sc_hypervisor_check_speed_gap_btw_ctxs(*sched_ctxs, *nsched_ctxs, workers_in, nworkers_in);
 	return 0;
 }
