@@ -22,7 +22,7 @@ CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_context CL_API_CALL
 soclCreateContextFromType(const cl_context_properties * properties,
 			  cl_device_type                device_type,
-			  void (*pfn_notify)(const char *, const void *, size_t, void *),
+			  void (CL_CALLBACK *pfn_notify)(const char *, const void *, size_t, void *),
 			  void *                        user_data,
 			  cl_int *                      errcode_ret)
 {
