@@ -17,10 +17,14 @@
 #ifndef __IDLE_HOOK_H__
 #define __IDLE_HOOK_H__
 
+#pragma GCC visibility push(hidden)
+
 /** @file */
 
 void _starpu_init_idle_hooks(void);
 
 unsigned _starpu_execute_registered_idle_hooks(void);
 
+#pragma GCC visibility pop
+
 #endif /* !__IDLE_HOOK_H__ */
