@@ -17,10 +17,14 @@
 #ifndef __PROGRESS_HOOK_H__
 #define __PROGRESS_HOOK_H__
 
+#pragma GCC visibility push(hidden)
+
 /** @file */
 
 void _starpu_init_progression_hooks(void);
 
 unsigned _starpu_execute_registered_progression_hooks(void);
 
+#pragma GCC visibility pop
+
 #endif /* !__PROGRESS_HOOK_H__ */
