@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2016       Uppsala University
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -36,6 +36,8 @@
 #include <hwloc.h>
 #endif
 
+#pragma GCC visibility push(hidden)
+
 #define NO_RESIZE -1
 #define REQ_RESIZE 0
 #define DO_RESIZE 1
@@ -105,11 +107,11 @@ struct _starpu_sched_ctx
 	   to this context*/
 	unsigned finished_submit;
 
-        /** By default we have a binary type of priority: either a task is a priority
+	/** By default we have a binary type of priority: either a task is a priority
          * task (level 1) or it is not (level 0). */
-     	int min_priority;
+	int min_priority;
 	int max_priority;
-     	int min_priority_is_set;
+	int min_priority_is_set;
 	int max_priority_is_set;
 
 	/** hwloc tree structure of workers */
@@ -149,8 +151,8 @@ struct _starpu_sched_ctx
 	   them awake & use them in the parallel code*/
 	unsigned awake_workers;
 
-	/** function called when initializing the scheduler */
-	void (*init_sched)(unsigned);
+	/** callback function called when initializing the scheduler */
+	void (*callback_sched)(unsigned);
 
 	int sub_ctxs[STARPU_NMAXWORKERS];
 	int nsub_ctxs;
@@ -184,8 +186,8 @@ void _starpu_init_all_sched_ctxs(struct _starpu_machine_config *config);
 /** allocate all structures belonging to a context */
 struct _starpu_sched_ctx*  _starpu_create_sched_ctx(struct starpu_sched_policy *policy, int *workerid, int nworkerids, unsigned is_init_sched, const char *sched_name,
 						    int min_prio_set, int min_prio,
-						    int max_prio_set, int max_prio, unsigned awake_workers, void (*sched_policy_init)(unsigned), void *user_data,
-							int nsub_ctxs, int *sub_ctxs, int nsms);
+						    int max_prio_set, int max_prio, unsigned awake_workers, void (*sched_policy_callback)(unsigned), void *user_data,
+						    int nsub_ctxs, int *sub_ctxs, int nsms);
 
 /** delete all sched_ctx */
 void _starpu_delete_all_sched_ctxs();
@@ -208,36 +210,32 @@ void _starpu_decrement_nready_tasks_of_sched_ctx(unsigned sched_ctx_id, double r
 unsigned _starpu_increment_nready_tasks_of_sched_ctx(unsigned sched_ctx_id, double ready_flops, struct starpu_task *task);
 int _starpu_wait_for_no_ready_of_sched_ctx(unsigned sched_ctx_id);
 
-/** Return the corresponding index of the workerid in the ctx table */
-int _starpu_get_index_in_ctx_of_workerid(unsigned sched_ctx, unsigned workerid);
-
-/** Get the mutex corresponding to the global workerid */
-starpu_pthread_mutex_t *_starpu_get_sched_mutex(struct _starpu_sched_ctx *sched_ctx, int worker);
-
-/** Get workers belonging to a certain context, it returns the number of workers
- take care: no mutex taken, the list of workers might not be updated */
+/** Get workers belonging to a certain context, it returns the number
+ * of workers take care: no mutex taken, the list of workers might not
+ * be updated
+ */
 int _starpu_get_workers_of_sched_ctx(unsigned sched_ctx_id, int *pus, enum starpu_worker_archtype arch);
 
-/** Let the worker know it does not belong to the context and that
-   it should stop poping from it */
+/** Let the worker know it does not belong to the context and that it
+ * should stop poping from it
+ */
 void _starpu_worker_gets_out_of_ctx(unsigned sched_ctx_id, struct _starpu_worker *worker);
 
 /** Check if the worker belongs to another sched_ctx */
 unsigned _starpu_worker_belongs_to_a_sched_ctx(int workerid, unsigned sched_ctx_id);
 
-/** indicates wheather this worker should go to sleep or not
-   (if it is the last one awake in a context he should better keep awake) */
+/** indicates whether this worker should go to sleep or not (if it is
+ * the last one awake in a context he should better keep awake)
+ */
 unsigned _starpu_sched_ctx_last_worker_awake(struct _starpu_worker *worker);
 
 /** If starpu_sched_ctx_set_context() has been called, returns the context
  * id set by its last call, or the id of the initial context */
-unsigned _starpu_sched_ctx_get_current_context();
+unsigned _starpu_sched_ctx_get_current_context() STARPU_ATTRIBUTE_VISIBILITY_DEFAULT;
 
 /** verify that some worker can execute a certain task */
 int _starpu_workers_able_to_execute_task(struct starpu_task *task, struct _starpu_sched_ctx *sched_ctx);
 
-void _starpu_fetch_tasks_from_empty_ctx_list(struct _starpu_sched_ctx *sched_ctx);
-
 unsigned _starpu_sched_ctx_allow_hypervisor(unsigned sched_ctx_id);
 
 struct starpu_perfmodel_arch * _starpu_sched_ctx_get_perf_archtype(unsigned sched_ctx);
@@ -266,6 +264,7 @@ static inline int _starpu_sched_ctx_check_write_locked(unsigned sched_ctx_id)
 
 static inline void _starpu_sched_ctx_lock_write(unsigned sched_ctx_id)
 {
+	STARPU_ASSERT(sched_ctx_id <= STARPU_NMAX_SCHED_CTXS);
 	struct _starpu_sched_ctx *sched_ctx = _starpu_get_sched_ctx_struct(sched_ctx_id);
 	STARPU_HG_DISABLE_CHECKING(sched_ctx->lock_write_owner);
 	STARPU_ASSERT(!starpu_pthread_equal(sched_ctx->lock_write_owner, starpu_pthread_self()));
@@ -276,6 +275,7 @@ static inline void _starpu_sched_ctx_lock_write(unsigned sched_ctx_id)
 
 static inline void _starpu_sched_ctx_unlock_write(unsigned sched_ctx_id)
 {
+	STARPU_ASSERT(sched_ctx_id <= STARPU_NMAX_SCHED_CTXS);
 	struct _starpu_sched_ctx *sched_ctx = _starpu_get_sched_ctx_struct(sched_ctx_id);
 	STARPU_HG_DISABLE_CHECKING(sched_ctx->lock_write_owner);
 	STARPU_ASSERT(starpu_pthread_equal(sched_ctx->lock_write_owner, starpu_pthread_self()));
@@ -286,6 +286,7 @@ static inline void _starpu_sched_ctx_unlock_write(unsigned sched_ctx_id)
 
 static inline void _starpu_sched_ctx_lock_read(unsigned sched_ctx_id)
 {
+	STARPU_ASSERT(sched_ctx_id <= STARPU_NMAX_SCHED_CTXS);
 	struct _starpu_sched_ctx *sched_ctx = _starpu_get_sched_ctx_struct(sched_ctx_id);
 	STARPU_HG_DISABLE_CHECKING(sched_ctx->lock_write_owner);
 	STARPU_ASSERT(!starpu_pthread_equal(sched_ctx->lock_write_owner, starpu_pthread_self()));
@@ -295,6 +296,7 @@ static inline void _starpu_sched_ctx_lock_read(unsigned sched_ctx_id)
 
 static inline void _starpu_sched_ctx_unlock_read(unsigned sched_ctx_id)
 {
+	STARPU_ASSERT(sched_ctx_id <= STARPU_NMAX_SCHED_CTXS);
 	struct _starpu_sched_ctx *sched_ctx = _starpu_get_sched_ctx_struct(sched_ctx_id);
 	STARPU_HG_DISABLE_CHECKING(sched_ctx->lock_write_owner);
 	STARPU_ASSERT(!starpu_pthread_equal(sched_ctx->lock_write_owner, starpu_pthread_self()));
@@ -317,4 +319,7 @@ static inline unsigned _starpu_sched_ctx_worker_is_master_for_child_ctx(unsigned
 /** Go through the list of deferred ctx changes of the current worker and apply
  * any ctx change operation found until the list is empty */
 void _starpu_worker_apply_deferred_ctx_changes(void);
+
+#pragma GCC visibility pop
+
 #endif // __SCHED_CONTEXT_H__
