@@ -72,6 +72,10 @@ static struct starpu_sched_policy *predefined_policies[] =
 	&_starpu_sched_modular_parallel_random_policy,
 	&_starpu_sched_modular_parallel_random_prio_policy,
 	&_starpu_sched_modular_ws_policy,
+	&_starpu_sched_modular_dmda_policy,
+	&_starpu_sched_modular_dmdap_policy,
+	&_starpu_sched_modular_dmdar_policy,
+	&_starpu_sched_modular_dmdas_policy,
 	&_starpu_sched_modular_heft_policy,
 	&_starpu_sched_modular_heft_prio_policy,
 	&_starpu_sched_modular_heft2_policy,
@@ -281,7 +285,7 @@ struct starpu_sched_policy *_starpu_select_sched_policy(struct _starpu_machine_c
 	if (selected_policy)
 		return selected_policy;
 
-	/* First, we check whether the application explicitely gave a scheduling policy or not */
+	/* First, we check whether the application explicitly gave a scheduling policy or not */
 	if (user_conf && (user_conf->sched_policy))
 		return user_conf->sched_policy;
 
@@ -321,7 +325,7 @@ void _starpu_init_sched_policy(struct _starpu_machine_config *config, struct _st
 #ifdef STARPU_HAVE_HWLOC
 		sched_ctx->sched_policy->worker_type = STARPU_WORKER_TREE;
 #else
-		_STARPU_DISP("STARPU_WORKER_TREE ignored, please rebuild StarPU with hwloc support to enable it.");
+		_STARPU_DISP("STARPU_WORKER_TREE ignored, please rebuild StarPU with hwloc support to enable it.\n");
 #endif
 	}
 	starpu_sched_ctx_create_worker_collection(sched_ctx->id,
@@ -394,7 +398,7 @@ static void _starpu_push_task_on_specific_worker_notify_sched(struct starpu_task
 	}
 }
 
-/* Enqueue a task into the list of tasks explicitely attached to a worker. In
+/* Enqueue a task into the list of tasks explicitly attached to a worker. In
  * case workerid identifies a combined worker, a task will be enqueued into
  * each worker of the combination. */
 static int _starpu_push_task_on_specific_worker(struct starpu_task *task, int workerid)
@@ -1101,7 +1105,7 @@ pick:
 
 	/*
 	 * This worker may not be able to execute this task. In this case, we
-	 * should return the task anyway. It will be pushed back almost immediatly.
+	 * should return the task anyway. It will be pushed back almost immediately.
 	 * This way, we avoid computing and executing the conversions tasks.
 	 * Here, we do not care about what implementation is used.
 	 */
