@@ -452,7 +452,7 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 	 * do anything since this is transparent ?) */
 	unsigned still_valid[STARPU_MAXNODES];
 
-	/* we do 2 passes : the first pass determines wether the data is still
+	/* we do 2 passes : the first pass determines whether the data is still
 	 * valid or not, the second pass is needed to choose between STARPU_SHARED and
 	 * STARPU_OWNER */
 
@@ -666,7 +666,7 @@ void starpu_data_partition_clean(starpu_data_handle_t root_handle, unsigned npar
 #ifdef STARPU_DEVEL
 #warning FIXME: better choose gathering node
 #endif
-	starpu_data_partition_clean_node(root_handle, nparts, children, STARPU_MAIN_RAM);
+	starpu_data_partition_clean_node(root_handle, nparts, children, root_handle->home_node);
 }
 
 static
@@ -764,7 +764,7 @@ void starpu_data_partition_readonly_submit_sequential_consistency(starpu_data_ha
 		_starpu_spin_unlock(&children[i]->header_lock);
 	}
 
-	STARPU_ASSERT_MSG(initial_handle->initialized, "It is odd to read-only-partition a data which does not have a value yet");
+	STARPU_ASSERT_MSG(initial_handle->initialized || initial_handle->init_cl, "It is odd to read-only-partition a data which does not have a value yet");
 	struct starpu_data_descr descr[nparts];
 	char handles_sequential_consistency[nparts+1];
 	handles_sequential_consistency[0] = sequential_consistency;
@@ -813,6 +813,55 @@ void starpu_data_partition_readwrite_upgrade_submit(starpu_data_handle_t initial
 	_starpu_data_invalidate_submit_noplan(initial_handle);
 }
 
+void starpu_data_partition_readonly_downgrade_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children)
+{
+	unsigned i;
+	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
+	_starpu_spin_lock(&initial_handle->header_lock);
+	STARPU_ASSERT_MSG(initial_handle->partitioned == 1, "One can't downgrade a read-write partition planning to read-only while other partition plannings are active");
+	STARPU_ASSERT_MSG(initial_handle->part_readonly == 0, "Partition is already read-only");
+	STARPU_ASSERT_MSG(nparts > 0, "One can't partition into 0 parts");
+	initial_handle->part_readonly = 1;
+	if (initial_handle->nactive_readonly_children < initial_handle->partitioned)
+	{
+		_STARPU_REALLOC(initial_handle->active_readonly_children, initial_handle->partitioned * sizeof(initial_handle->active_readonly_children[0]));
+		_STARPU_REALLOC(initial_handle->active_readonly_nchildren, initial_handle->partitioned * sizeof(initial_handle->active_readonly_nchildren[0]));
+		initial_handle->nactive_readonly_children = initial_handle->partitioned;
+	}
+	initial_handle->active_readonly_children[initial_handle->partitioned-1] = children[0]->siblings;
+	initial_handle->active_readonly_nchildren[initial_handle->partitioned-1] = children[0]->nsiblings;
+	initial_handle->active_children = NULL;
+	initial_handle->active_nchildren = 0;
+	_starpu_spin_unlock(&initial_handle->header_lock);
+
+	for (i = 0; i < nparts; i++)
+	{
+		_starpu_spin_lock(&children[i]->header_lock);
+		children[i]->active = 1;
+		children[i]->active_ro = 1;
+		_starpu_spin_unlock(&children[i]->header_lock);
+	}
+
+	struct starpu_data_descr descr[nparts];
+	unsigned n;
+	for (i = 0, n = 0; i < nparts; i++)
+	{
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "child(%d) %p is partitioned from %p and not from the given parameter %p", i, children[i], children[i]->father_handle, initial_handle);
+		if (!children[i]->initialized)
+			/* Dropped value, do not care about coherency for this one */
+			continue;
+		descr[n].handle = children[i];
+		descr[n].mode = STARPU_R;
+		n++;
+	}
+	/* TODO: assert nparts too */
+	int ret = starpu_task_insert(initial_handle->switch_cl, initial_handle->initialized?STARPU_RW:STARPU_W, initial_handle,
+				     STARPU_DATA_MODE_ARRAY, descr, n,
+				     ///STARPU_HANDLES_SEQUENTIAL_CONSISTENCY, handles_sequential_consistency,
+				     0);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
+}
+
 void _starpu_data_unpartition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node, unsigned char *handles_sequential_consistency, void (*callback_func)(void *), void *callback_arg)
 {
 	unsigned i;
@@ -1009,7 +1058,7 @@ static void _starpu_data_partition_access_look_up(starpu_data_handle_t ancestor,
 #ifdef STARPU_DEVEL
 #warning FIXME: better choose gathering node
 #endif
-		starpu_data_unpartition_submit_r(ancestor, STARPU_MAIN_RAM);
+		starpu_data_unpartition_submit_r(ancestor, ancestor->home_node);
 	}
 
 	if (!target)
