@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2008-2011,2014                           Université de Bordeaux
- * Copyright (C) 2010,2015                                CNRS
+ * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +17,8 @@
 #ifndef __FILTERS_H__
 #define __FILTERS_H__
 
+/** @file */
+
 #include <stdarg.h>
 #include <datawizard/coherency.h>
 #include <datawizard/memalloc.h>
@@ -26,9 +26,11 @@
 #include <starpu.h>
 #include <common/config.h>
 
-void
-_starpu_filter_nparts_compute_chunk_size_and_offset(unsigned n, unsigned nparts,
-					     size_t elemsize, unsigned id,
-					     unsigned ld, unsigned *chunk_size,
-					     size_t *offset);
+#pragma GCC visibility push(hidden)
+
+/** submit asynchronous unpartitioning / partitioning to make target active read-only or read-write */
+void _starpu_data_partition_access_submit(starpu_data_handle_t target, int write);
+
+#pragma GCC visibility pop
+
 #endif
