@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -439,6 +439,7 @@ void starpu_data_register_ops(struct starpu_data_interface_ops *ops)
 
 void starpu_data_register(starpu_data_handle_t *handleptr, int home_node, void *data_interface, struct starpu_data_interface_ops *ops)
 {
+	STARPU_ASSERT_MSG(home_node >= -1 && home_node < (int)starpu_memory_nodes_get_count(), "Invalid memory node number");
 	starpu_data_handle_t handle = _starpu_data_handle_allocate(ops, home_node);
 
 	STARPU_ASSERT(handleptr);
@@ -929,14 +930,8 @@ void starpu_data_unregister_submit(starpu_data_handle_t handle)
 	starpu_data_acquire_on_node_cb(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL, handle->initialized?STARPU_RW:STARPU_W, _starpu_data_unregister_submit_cb, handle);
 }
 
-static void _starpu_data_invalidate(void *data)
+static void __starpu_data_deinitialize(starpu_data_handle_t handle)
 {
-	starpu_data_handle_t handle = data;
-	size_t size = _starpu_data_get_alloc_size(handle);
-	_starpu_spin_lock(&handle->header_lock);
-
-	//_STARPU_DEBUG("Really invalidating data %p\n", data);
-
 #ifdef STARPU_DEBUG
 	{
 		/* There shouldn't be any pending request since we acquired the data in W mode */
@@ -949,6 +944,43 @@ static void _starpu_data_invalidate(void *data)
 
 	unsigned node;
 
+	for (node = 0; node < STARPU_MAXNODES; node++)
+	{
+		struct _starpu_data_replicate *local = &handle->per_node[node];
+
+		if (local->state != STARPU_INVALID)
+			_STARPU_TRACE_DATA_STATE_INVALID(handle, node);
+		local->state = STARPU_INVALID;
+		local->initialized = 0;
+	}
+
+	if (handle->per_worker)
+	{
+		unsigned worker;
+		unsigned nworkers = starpu_worker_get_count();
+		for (worker = 0; worker < nworkers; worker++)
+		{
+			struct _starpu_data_replicate *local = &handle->per_worker[worker];
+
+			local->state = STARPU_INVALID;
+		}
+	}
+}
+
+
+static void _starpu_data_invalidate(void *data)
+{
+	starpu_data_handle_t handle = data;
+	size_t size = _starpu_data_get_alloc_size(handle);
+
+	_starpu_spin_lock(&handle->header_lock);
+
+	//_STARPU_DEBUG("Really invalidating data %p\n", data);
+
+	__starpu_data_deinitialize(handle);
+
+	unsigned node;
+
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct _starpu_data_replicate *local = &handle->per_node[node];
@@ -966,11 +998,6 @@ static void _starpu_data_invalidate(void *data)
 				_starpu_request_mem_chunk_removal(handle, local, node, size);
 			}
 		}
-
-		if (local->state != STARPU_INVALID)
-			_STARPU_TRACE_DATA_STATE_INVALID(handle, node);
-		local->state = STARPU_INVALID;
-		local->initialized = 0;
 	}
 
 	if (handle->per_worker)
@@ -984,8 +1011,6 @@ static void _starpu_data_invalidate(void *data)
 			if (local->mc && local->allocated && local->automatically_allocated)
 				/* free the data copy in a lazy fashion */
 				_starpu_request_mem_chunk_removal(handle, local, starpu_worker_get_memory_node(worker), size);
-
-			local->state = STARPU_INVALID;
 		}
 	}
 
@@ -994,6 +1019,75 @@ static void _starpu_data_invalidate(void *data)
 	starpu_data_release_on_node(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL);
 }
 
+static void _starpu_data_deinitialize(void *data)
+{
+	starpu_data_handle_t handle = data;
+
+	_starpu_spin_lock(&handle->header_lock);
+
+	//_STARPU_DEBUG("Really deinitializing data %p\n", data);
+
+	__starpu_data_deinitialize(handle);
+
+	unsigned node;
+
+	for (node = 0; node < STARPU_MAXNODES; node++)
+	{
+		struct _starpu_data_replicate *local = &handle->per_node[node];
+
+		if (local->mc && local->allocated && local->automatically_allocated)
+			/* note that the data is now clean */
+			_starpu_memchunk_clean(local->mc, node);
+	}
+
+	if (handle->per_worker)
+	{
+		unsigned worker;
+		unsigned nworkers = starpu_worker_get_count();
+		for (worker = 0; worker < nworkers; worker++)
+		{
+			struct _starpu_data_replicate *local = &handle->per_worker[worker];
+
+			if (local->mc && local->allocated && local->automatically_allocated)
+				/* note that the data is now clean */
+				_starpu_memchunk_clean(local->mc, starpu_worker_get_memory_node(worker));
+		}
+	}
+
+	_starpu_spin_unlock(&handle->header_lock);
+
+	starpu_data_release_on_node(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL);
+}
+
+void starpu_data_deinitialize(starpu_data_handle_t handle)
+{
+	STARPU_ASSERT(handle);
+
+	starpu_data_acquire_on_node(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL, STARPU_W);
+
+	_starpu_data_deinitialize(handle);
+
+	handle->initialized = 0;
+}
+
+void starpu_data_deinitialize_submit(starpu_data_handle_t handle)
+{
+	STARPU_ASSERT(handle);
+
+	starpu_data_acquire_on_node_cb(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL, STARPU_W, _starpu_data_deinitialize, handle);
+
+	handle->initialized = 0;
+}
+
+void _starpu_data_deinitialize_submit_noplan(starpu_data_handle_t handle)
+{
+	STARPU_ASSERT(handle);
+
+	starpu_data_acquire_on_node_cb(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL, STARPU_W | STARPU_NOPLAN, _starpu_data_deinitialize, handle);
+
+	handle->initialized = 0;
+}
+
 void starpu_data_invalidate(starpu_data_handle_t handle)
 {
 	STARPU_ASSERT(handle);
