@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2014  Université de Bordeaux
- * Copyright (C) 2010  CNRS
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,6 +17,8 @@
 #ifndef __SORT_DATA_HANDLES_H__
 #define __SORT_DATA_HANDLES_H__
 
+/** @file */
+
 #include <starpu.h>
 #include <common/config.h>
 #include <stdlib.h>
@@ -27,9 +28,19 @@
 #include <datawizard/coherency.h>
 #include <datawizard/memalloc.h>
 
-/* To avoid deadlocks, we reorder the different buffers accessed to by the task
+#pragma GCC visibility push(hidden)
+
+/** To avoid deadlocks, we reorder the different buffers accessed to by the task
  * so that we always grab the rw-lock associated to the handles in the same
  * order. */
 void _starpu_sort_task_handles(struct _starpu_data_descr descr[], unsigned nbuffers);
 
+/** The reordering however puts alongside some different handles, just because
+ * they have the same root. When avoiding to lock/acquire/load the same handle
+ * several times, we need to keep looking among those.
+ */
+int _starpu_handles_same_root(starpu_data_handle_t dataA, starpu_data_handle_t dataB);
+
+#pragma GCC visibility pop
+
 #endif // SORT_DATA_HANDLES
