@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,6 +23,8 @@
 #include <starpu_bound.h>
 #include <core/jobs.h>
 
+#pragma GCC visibility push(hidden)
+
 /** Are we recording? */
 extern int _starpu_bound_recording;
 
@@ -41,4 +43,6 @@ extern void _starpu_bound_job_id_dep(starpu_data_handle_t handle, struct _starpu
 /** Clear recording */
 extern void starpu_bound_clear(void);
 
+#pragma GCC visibility pop
+
 #endif // __BOUND_H__
