@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2020       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -71,6 +71,7 @@ int _starpu_profiling =
 	;
 
 int _starpu_codelet_profiling = 1;
+int _starpu_energy_profiling = 0;
 
 void starpu_profiling_init()
 {
@@ -254,6 +255,7 @@ void _starpu_profiling_start(void)
 		starpu_profiling_status_set(STARPU_PROFILING_ENABLE);
 	}
 	_starpu_codelet_profiling = starpu_get_env_number_default("STARPU_CODELET_PROFILING", 1);
+	_starpu_energy_profiling = starpu_get_env_number_default("STARPU_ENERGY_PROFILING", 0);
 }
 
 void _starpu_profiling_terminate(void)
