@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Simon Archipoff
+ * Copyright (C) 2020       Télécom-Sud Paris
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,12 +20,13 @@
  * GPUs take accelerated tasks first and CPUs take non-accelerated tasks first */
 
 #include <starpu_sched_component.h>
-#include "prio_deque.h"
 #include <starpu_perfmodel.h>
+#include <schedulers/starpu_scheduler_toolbox.h>
 #include "helper_mct.h"
 #include <float.h>
 #include <core/sched_policy.h>
 #include <core/task.h>
+#include <sched_policies/prio_deque.h>
 
 /* Approximation ratio for acceleration factor bucketing
  * We will put tasks with +-10% similar acceleration into the same bucket. */
@@ -34,12 +36,12 @@ struct _starpu_heteroprio_data
 {
 	/* This is an array of priority queues.
 	 * The array is sorted by acceleration factor, most accelerated first */
-	struct _starpu_prio_deque **bucket;
+	struct starpu_st_prio_deque **bucket;
 	float *accel;
 	unsigned naccel;
 
 	/* This contains tasks which are not supported on all archs. */
-	struct _starpu_prio_deque no_accel;
+	struct starpu_st_prio_deque no_accel;
 
 	/* This protects all queues */
 	starpu_pthread_mutex_t mutex;
@@ -63,7 +65,7 @@ static int heteroprio_progress_accel(struct starpu_sched_component *component, s
 		/* Pick up accelerated tasks first */
 		for (j = 0; j < (int) data->naccel; j++)
 		{
-			task = _starpu_prio_deque_pop_task(data->bucket[j]);
+			task = starpu_st_prio_deque_pop_task(data->bucket[j]);
 			if (task)
 				break;
 		}
@@ -72,9 +74,9 @@ static int heteroprio_progress_accel(struct starpu_sched_component *component, s
 		for (j = (int) data->naccel-1; j >= 0; j--)
 		{
 			if (data->batch && 0)
-				task = _starpu_prio_deque_pop_back_task(data->bucket[j]);
+				task = starpu_st_prio_deque_pop_back_task(data->bucket[j]);
 			else
-				task = _starpu_prio_deque_pop_task(data->bucket[j]);
+				task = starpu_st_prio_deque_pop_task(data->bucket[j]);
 			if (task)
 				break;
 		}
@@ -106,10 +108,13 @@ static int heteroprio_progress_accel(struct starpu_sched_component *component, s
 	/* Estimated transfer+task termination for each child */
 	double estimated_ends_with_task[component->nchildren];
 
-	/* Minimum transfer+task termination on all children */
-	double min_exp_end_with_task;
-	/* Maximum transfer+task termination on all children */
-	double max_exp_end_with_task;
+	/* provided local energy */
+	double local_energy[component->nchildren];
+
+	/* Minimum transfer+task termination of the task over all workers */
+	double min_exp_end_of_task;
+	/* Maximum termination of the already-scheduled tasks over all workers */
+	double max_exp_end_of_workers;
 
 	unsigned suitable_components[component->nchildren];
 	unsigned nsuitable_components;
@@ -128,9 +133,9 @@ static int heteroprio_progress_accel(struct starpu_sched_component *component, s
 		for (i = 0; i < component->nchildren; i++)
 		{
 			int idworker;
-			for(idworker = starpu_bitmap_first(component->children[i]->workers);
+			for(idworker = starpu_bitmap_first(&component->children[i]->workers);
 				idworker != -1;
-				idworker = starpu_bitmap_next(component->children[i]->workers, idworker))
+				idworker = starpu_bitmap_next(&component->children[i]->workers, idworker))
 			{
 				if (starpu_worker_get_type(idworker) == archtype)
 					break;
@@ -138,7 +143,7 @@ static int heteroprio_progress_accel(struct starpu_sched_component *component, s
 
 			if (idworker == -1)
 			{
-				/* Not the targetted arch, avoid it */
+				/* Not the targeted arch, avoid it */
 
 				/* XXX: INFINITY doesn't seem to be working properly */
 				estimated_lengths[i] = 1000000000;
@@ -155,27 +160,40 @@ static int heteroprio_progress_accel(struct starpu_sched_component *component, s
 			estimated_lengths,
 			estimated_transfer_length,
 			estimated_ends_with_task,
-			&min_exp_end_with_task, &max_exp_end_with_task,
+			&min_exp_end_of_task, &max_exp_end_of_workers,
 			suitable_components, nsuitable_components);
 
+	/* Compute the energy, if provided*/
+	starpu_mct_compute_energy(component, task, local_energy, suitable_components, nsuitable_components);
+
 	/* And now find out which worker suits best for this task,
 	 * including data transfer */
-	int best_icomponent = starpu_mct_get_best_component(d, task,
-			estimated_lengths,
-			estimated_transfer_length,
-			estimated_ends_with_task,
-			min_exp_end_with_task, max_exp_end_with_task,
-			suitable_components, nsuitable_components);
 
-	if (best_icomponent == -1)
-		goto out;
+	int best_icomponent = starpu_mct_get_best_component(d, task,
+							    estimated_lengths,
+							    estimated_transfer_length,
+							    estimated_ends_with_task,
+							    local_energy,
+							    min_exp_end_of_task, max_exp_end_of_workers,
+							    suitable_components, nsuitable_components);
+
+	/* If no best component is found, it means that the perfmodel of
+	 * the task had been purged since it has been pushed on the mct component. */
+	/* FIXME: We should perform a push_back message to its parent so that it will
+	 * be able to reschedule the task properly. */
+	if(best_icomponent == -1)
+	{
+		STARPU_COMPONENT_MUTEX_UNLOCK(&d->scheduling_mutex);
+		return eager_calibration_push_task(component, task);
+	}
+
 
 	best_component = component->children[best_icomponent];
 
 	int idworker;
-	for(idworker = starpu_bitmap_first(best_component->workers);
+	for(idworker = starpu_bitmap_first(&best_component->workers);
 		idworker != -1;
-		idworker = starpu_bitmap_next(best_component->workers, idworker))
+		idworker = starpu_bitmap_next(&best_component->workers, idworker))
 	{
 		if (starpu_worker_get_type(idworker) == archtype)
 			break;
@@ -210,7 +228,7 @@ out:
 	{
 		if (acceleration == data->accel[j])
 		{
-			_starpu_prio_deque_push_front_task(data->bucket[j], task);
+			starpu_st_prio_deque_push_front_task(data->bucket[j], task);
 			break;
 		}
 	}
@@ -236,10 +254,13 @@ static int heteroprio_progress_noaccel(struct starpu_sched_component *component,
 	/* Estimated transfer+task termination for each child */
 	double estimated_ends_with_task[component->nchildren];
 
-	/* Minimum transfer+task termination on all children */
-	double min_exp_end_with_task;
-	/* Maximum transfer+task termination on all children */
-	double max_exp_end_with_task;
+	/* estimated energy */
+	double local_energy[component->nchildren];
+
+	/* Minimum transfer+task termination of the task over all workers */
+	double min_exp_end_of_task;
+	/* Maximum termination of the already-scheduled tasks over all workers */
+	double max_exp_end_of_workers;
 
 	unsigned suitable_components[component->nchildren];
 	unsigned nsuitable_components;
@@ -250,11 +271,11 @@ static int heteroprio_progress_noaccel(struct starpu_sched_component *component,
 			suitable_components);
 
 	/* If no suitable components were found, it means that the perfmodel of
-	 * the task had been purged since it has been pushed on the mct component.
-	 * We should send a push_fail message to its parent so that it will
+	 * the task had been purged since it has been pushed on the mct component. */
+	/* FIXME: We should perform a push_back message to its parent so that it will
 	 * be able to reschedule the task properly. */
 	if(nsuitable_components == 0)
-		return 1;
+		return eager_calibration_push_task(component, task);
 
 	/* Entering critical section to make sure no two workers
 	   make scheduling decisions at the same time */
@@ -264,26 +285,31 @@ static int heteroprio_progress_noaccel(struct starpu_sched_component *component,
 			estimated_lengths,
 			estimated_transfer_length,
 			estimated_ends_with_task,
-			&min_exp_end_with_task, &max_exp_end_with_task,
+			&min_exp_end_of_task, &max_exp_end_of_workers,
 			suitable_components, nsuitable_components);
 
+	/* Compute the energy, if provided*/
+	starpu_mct_compute_energy(component, task, local_energy, suitable_components, nsuitable_components);
+
 	/* And now find out which worker suits best for this task,
 	 * including data transfer */
+
 	int best_icomponent = starpu_mct_get_best_component(d, task,
-			estimated_lengths,
-			estimated_transfer_length,
-			estimated_ends_with_task,
-			min_exp_end_with_task, max_exp_end_with_task,
-			suitable_components, nsuitable_components);
+							    estimated_lengths,
+							    estimated_transfer_length,
+							    estimated_ends_with_task,
+							    local_energy,
+							    min_exp_end_of_task, max_exp_end_of_workers,
+							    suitable_components, nsuitable_components);
 
 	/* If no best component is found, it means that the perfmodel of
-	 * the task had been purged since it has been pushed on the mct component.
-	 * We should send a push_fail message to its parent so that it will
+	 * the task had been purged since it has been pushed on the mct component. */
+	/* FIXME: We should perform a push_back message to its parent so that it will
 	 * be able to reschedule the task properly. */
 	if(best_icomponent == -1)
 	{
 		STARPU_COMPONENT_MUTEX_UNLOCK(&d->scheduling_mutex);
-		return 1;
+		return eager_calibration_push_task(component, task);
 	}
 
 	best_component = component->children[best_icomponent];
@@ -301,9 +327,9 @@ static int heteroprio_progress_one(struct starpu_sched_component *component)
 	starpu_pthread_mutex_t * mutex = &data->mutex;
 	struct starpu_task *task;
 
-	struct _starpu_prio_deque * no_accel = &data->no_accel;
+	struct starpu_st_prio_deque * no_accel = &data->no_accel;
 	STARPU_COMPONENT_MUTEX_LOCK(mutex);
-	task = _starpu_prio_deque_pop_task(no_accel);
+	task = starpu_st_prio_deque_pop_task(no_accel);
 	STARPU_COMPONENT_MUTEX_UNLOCK(mutex);
 
 	if (task)
@@ -312,7 +338,7 @@ static int heteroprio_progress_one(struct starpu_sched_component *component)
 		{
 			/* Could not push to child actually, push that one back */
 			STARPU_COMPONENT_MUTEX_LOCK(mutex);
-			_starpu_prio_deque_push_front_task(no_accel, task);
+			starpu_st_prio_deque_push_front_task(no_accel, task);
 			STARPU_COMPONENT_MUTEX_UNLOCK(mutex);
 		}
 	}
@@ -320,9 +346,9 @@ static int heteroprio_progress_one(struct starpu_sched_component *component)
 	/* Note: this hardcodes acceleration order */
 	if (!heteroprio_progress_accel(component, data, STARPU_CUDA_WORKER, 1))
 		return 0;
-	if (!heteroprio_progress_accel(component, data, STARPU_OPENCL_WORKER, 1))
+	if (!heteroprio_progress_accel(component, data, STARPU_HIP_WORKER, 1))
 		return 0;
-	if (!heteroprio_progress_accel(component, data, STARPU_MIC_WORKER, 1))
+	if (!heteroprio_progress_accel(component, data, STARPU_OPENCL_WORKER, 1))
 		return 0;
 	if (!heteroprio_progress_accel(component, data, STARPU_MPI_MS_WORKER, 0))
 		return 0;
@@ -356,9 +382,9 @@ static int heteroprio_push_task(struct starpu_sched_component * component, struc
 
 	/* Compute acceleration between best-performing arch and least-performing arch */
 	int workerid;
-	for(workerid = starpu_bitmap_first(component->workers_in_ctx);
+	for(workerid = starpu_bitmap_first(&component->workers_in_ctx);
 	    workerid != -1;
-	    workerid = starpu_bitmap_next(component->workers_in_ctx, workerid))
+	    workerid = starpu_bitmap_next(&component->workers_in_ctx, workerid))
 	{
 		unsigned impl_mask;
 		if (!starpu_worker_can_execute_task_impl(workerid, task, &impl_mask))
@@ -417,10 +443,13 @@ static int heteroprio_push_task(struct starpu_sched_component * component, struc
 			/* Didn't find it, add one */
 			data->naccel++;
 
-			float *newaccel = malloc(data->naccel * sizeof(*newaccel));
-			struct _starpu_prio_deque **newbuckets = malloc(data->naccel * sizeof(*newbuckets));
-			struct _starpu_prio_deque *newbucket = malloc(sizeof(*newbucket));
-			_starpu_prio_deque_init(newbucket);
+			float *newaccel;
+			_STARPU_MALLOC(newaccel, data->naccel * sizeof(*newaccel));
+			struct starpu_st_prio_deque **newbuckets;
+			_STARPU_MALLOC(newbuckets, data->naccel * sizeof(*newbuckets));
+			struct starpu_st_prio_deque *newbucket;
+			_STARPU_MALLOC(newbucket, sizeof(*newbucket));
+			starpu_st_prio_deque_init(newbucket);
 			int inserted = 0;
 
 			for (j = 0; j < data->naccel-1; j++)
@@ -455,7 +484,7 @@ static int heteroprio_push_task(struct starpu_sched_component * component, struc
 		}
 		fprintf(stderr,"\ninserting %p %f to %d\n", task, acceleration, i);
 #endif
-		_starpu_prio_deque_push_back_task(data->bucket[i],task);
+		starpu_st_prio_deque_push_back_task(data->bucket[i],task);
 		STARPU_COMPONENT_MUTEX_UNLOCK(mutex);
 	}
 	else
@@ -463,9 +492,9 @@ static int heteroprio_push_task(struct starpu_sched_component * component, struc
 		/* Not all archs can run it, will resort to HEFT strategy */
 		acceleration = INFINITY;
 		//fprintf(stderr,"%s: some archs can't do it\n", starpu_task_get_name(task));
-		struct _starpu_prio_deque * no_accel = &data->no_accel;
+		struct starpu_st_prio_deque * no_accel = &data->no_accel;
 		STARPU_COMPONENT_MUTEX_LOCK(mutex);
-		_starpu_prio_deque_push_back_task(no_accel,task);
+		starpu_st_prio_deque_push_back_task(no_accel,task);
 		STARPU_COMPONENT_MUTEX_UNLOCK(mutex);
 	}
 
@@ -501,12 +530,12 @@ static void heteroprio_component_deinit_data(struct starpu_sched_component * com
 	unsigned i;
 	for (i = 0; i < d->naccel; i++)
 	{
-		_starpu_prio_deque_destroy(d->bucket[i]);
+		starpu_st_prio_deque_destroy(d->bucket[i]);
 		free(d->bucket[i]);
 	}
 	free(d->bucket);
 	free(d->accel);
-	_starpu_prio_deque_destroy(&d->no_accel);
+	starpu_st_prio_deque_destroy(&d->no_accel);
 	STARPU_PTHREAD_MUTEX_DESTROY(&d->mutex);
 	STARPU_PTHREAD_MUTEX_DESTROY(&mct_d->scheduling_mutex);
 	free(mct_d);
@@ -528,7 +557,7 @@ struct starpu_sched_component * starpu_sched_component_heteroprio_create(struct
 	data->bucket = NULL;
 	data->accel = NULL;
 	data->naccel = 0;
-	_starpu_prio_deque_init(&data->no_accel);
+	starpu_st_prio_deque_init(&data->no_accel);
 	STARPU_PTHREAD_MUTEX_INIT(&data->mutex,NULL);
 	data->mct_data = mct_data;
 	STARPU_PTHREAD_MUTEX_INIT(&mct_data->scheduling_mutex,NULL);
