@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013,2014,2017                           Inria
- * Copyright (C) 2014-2017                                CNRS
- * Copyright (C) 2014-2019                                Université de Bordeaux
- * Copyright (C) 2013                                     Simon Archipoff
+ * Copyright (C) 2013-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Simon Archipoff
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,21 +19,21 @@
 #include <core/workers.h>
 #include <starpu_sched_component.h>
 #include <starpu_thread_util.h>
+#include <datawizard/memory_nodes.h>
 
 #include <float.h>
 
 #include "sched_component.h"
 
-
-
 /******************************************************************************
- *				Generic Scheduling Components' helper functions        		  *
+ *	 Generic Scheduling Components' helper functions        	      *
  ******************************************************************************/
 
-
-
-/* this function find the best implementation or an implementation that need to be calibrated for a worker available
- * and set prediction in *length. nan if a implementation need to be calibrated, 0.0 if no perf model are available
+/*
+ * this function find the best implementation or an implementation
+ * that need to be calibrated for a worker available and set
+ * prediction in *length. nan if a implementation need to be
+ * calibrated, 0.0 if no perf model are available
  * return false if no worker on the component can execute that task
  */
 int starpu_sched_component_execute_preds(struct starpu_sched_component * component, struct starpu_task * task, double * length)
@@ -45,13 +43,11 @@ int starpu_sched_component_execute_preds(struct starpu_sched_component * compone
 	starpu_task_bundle_t bundle = task->bundle;
 	double len = DBL_MAX;
 
-
 	int workerid;
-	for(workerid = starpu_bitmap_first(component->workers_in_ctx);
+	for(workerid = starpu_bitmap_first(&component->workers_in_ctx);
 	    workerid != -1;
-	    workerid = starpu_bitmap_next(component->workers_in_ctx, workerid))
+	    workerid = starpu_bitmap_next(&component->workers_in_ctx, workerid))
 	{
-		struct starpu_perfmodel_arch* archtype = starpu_worker_get_perf_archtype(workerid, component->tree->sched_ctx_id);
 		int nimpl;
 		for(nimpl = 0; nimpl < STARPU_MAXIMPLEMENTATIONS; nimpl++)
 		{
@@ -61,9 +57,13 @@ int starpu_sched_component_execute_preds(struct starpu_sched_component * compone
 				double d;
 				can_execute = 1;
 				if(bundle)
+				{
+					struct starpu_perfmodel_arch* archtype =
+						starpu_worker_get_perf_archtype(workerid, component->tree->sched_ctx_id);
 					d = starpu_task_bundle_expected_length(bundle, archtype, nimpl);
+				}
 				else
-					d = starpu_task_expected_length(task, archtype, nimpl);
+					d = starpu_task_worker_expected_length(task, workerid, component->tree->sched_ctx_id, nimpl);
 				if(isnan(d))
 				{
 					*length = d;
@@ -99,12 +99,25 @@ int starpu_sched_component_can_execute_task(struct starpu_sched_component * comp
 	unsigned nimpl;
 	int worker;
 	for (nimpl = 0; nimpl < STARPU_MAXIMPLEMENTATIONS; nimpl++)
-		for(worker = starpu_bitmap_first(component->workers_in_ctx);
+		for(worker = starpu_bitmap_first(&component->workers_in_ctx);
 		    -1 != worker;
-		    worker = starpu_bitmap_next(component->workers_in_ctx, worker))
+		    worker = starpu_bitmap_next(&component->workers_in_ctx, worker))
 			if (starpu_worker_can_execute_task(worker, task, nimpl)
-			     || starpu_combined_worker_can_execute_task(worker, task, nimpl))
-			    return 1;
+			    || starpu_combined_worker_can_execute_task(worker, task, nimpl))
+			{
+				if (starpu_worker_can_execute_task(worker, task, nimpl) == 1)
+					_STARPU_EXTRA_DEBUG("worker %d CAN execute task %s with impl %d\n", worker, starpu_task_get_name(task), nimpl);
+				if (starpu_combined_worker_can_execute_task(worker, task, nimpl) == 1)
+					_STARPU_EXTRA_DEBUG("combined worker %d CAN execute task %s with impl %d\n", worker, starpu_task_get_name(task), nimpl);
+				return 1;
+			}
+			else
+			{
+				if (starpu_worker_can_execute_task(worker, task, nimpl) == 0)
+					_STARPU_EXTRA_DEBUG("worker %d CANNOT execute task %s with impl %d\n", worker, starpu_task_get_name(task), nimpl);
+				if (starpu_combined_worker_can_execute_task(worker, task, nimpl) == 0)
+					_STARPU_EXTRA_DEBUG("combined worker %d CANNOT execute task %s with impl %d\n", worker, starpu_task_get_name(task), nimpl);
+			}
 	return 0;
 }
 
@@ -114,21 +127,21 @@ int starpu_sched_component_can_execute_task(struct starpu_sched_component * comp
 double starpu_sched_component_transfer_length(struct starpu_sched_component * component, struct starpu_task * task)
 {
 	STARPU_ASSERT(component && task);
-	int nworkers = starpu_bitmap_cardinal(component->workers_in_ctx);
+	int nworkers = starpu_bitmap_cardinal(&component->workers_in_ctx);
 	double sum = 0.0;
 	int worker;
 	if(STARPU_SCHED_COMPONENT_IS_SINGLE_MEMORY_NODE(component))
 	{
-		unsigned memory_node  = starpu_worker_get_memory_node(starpu_bitmap_first(component->workers_in_ctx));
+		unsigned memory_node  = starpu_worker_get_memory_node(starpu_bitmap_first(&component->workers_in_ctx));
 		if(task->bundle)
 			return starpu_task_bundle_expected_data_transfer_time(task->bundle,memory_node);
 		else
 			return starpu_task_expected_data_transfer_time(memory_node, task);
 	}
 
-	for(worker = starpu_bitmap_first(component->workers_in_ctx);
+	for(worker = starpu_bitmap_first(&component->workers_in_ctx);
 	    worker != -1;
-	    worker = starpu_bitmap_next(component->workers_in_ctx, worker))
+	    worker = starpu_bitmap_next(&component->workers_in_ctx, worker))
 	{
 		unsigned memory_node  = starpu_worker_get_memory_node(worker);
 		if(task->bundle)
@@ -155,10 +168,9 @@ void starpu_sched_component_prefetch_on_node(struct starpu_sched_component * com
 	if (starpu_get_prefetch_flag() && (!task->prefetched)
 		&& (component->properties & STARPU_SCHED_COMPONENT_SINGLE_MEMORY_NODE))
 	{
-		int worker = starpu_bitmap_first(component->workers_in_ctx);
+		int worker = starpu_bitmap_first(&component->workers_in_ctx);
 		unsigned memory_node = starpu_worker_get_memory_node(worker);
 		starpu_prefetch_task_input_on_node(task, memory_node);
-		task->prefetched = 1;
 	}
 }
 
@@ -194,8 +206,6 @@ void starpu_sched_component_destroy(struct starpu_sched_component *component)
 	free(component->children);
 	free(component->parents);
 	free(component->name);
-	starpu_bitmap_destroy(component->workers);
-	starpu_bitmap_destroy(component->workers_in_ctx);
 	free(component);
 }
 
@@ -217,12 +227,12 @@ void starpu_sched_component_destroy_rec(struct starpu_sched_component * componen
 		starpu_sched_component_destroy(component);
 }
 
-void set_properties(struct starpu_sched_component * component)
+static void set_properties(struct starpu_sched_component * component)
 {
 	STARPU_ASSERT(component);
 	component->properties = 0;
 
-	int worker = starpu_bitmap_first(component->workers_in_ctx);
+	int worker = starpu_bitmap_first(&component->workers_in_ctx);
 	if (worker == -1)
 		return;
 	if (starpu_worker_is_combined_worker(worker))
@@ -236,7 +246,7 @@ void set_properties(struct starpu_sched_component * component)
 	int is_all_same_component = 1;
 	for(;
 	    worker != -1;
-	    worker = starpu_bitmap_next(component->workers_in_ctx, worker))
+	    worker = starpu_bitmap_next(&component->workers_in_ctx, worker))
 	{
 		if(starpu_worker_is_combined_worker(worker))
 			continue;
@@ -261,12 +271,12 @@ void _starpu_sched_component_update_workers(struct starpu_sched_component * comp
 	STARPU_ASSERT(component);
 	if(starpu_sched_component_is_worker(component))
 		return;
-	starpu_bitmap_unset_all(component->workers);
+	starpu_bitmap_unset_all(&component->workers);
 	unsigned i;
 	for(i = 0; i < component->nchildren; i++)
 	{
 		_starpu_sched_component_update_workers(component->children[i]);
-		starpu_bitmap_or(component->workers, component->children[i]->workers);
+		starpu_bitmap_or(&component->workers, &component->children[i]->workers);
 	}
 	component->notify_change_workers(component);
 }
@@ -281,10 +291,12 @@ void _starpu_sched_component_update_workers_in_ctx(struct starpu_sched_component
 	if(starpu_sched_component_is_worker(component))
 		return;
 	struct starpu_bitmap * workers_in_ctx = _starpu_get_worker_mask(sched_ctx_id);
-	starpu_bitmap_unset_and(component->workers_in_ctx,component->workers, workers_in_ctx);
+	starpu_bitmap_unset_and(&component->workers_in_ctx,&component->workers, workers_in_ctx);
 	unsigned i,j;
-	for(i = starpu_worker_get_count(); i < starpu_worker_get_count() + starpu_combined_worker_get_count(); i++) {
-		if (starpu_bitmap_get(component->workers, i)) {
+	for(i = starpu_worker_get_count(); i < starpu_worker_get_count() + starpu_combined_worker_get_count(); i++)
+	{
+		if (starpu_bitmap_get(&component->workers, i))
+		{
 			/* Component has this combined worker, check whether the
 			 * context has all the corresponding workers */
 			int worker_size;
@@ -294,7 +306,7 @@ void _starpu_sched_component_update_workers_in_ctx(struct starpu_sched_component
 				if (!starpu_bitmap_get(workers_in_ctx, combined_workerid[j]))
 					goto nocombined;
 			/* We have all workers, add it */
-			starpu_bitmap_set(component->workers_in_ctx, i);
+			starpu_bitmap_set(&component->workers_in_ctx, i);
 		}
 nocombined:
 		(void)0;
@@ -321,7 +333,7 @@ struct starpu_bitmap * _starpu_get_worker_mask(unsigned sched_ctx_id)
 	STARPU_ASSERT(sched_ctx_id < STARPU_NMAX_SCHED_CTXS);
 	struct starpu_sched_tree * t = starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	STARPU_ASSERT(t);
-	return t->workers;
+	return &t->workers;
 }
 
 void starpu_sched_tree_update_workers_in_ctx(struct starpu_sched_tree * t)
@@ -369,9 +381,10 @@ int starpu_sched_tree_push_task(struct starpu_task * task)
 int starpu_sched_component_push_task(struct starpu_sched_component *from STARPU_ATTRIBUTE_UNUSED, struct starpu_sched_component *to, struct starpu_task *task)
 {
 	int pushback;
+	int priority = task->priority;
 	pushback = to->push_task(to, task);
 	if (!pushback)
-		_STARPU_TRACE_SCHED_COMPONENT_PUSH(from, to, task);
+		_STARPU_TRACE_SCHED_COMPONENT_PUSH(from, to, task, priority);
 	return pushback;
 }
 
@@ -395,8 +408,11 @@ struct starpu_task * starpu_sched_component_pull_task(struct starpu_sched_compon
 }
 
 
-/* Pump mechanic to get the task flow rolling. Takes tasks from component and send them to the child.
-   To be used by components with only one child */
+/*
+ * Pump mechanic to get the task flow rolling. Takes tasks from
+ * component and send them to the child.
+ * To be used by components with only one child
+*/
 struct starpu_task* starpu_sched_component_pump_to(struct starpu_sched_component *component, struct starpu_sched_component *child, int* success)
 {
 	int ret = 0;
@@ -439,7 +455,7 @@ void starpu_sched_tree_add_workers(unsigned sched_ctx_id, int *workerids, unsign
 
 	unsigned i;
 	for(i = 0; i < nworkers; i++)
-		starpu_bitmap_set(t->workers, workerids[i]);
+		starpu_bitmap_set(&t->workers, workerids[i]);
 
 	starpu_sched_tree_update_workers_in_ctx(t);
 
@@ -458,7 +474,7 @@ void starpu_sched_tree_remove_workers(unsigned sched_ctx_id, int *workerids, uns
 
 	unsigned i;
 	for(i = 0; i < nworkers; i++)
-		starpu_bitmap_unset(t->workers, workerids[i]);
+		starpu_bitmap_unset(&t->workers, workerids[i]);
 
 	starpu_sched_tree_update_workers_in_ctx(t);
 
@@ -466,6 +482,26 @@ void starpu_sched_tree_remove_workers(unsigned sched_ctx_id, int *workerids, uns
 	STARPU_COMPONENT_MUTEX_UNLOCK(&t->lock);
 }
 
+static void _starpu_sched_tree_do_schedule(struct starpu_sched_component *component)
+{
+	unsigned i;
+
+	if (component->do_schedule)
+		component->do_schedule(component);
+
+	for (i = 0; i < component->nchildren; i++)
+		_starpu_sched_tree_do_schedule(component->children[i]);
+}
+
+void starpu_sched_tree_do_schedule(unsigned sched_ctx_id)
+{
+	STARPU_ASSERT(sched_ctx_id < STARPU_NMAX_SCHED_CTXS);
+	struct starpu_sched_tree * t = starpu_sched_ctx_get_policy_data(sched_ctx_id);
+
+	if (t->root)
+		_starpu_sched_tree_do_schedule(t->root);
+}
+
 static struct starpu_sched_tree *trees[STARPU_NMAX_SCHED_CTXS];
 
 struct starpu_sched_tree * starpu_sched_tree_create(unsigned sched_ctx_id)
@@ -475,7 +511,7 @@ struct starpu_sched_tree * starpu_sched_tree_create(unsigned sched_ctx_id)
 	struct starpu_sched_tree *t;
 	_STARPU_CALLOC(t, 1, sizeof(*t));
 	t->sched_ctx_id = sched_ctx_id;
-	t->workers = starpu_bitmap_create();
+	starpu_bitmap_init(&t->workers);
 	STARPU_PTHREAD_MUTEX_INIT(&t->lock,NULL);
 	trees[sched_ctx_id] = t;
 	return t;
@@ -488,7 +524,6 @@ void starpu_sched_tree_destroy(struct starpu_sched_tree * tree)
 	trees[tree->sched_ctx_id] = NULL;
 	if(tree->root)
 		starpu_sched_component_destroy_rec(tree->root);
-	starpu_bitmap_destroy(tree->workers);
 	STARPU_PTHREAD_MUTEX_DESTROY(&tree->lock);
 	free(tree);
 }
@@ -498,6 +533,13 @@ struct starpu_sched_tree * starpu_sched_tree_get(unsigned sched_ctx_id)
 	return trees[sched_ctx_id];
 }
 
+void starpu_sched_tree_deinitialize(unsigned sched_ctx_id)
+{
+	struct starpu_sched_tree *t = (struct starpu_sched_tree*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
+
+	starpu_sched_tree_destroy(t);
+}
+
 
 /******************************************************************************
  *          Interface Functions for Generic Scheduling Components             *
@@ -604,9 +646,9 @@ int starpu_sched_component_can_push(struct starpu_sched_component * component, s
 	return ret;
 }
 
-/* A can_pull call will try to wake up one worker associated to the childs of the
- * component. It is currenly called by components which holds a queue (like fifo and prio
- * components) to signify its childs that a task has been pushed on its local queue.
+/* A can_pull call will try to wake up one worker associated to the children of the
+ * component. It is currently called by components which holds a queue (like fifo and prio
+ * components) to signify its children that a task has been pushed on its local queue.
  */
 int starpu_sched_component_can_pull(struct starpu_sched_component * component)
 {
@@ -621,10 +663,9 @@ int starpu_sched_component_can_pull(struct starpu_sched_component * component)
 	return 0;
 }
 
-
-/* A can_pull call will try to wake up one worker associated to the childs of the
- * component. It is currenly called by components which holds a queue (like fifo and prio
- * components) to signify its childs that a task has been pushed on its local queue.
+/* A can_pull call will try to wake up one worker associated to the children of the
+ * component. It is currently called by components which holds a queue (like fifo and prio
+ * components) to signify its children that a task has been pushed on its local queue.
  */
 int starpu_sched_component_can_pull_all(struct starpu_sched_component * component)
 {
@@ -636,11 +677,11 @@ int starpu_sched_component_can_pull_all(struct starpu_sched_component * componen
 	return 0;
 }
 
-
 /* Alternative can_pull which says that this component does not want
-   to pull but prefers that you push. It can be used by decision
-   components, in which decisions are usually taken in their push()
-   functions */
+ * to pull but prefers that you push. It can be used by decision
+ * components, in which decisions are usually taken in their push()
+ * functions
+*/
 int starpu_sched_component_send_can_push_to_parents(struct starpu_sched_component * component)
 {
 	STARPU_ASSERT(component);
@@ -662,12 +703,11 @@ int starpu_sched_component_send_can_push_to_parents(struct starpu_sched_componen
 	return ret != 0;
 }
 
-
 double starpu_sched_component_estimated_load(struct starpu_sched_component * component)
 {
 	double sum = 0.0;
 	unsigned i;
-	for( i = 0; i < component->nchildren; i++)
+	for(i = 0; i < component->nchildren; i++)
 	{
 		struct starpu_sched_component * c = component->children[i];
 		sum += c->estimated_load(c);
@@ -680,9 +720,10 @@ double starpu_sched_component_estimated_end_min_add(struct starpu_sched_componen
 	STARPU_ASSERT(component);
 	double min = DBL_MAX;
 	unsigned i;
+	double ends[component->nchildren];
 	for(i = 0; i < component->nchildren; i++)
 	{
-		double tmp = component->children[i]->estimated_end(component->children[i]);
+		double tmp = ends[i] = component->children[i]->estimated_end(component->children[i]);
 		if(tmp < min)
 			min = tmp;
 	}
@@ -690,15 +731,14 @@ double starpu_sched_component_estimated_end_min_add(struct starpu_sched_componen
 	{
 		/* We don't know which workers will do this, assume it will be
 		 * evenly distributed to existing work */
-		int card = starpu_bitmap_cardinal(component->workers_in_ctx);
+		int card = starpu_bitmap_cardinal(&component->workers_in_ctx);
 		if (card == 0)
 			/* Oops, no resources to compute our tasks. Let's just hope that
 			 * we will be given one at some point */
 			card = 1;
 		for(i = 0; i < component->nchildren; i++)
 		{
-			double tmp = component->children[i]->estimated_end(component->children[i]);
-			exp_len += tmp - min;
+			exp_len += ends[i] - min;
 		}
 		min += exp_len / card;
 	}
@@ -707,7 +747,7 @@ double starpu_sched_component_estimated_end_min_add(struct starpu_sched_componen
 
 double starpu_sched_component_estimated_end_min(struct starpu_sched_component * component)
 {
-  return starpu_sched_component_estimated_end_min_add(component, 0.);
+	return starpu_sched_component_estimated_end_min_add(component, 0.);
 }
 
 double starpu_sched_component_estimated_end_average(struct starpu_sched_component * component)
@@ -729,8 +769,8 @@ struct starpu_sched_component * starpu_sched_component_create(struct starpu_sche
 	struct starpu_sched_component *component;
 	_STARPU_CALLOC(component, 1, sizeof(*component));
 	component->tree = tree;
-	component->workers = starpu_bitmap_create();
-	component->workers_in_ctx = starpu_bitmap_create();
+	starpu_bitmap_init(&component->workers);
+	starpu_bitmap_init(&component->workers_in_ctx);
 	component->add_child = starpu_sched_component_add_child;
 	component->remove_child = starpu_sched_component_remove_child;
 	component->add_parent = starpu_sched_component_add_parent;
