@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,16 +16,18 @@
 
 /* Work stealing policy */
 
+#include <starpu_scheduler.h>
+#include <schedulers/starpu_scheduler_toolbox.h>
+
 #include <float.h>
 #include <limits.h>
 
 #include <core/workers.h>
-#include <sched_policies/prio_deque.h>
 #include <core/debug.h>
-#include <starpu_scheduler.h>
 #include <core/sched_policy.h>
 #include <core/debug.h>
 #include <core/task.h>
+#include <sched_policies/prio_deque.h>
 
 /* Experimental (dead) code which needs to be tested, fixed... */
 /* #define USE_OVERLOAD */
@@ -71,10 +73,21 @@ struct locality_entry
 
 struct _starpu_work_stealing_data_per_worker
 {
-	struct _starpu_prio_deque queue;
+	char fill1[STARPU_CACHELINE_SIZE];
+	/* This is read-mostly, only updated when the queue becomes empty or
+	 * becomes non-empty, to make it generally cheap to check */
+	unsigned notask;	/* whether the queue is empty */
+	char fill2[STARPU_CACHELINE_SIZE];
+
+	struct starpu_st_prio_deque queue;
 	int running;
 	int *proxlist;
-	int busy;
+	int busy;	/* Whether this worker is working on a task */
+
+	/* keep track of the work performed from the beginning of the algorithm to make
+	 * better decisions about which queue to select when deferring work
+	 */
+	unsigned last_pop_worker;
 
 #ifdef USE_LOCALITY_TASKS
 	/* This records the same as queue, but hashed by data accessed with locality flag.  */
@@ -93,9 +106,8 @@ struct _starpu_work_stealing_data
 	int (*select_victim)(struct _starpu_work_stealing_data *, unsigned, int);
 	struct _starpu_work_stealing_data_per_worker *per_worker;
 	/* keep track of the work performed from the beginning of the algorithm to make
-	 * better decisions about which queue to select when stealing or deferring work
+	 * better decisions about which queue to select when deferring work
 	 */
-	unsigned last_pop_worker;
 	unsigned last_push_worker;
 };
 
@@ -118,7 +130,8 @@ static int calibration_value = 0;
  */
 static int select_victim_round_robin(struct _starpu_work_stealing_data *ws, unsigned sched_ctx_id)
 {
-	unsigned worker = ws->last_pop_worker;
+	unsigned workerid = starpu_worker_get_id_check();
+	unsigned worker = ws->per_worker[workerid].last_pop_worker;
 	unsigned nworkers;
 	int *workerids = NULL;
 	nworkers = starpu_sched_ctx_get_workers_list_raw(sched_ctx_id, &workerids);
@@ -131,14 +144,18 @@ static int select_victim_round_robin(struct _starpu_work_stealing_data *ws, unsi
 		/* Here helgrind would shout that this is unprotected, but we
 		 * are fine with getting outdated values, this is just an
 		 * estimation */
-		ntasks = ws->per_worker[workerids[worker]].queue.ntasks;
-
-		if (ntasks && (ws->per_worker[workerids[worker]].busy
-					   || starpu_worker_is_blocked_in_parallel(workerids[worker])))
-			break;
+		if (!ws->per_worker[workerids[worker]].notask)
+		{
+			if (ws->per_worker[workerids[worker]].busy
+			    || starpu_worker_is_blocked_in_parallel(workerids[worker]))
+			{
+				ntasks = 1;
+				break;
+			}
+		}
 
 		worker = (worker + 1) % nworkers;
-		if (worker == ws->last_pop_worker)
+		if (worker == ws->per_worker[workerid].last_pop_worker)
 		{
 			/* We got back to the first worker,
 			 * don't go in infinite loop */
@@ -147,7 +164,7 @@ static int select_victim_round_robin(struct _starpu_work_stealing_data *ws, unsi
 		}
 	}
 
-	ws->last_pop_worker = (worker + 1) % nworkers;
+	ws->per_worker[workerid].last_pop_worker = (worker + 1) % nworkers;
 
 	worker = workerids[worker];
 
@@ -326,16 +343,31 @@ static struct starpu_task *ws_pick_task(struct _starpu_work_stealing_data *ws, i
 	if (best_n > 0)
 	{
 		/* found an interesting task, try to pick it! */
-		if (_starpu_prio_deque_pop_this_task(&data_source->queue, target, best_task))
+		if (starpu_st_prio_deque_pop_this_task(&data_source->queue, target, best_task))
+		{
+			if (!data_source->queue.ntasks)
+			{
+				STARPU_ASSERT(ws->per_worker[source].notask == 0);
+				ws->per_worker[source].notask = 1;
+			}
 			return best_task;
+		}
 	}
 
-	/* Didn't find an interesting task, or couldn't run it :( */
-	int skipped;
+	/* Didn't find an interesting task, or couldn't run it */
+	struct starpu_task *task;
+
 	if (source != target)
-		return _starpu_prio_deque_deque_task_for_worker(&data_source->queue, target, &skipped);
+		task = starpu_st_prio_deque_deque_task_for_worker(&data_source->queue, target, NULL);
 	else
-		return _starpu_prio_deque_pop_task_for_worker(&data_source->queue, target, &skipped);
+		task = starpu_st_prio_deque_pop_task_for_worker(&data_source->queue, target, NULL);
+
+	if (task && !data_source->queue.ntasks)
+	{
+		STARPU_ASSERT(ws->per_worker[source].notask == 0);
+		ws->per_worker[source].notask = 1;
+	}
+	return task;
 }
 
 /* Called when popping a task from a queue */
@@ -370,11 +402,18 @@ static void locality_pushed_task(struct _starpu_work_stealing_data *ws STARPU_AT
 /* Pick a task from workerid's queue, for execution on target */
 static struct starpu_task *ws_pick_task(struct _starpu_work_stealing_data *ws, int source, int target)
 {
-	int skipped;
+	struct starpu_task *task;
 	if (source != target)
-		return _starpu_prio_deque_deque_task_for_worker(&ws->per_worker[source].queue, target, &skipped);
+		task = starpu_st_prio_deque_deque_task_for_worker(&ws->per_worker[source].queue, target, NULL);
 	else
-		return _starpu_prio_deque_pop_task_for_worker(&ws->per_worker[source].queue, target, &skipped);
+		task = starpu_st_prio_deque_pop_task_for_worker(&ws->per_worker[source].queue, target, NULL);
+
+	if (task && !ws->per_worker[source].queue.ntasks)
+	{
+		STARPU_ASSERT(ws->per_worker[source].notask == 0);
+		ws->per_worker[source].notask = 1;
+	}
+	return task;
 }
 /* Called when popping a task from a queue */
 static void locality_popped_task(struct _starpu_work_stealing_data *ws STARPU_ATTRIBUTE_UNUSED, struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, int workerid STARPU_ATTRIBUTE_UNUSED, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
@@ -390,7 +429,7 @@ static void locality_popped_task(struct _starpu_work_stealing_data *ws STARPU_AT
  *
  * \return	a ratio with a positive or negative value, describing the current state of the worker :
  * 		a smaller value implies a faster worker with an relatively emptier queue : more suitable to put tasks in
- * 		a bigger value implies a slower worker with an reletively more replete queue : more suitable to steal tasks from
+ * 		a bigger value implies a slower worker with an relatively more replete queue : more suitable to steal tasks from
  */
 static float overload_metric(struct _starpu_work_stealing_data *ws, unsigned sched_ctx_id, unsigned id)
 {
@@ -429,7 +468,7 @@ static int select_victim_overload(struct _starpu_work_stealing_data *ws, unsigne
 	float best_ratio = FLT_MIN;
 
 	/* Don't try to play smart until we get
-	 * enough informations. */
+	 * enough information. */
 	if (ws->performed_total < calibration_value)
 		return select_victim_round_robin(ws, sched_ctx_id);
 
@@ -439,7 +478,7 @@ static int select_victim_overload(struct _starpu_work_stealing_data *ws, unsigne
 	workers->init_iterator(workers, &it);
 	while(workers->has_next(workers, &it))
 	{
-                unsigned worker = workers->get_next(workers, &it);
+		unsigned worker = workers->get_next(workers, &it);
 		float worker_ratio = overload_metric(ws, sched_ctx_id, worker);
 
 		if (worker_ratio > best_ratio && ws->per_worker[worker].running && ws->per_worker[worker].busy)
@@ -461,12 +500,11 @@ static int select_victim_overload(struct _starpu_work_stealing_data *ws, unsigne
  */
 static unsigned select_worker_overload(struct _starpu_work_stealing_data *ws, struct starpu_task *task, unsigned sched_ctx_id)
 {
-	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	unsigned best_worker = 0;
 	float best_ratio = FLT_MAX;
 
 	/* Don't try to play smart until we get
-	 * enough informations. */
+	 * enough information. */
 	if (ws->performed_total < calibration_value)
 		return select_worker_round_robin(task, sched_ctx_id);
 
@@ -530,10 +568,11 @@ static struct starpu_task *ws_pop_task(unsigned sched_ctx_id)
 	struct starpu_task *task = NULL;
 	unsigned workerid = starpu_worker_get_id_check();
 
-	ws->per_worker[workerid].busy = 0;
+	if (ws->per_worker[workerid].busy)
+		ws->per_worker[workerid].busy = 0;
 
 #ifdef STARPU_NON_BLOCKING_DRIVERS
-	if (STARPU_RUNNING_ON_VALGRIND || !_starpu_prio_deque_is_empty(&ws->per_worker[workerid].queue))
+	if (STARPU_RUNNING_ON_VALGRIND || !starpu_st_prio_deque_is_empty(&ws->per_worker[workerid].queue))
 #endif
 	{
 		task = ws_pick_task(ws, workerid, workerid);
@@ -570,6 +609,11 @@ static struct starpu_task *ws_pop_task(unsigned sched_ctx_id)
 	if (_starpu_worker_trylock(victim))
 	{
 		/* victim is busy, don't bother it, come back later */
+#ifdef STARPU_SIMGRID
+		starpu_sleep(0.000001);
+		/* Make sure we come back and not block */
+		starpu_wake_worker_no_relax(workerid);
+#endif
 		return NULL;
 	}
 	if (ws->per_worker[victim].running && ws->per_worker[victim].queue.ntasks > 0)
@@ -589,8 +633,8 @@ static struct starpu_task *ws_pop_task(unsigned sched_ctx_id)
 	starpu_worker_unlock(victim);
 
 #ifndef STARPU_NON_BLOCKING_DRIVERS
-        /* While stealing, perhaps somebody actually give us a task, don't miss
-         * the opportunity to take it before going to sleep. */
+	/* While stealing, perhaps somebody actually give us a task, don't miss
+	 * the opportunity to take it before going to sleep. */
 	{
 		struct _starpu_worker *worker = _starpu_get_worker_struct(starpu_worker_get_id());
 		if (!task && worker->state_keep_awake)
@@ -617,7 +661,8 @@ static struct starpu_task *ws_pop_task(unsigned sched_ctx_id)
 		if (!task)
 			return NULL;
 	}
-	ws->per_worker[workerid].busy = !!task;
+	if (ws->per_worker[workerid].busy != !!task)
+		ws->per_worker[workerid].busy = !!task;
 	return task;
 }
 
@@ -647,7 +692,12 @@ int ws_push_task(struct starpu_task *task)
 	starpu_sched_task_break(task);
 	record_data_locality(task, workerid);
 	STARPU_ASSERT_MSG(ws->per_worker[workerid].running, "workerid=%d, ws=%p\n", workerid, ws);
-	_starpu_prio_deque_push_back_task(&ws->per_worker[workerid].queue, task);
+	starpu_st_prio_deque_push_back_task(&ws->per_worker[workerid].queue, task);
+	if (ws->per_worker[workerid].queue.ntasks == 1)
+	{
+		STARPU_ASSERT(ws->per_worker[workerid].notask == 1);
+		ws->per_worker[workerid].notask = 0;
+	}
 	locality_pushed_task(ws, task, workerid, sched_ctx_id);
 
 	starpu_push_task_end(task);
@@ -666,6 +716,15 @@ int ws_push_task(struct starpu_task *task)
 	return 0;
 }
 
+static void ws_push_task_notify(struct starpu_task *task, int workerid, int perf_workerid, unsigned sched_ctx_id)
+{
+	(void)task;
+	(void)perf_workerid;
+
+	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
+	ws->per_worker[workerid].busy = 1;
+}
+
 static void ws_add_workers(unsigned sched_ctx_id, int *workerids,unsigned nworkers)
 {
 	struct _starpu_work_stealing_data *ws = (struct _starpu_work_stealing_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
@@ -675,11 +734,13 @@ static void ws_add_workers(unsigned sched_ctx_id, int *workerids,unsigned nworke
 	{
 		int workerid = workerids[i];
 		starpu_sched_ctx_worker_shares_tasks_lists(workerid, sched_ctx_id);
-		_starpu_prio_deque_init(&ws->per_worker[workerid].queue);
+		starpu_st_prio_deque_init(&ws->per_worker[workerid].queue);
+		ws->per_worker[workerid].notask = 1;
 		ws->per_worker[workerid].running = 1;
 
 		/* Tell helgrind that we are fine with getting outdated values,
 		 * this is just an estimation */
+		STARPU_HG_DISABLE_CHECKING(ws->per_worker[workerid].notask);
 		STARPU_HG_DISABLE_CHECKING(ws->per_worker[workerid].queue.ntasks);
 		ws->per_worker[workerid].busy = 0;
 		STARPU_HG_DISABLE_CHECKING(ws->per_worker[workerid].busy);
@@ -695,7 +756,7 @@ static void ws_remove_workers(unsigned sched_ctx_id, int *workerids, unsigned nw
 	{
 		int workerid = workerids[i];
 
-		_starpu_prio_deque_destroy(&ws->per_worker[workerid].queue);
+		starpu_st_prio_deque_destroy(&ws->per_worker[workerid].queue);
 		ws->per_worker[workerid].running = 0;
 		free(ws->per_worker[workerid].proxlist);
 		ws->per_worker[workerid].proxlist = NULL;
@@ -708,9 +769,7 @@ static void initialize_ws_policy(unsigned sched_ctx_id)
 	_STARPU_MALLOC(ws, sizeof(struct _starpu_work_stealing_data));
 	starpu_sched_ctx_set_policy_data(sched_ctx_id, (void*)ws);
 
-	ws->last_pop_worker = 0;
 	ws->last_push_worker = 0;
-	STARPU_HG_DISABLE_CHECKING(ws->last_pop_worker);
 	STARPU_HG_DISABLE_CHECKING(ws->last_push_worker);
 	ws->select_victim = select_victim;
 
@@ -740,9 +799,9 @@ struct starpu_sched_policy _starpu_sched_ws_policy =
 	.remove_workers = ws_remove_workers,
 	.push_task = ws_push_task,
 	.pop_task = ws_pop_task,
+	.push_task_notify = ws_push_task_notify,
 	.pre_exec_hook = NULL,
 	.post_exec_hook = NULL,
-	.pop_every_task = NULL,
 	.policy_name = "ws",
 	.policy_description = "work stealing",
 	.worker_type = STARPU_WORKER_LIST,
@@ -750,7 +809,7 @@ struct starpu_sched_policy _starpu_sched_ws_policy =
 
 /* local work stealing policy */
 /* Return a worker to steal a task from. The worker is selected according to
- * the proximity list built using the info on te architecture provided by hwloc
+ * the proximity list built using the info on the architecture provided by hwloc
  */
 #ifdef STARPU_HAVE_HWLOC
 static int lws_select_victim(struct _starpu_work_stealing_data *ws, unsigned sched_ctx_id, int workerid)
@@ -760,11 +819,12 @@ static int lws_select_victim(struct _starpu_work_stealing_data *ws, unsigned sch
 	for (i = 0; i < nworkers; i++)
 	{
 		int neighbor = ws->per_worker[workerid].proxlist[i];
-                /* FIXME: do not keep looking again and again at some worker
-                 * which has tasks, but that can't execute on me */
-		int ntasks = ws->per_worker[neighbor].queue.ntasks;
-		if (ntasks && (ws->per_worker[neighbor].busy
-					   || starpu_worker_is_blocked_in_parallel(neighbor)))
+		if (ws->per_worker[neighbor].notask)
+			continue;
+		/* FIXME: do not keep looking again and again at some worker
+		 * which has tasks, but that can't execute on me */
+		if (ws->per_worker[neighbor].busy
+		    || starpu_worker_is_blocked_in_parallel(neighbor))
 			return neighbor;
 	}
 	return -1;
@@ -850,9 +910,9 @@ struct starpu_sched_policy _starpu_sched_lws_policy =
 	.remove_workers = ws_remove_workers,
 	.push_task = ws_push_task,
 	.pop_task = ws_pop_task,
+	.push_task_notify = ws_push_task_notify,
 	.pre_exec_hook = NULL,
 	.post_exec_hook = NULL,
-	.pop_every_task = NULL,
 	.policy_name = "lws",
 	.policy_description = "locality work stealing",
 #ifdef STARPU_HAVE_HWLOC
