@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2012-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,9 +21,13 @@
 
 #include <starpu.h>
 
+#pragma GCC visibility push(hidden)
+
 int _starpu_data_cpy(starpu_data_handle_t dst_handle, starpu_data_handle_t src_handle,
-			int asynchronous, void (*callback_func)(void*), void *callback_arg,
-			int reduction, struct starpu_task *reduction_dep_task);
+		     int asynchronous, void (*callback_func)(void*), void *callback_arg,
+		     int reduction, struct starpu_task *reduction_dep_task, int priority);
+
+#pragma GCC visibility pop
 
 #endif // __STARPU_DATA_CPY_H__
 
