@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -205,7 +205,7 @@ STARPUFFT(twist2_2d_kernel_cpu)(void *descr[], void *_args)
 	STARPUFFT(plan) plan = args->plan;
 	int kk = args->kk;	/* between 0 and DIV_2D_N */
 	int ll = args->ll;	/* between 0 and DIV_2D_M */
-	int kkk, lll;		/* beetween 0,0 and n3,m3 */
+	int kkk, lll;		/* between 0,0 and n3,m3 */
 	int i, j;
 	int n1 = plan->n1[0];
 	int n2 = plan->n2[0];
@@ -259,7 +259,7 @@ STARPUFFT(twist3_2d_kernel_cpu)(void *descr[], void *_args)
 	STARPUFFT(plan) plan = args->plan;
 	int kk = args->kk;	/* between 0 and DIV_2D_N */
 	int ll = args->ll;	/* between 0 and DIV_2D_M */
-	int kkk, lll;		/* beetween 0,0 and n3,m3 */
+	int kkk, lll;		/* between 0,0 and n3,m3 */
 	int i, j;
 	int n1 = plan->n1[0];
 	int n2 = plan->n2[0];
