@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012,2013                                Inria
- * Copyright (C) 2012,2013,2015,2017,2019                 CNRS
- * Copyright (C) 2013                                     Université de Bordeaux
- * Copyright (C) 2013                                     Thibaut Lambert
+ * Copyright (C) 2012-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,7 +21,7 @@
 #define NY 2
 #define MATRIX_SIZE (NX*NY)
 
-#if defined(STARPU_USE_CPU) || defined(STAPRU_USE_MIC)
+#if defined(STARPU_USE_CPU)
 void test_coo_cpu_func(void *buffers[], void *args);
 #endif
 #ifdef STARPU_USE_CUDA
@@ -35,6 +33,13 @@ extern void test_coo_opencl_func(void *buffers[], void *args);
 
 static starpu_data_handle_t coo_handle, coo2_handle;
 
+static uint32_t columns[MATRIX_SIZE];
+static uint32_t rows[MATRIX_SIZE];
+static int values[MATRIX_SIZE];
+static uint32_t columns2[MATRIX_SIZE];
+static uint32_t rows2[MATRIX_SIZE];
+static int values2[MATRIX_SIZE];
+
 struct test_config coo_config =
 {
 #ifdef STARPU_USE_CPU
@@ -46,11 +51,10 @@ struct test_config coo_config =
 #ifdef STARPU_USE_OPENCL
 	.opencl_func   = test_coo_opencl_func,
 #endif /* !STARPU_USE_OPENCL */
-#ifdef STARPU_USE_MIC
-	.cpu_func_name = "test_coo_cpu_func",
-#endif
 	.handle        = &coo_handle,
+	.ptr           = values,
 	.dummy_handle  = &coo2_handle,
+	.dummy_ptr     = values2,
 	.copy_failed   = SUCCESS,
 	.name          = "coo_interface"
 };
@@ -59,29 +63,22 @@ void
 test_coo_cpu_func(void *buffers[], void *args)
 {
 	int factor = *(int *) args;
-	int *values = (int *) STARPU_COO_GET_VALUES(buffers[0]);
+	int *vals = (int *) STARPU_COO_GET_VALUES(buffers[0]);
 	unsigned size = STARPU_COO_GET_NVALUES(buffers[0]);
 
 	int i;
 	for (i = 0; i < (int)size; i++)
 	{
-		if (values[i] != i * factor)
+		if (vals[i] != i * factor)
 		{
 			coo_config.copy_failed = FAILURE;
 			return;
 		}
-		values[i] *= -1;
+		vals[i] *= -1;
 	}
 }
 
 
-static uint32_t columns[MATRIX_SIZE];
-static uint32_t rows[MATRIX_SIZE];
-static int values[MATRIX_SIZE];
-static uint32_t columns2[MATRIX_SIZE];
-static uint32_t rows2[MATRIX_SIZE];
-static int values2[MATRIX_SIZE];
-
 static void
 register_data(void)
 {
@@ -159,22 +156,19 @@ main(int argc, char **argv)
 	starpu_conf_init(&conf);
 	conf.ncuda = 2;
 	conf.nopencl = 1;
-	conf.nmic = -1;
 
-	if (starpu_initialize(&conf, &argc, &argv) == -ENODEV || starpu_cpu_worker_get_count() == 0)
-		goto enodev;
+	int ret = starpu_initialize(&conf, &argc, &argv);
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	register_data();
 
-	run_tests(&coo_config, &summary);
+	ret = run_tests(&coo_config, &summary);
 
 	unregister_data();
 
-	data_interface_test_summary_print(stderr, &summary);
+	if (ret) data_interface_test_summary_print(stderr, &summary);
 
 	starpu_shutdown();
 	return data_interface_test_summary_success(&summary);
-
-enodev:
-	return STARPU_TEST_SKIPPED;
 }
