@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,8 +16,7 @@
 
 #include <starpu.h>
 
-/* XXX Why cant we dereference a handle without this one ? */
-#include <core/sched_policy.h>
+#include <datawizard/coherency.h>
 
 #include <assert.h>
 
@@ -44,11 +43,12 @@ static char *enum_to_string(int exit_code)
 		case UNTESTED:
 			return "Untested";
 		case NO_DEVICE:
+		case -ENODEV:
 			return "No device available";
 		case TASK_SUBMISSION_FAILURE:
 			return "Task submission failed";
 		default:
-			assert(0);
+			STARPU_ASSERT_MSG(0, "unknown error code %d\n", exit_code);
 	}
 }
 
@@ -66,9 +66,6 @@ void data_interface_test_summary_print(FILE *f, struct data_interface_test_summa
 	FPRINTF(f, "\tCPU    -> OpenCL : %s\n", enum_to_string(s->cpu_to_opencl_async));
 	FPRINTF(f, "\tOpenCL -> CPU    : %s\n", enum_to_string(s->opencl_to_cpu_async));
 	FPRINTF(f, "\n");
-	FPRINTF(f, "\tCPU    -> MIC    : %s\n", enum_to_string(s->cpu_to_mic_async));
-	FPRINTF(f, "\tMIC    -> CPU    : %s\n", enum_to_string(s->mic_to_cpu_async));
-	FPRINTF(f, "\n");
 
 	FPRINTF(f, "Synchronous :\n");
 	FPRINTF(f, "\tCPU    -> CUDA   : %s\n", enum_to_string(s->cpu_to_cuda));
@@ -78,13 +75,10 @@ void data_interface_test_summary_print(FILE *f, struct data_interface_test_summa
 	FPRINTF(f, "\tCPU    -> OpenCL : %s\n", enum_to_string(s->cpu_to_opencl));
 	FPRINTF(f, "\tOpenCL -> CPU    : %s\n", enum_to_string(s->opencl_to_cpu));
 	FPRINTF(f, "\n");
-	FPRINTF(f, "\tCPU    -> MIC    : %s\n", enum_to_string(s->cpu_to_mic));
-	FPRINTF(f, "\tMIC    -> CPU    : %s\n",	enum_to_string(s->mic_to_cpu));
 
 	FPRINTF(f, "\n");
 	FPRINTF(f, "CPU -> CPU          : %s\n", enum_to_string(s->cpu_to_cpu));
 	FPRINTF(f, "to_pointer()        : %s\n", enum_to_string(s->to_pointer));
-	FPRINTF(f, "pointer_is_inside() : %s\n", enum_to_string(s->pointer_is_inside));
 	FPRINTF(f, "compare()           : %s\n", enum_to_string(s->compare));
 	FPRINTF(f, "pack_unpack()       : %s\n", enum_to_string(s->pack));
 }
@@ -114,12 +108,7 @@ static void summary_init(struct data_interface_test_summary *s)
 	s->opencl_to_cpu         = UNTESTED;
 	s->cpu_to_opencl_async   = UNTESTED;
 	s->opencl_to_cpu_async   = UNTESTED;
-	s->cpu_to_mic            = UNTESTED;
-	s->mic_to_cpu            = UNTESTED;
-	s->cpu_to_mic_async      = UNTESTED;
-	s->mic_to_cpu_async      = UNTESTED;
 	s->to_pointer            = UNTESTED;
-	s->pointer_is_inside     = UNTESTED;
 	s->pack                  = UNTESTED;
 	s->success               = SUCCESS;
 };
@@ -147,12 +136,10 @@ static int create_task(struct starpu_task **taskp, enum starpu_worker_archtype t
 	static int cpu_workers[STARPU_MAXCPUS];
 	static int cuda_workers[STARPU_MAXCUDADEVS];
 	static int opencl_workers[STARPU_MAXOPENCLDEVS];
-	static int mic_workers[STARPU_MAXMICDEVS];
 
 	static int n_cpus = -1;
 	static int n_cudas = -1;
 	static int n_opencls = -1;
-	static int n_mics = -1;
 
 	if (n_cpus == -1) /* First time here */
 	{
@@ -163,7 +150,6 @@ static int create_task(struct starpu_task **taskp, enum starpu_worker_archtype t
 		n_cpus = starpu_worker_get_ids_by_type(STARPU_CPU_WORKER, cpu_workers, STARPU_MAXCPUS);
 		n_cudas = starpu_worker_get_ids_by_type(STARPU_CUDA_WORKER, cuda_workers, STARPU_MAXCUDADEVS);
 		n_opencls = starpu_worker_get_ids_by_type(STARPU_OPENCL_WORKER, opencl_workers, STARPU_MAXOPENCLDEVS);
-		n_mics = starpu_worker_get_ids_by_type(STARPU_MIC_WORKER, mic_workers, STARPU_MAXMICDEVS);
 	}
 
 	int *workers;
@@ -205,17 +191,6 @@ static int create_task(struct starpu_task **taskp, enum starpu_worker_archtype t
 		workers = opencl_workers;
 		cl.opencl_funcs[0] = current_config->opencl_func;
 	}
-	else if (type == STARPU_MIC_WORKER)
-	{
-		if (n_mics == 0) return -ENODEV;
-		if (id != -1 && id >= n_mics)
-		{
-			FPRINTF(stderr, "Not enough MIC workers\n");
-			return -ENODEV;
-		}
-		workers = mic_workers;
-		cl.cpu_funcs_name[0] = current_config->cpu_func_name;
-	}
 	else
 	{
 		return -ENODEV;
@@ -328,40 +303,6 @@ static enum exit_code opencl_to_ram(void)
 	FPRINTF(stderr, "[%s] : %d\n", __starpu_func__, current_config->copy_failed);
 	return current_config->copy_failed;
 }
-
-static enum exit_code ram_to_mic()
-{
-	int err;
-	struct starpu_task *task;
-
-	err = create_task(&task, STARPU_MIC_WORKER, -1);
-	if (err != 0)
-		return NO_DEVICE;
-
-	err = starpu_task_submit(task);
-	if (err != 0)
-		return TASK_SUBMISSION_FAILURE;
-
-	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
-	return current_config->copy_failed;
-}
-
-static enum exit_code mic_to_ram()
-{
-	int err;
-	struct starpu_task *task;
-
-	err = create_task(&task, STARPU_CPU_WORKER, -1);
-	if (err != 0)
-		return NO_DEVICE;
-
-	err = starpu_task_submit(task);
-	if (err != 0)
-		return TASK_SUBMISSION_FAILURE;
-
-	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
-	return current_config->copy_failed;
-}
 /* End of the <device1>_to_<device2> functions. */
 
 static void run_cuda(int async, struct data_interface_test_summary *s)
@@ -404,19 +345,6 @@ static void run_opencl(int async, struct data_interface_test_summary *s)
 	set_field(s, async==1?&s->opencl_to_cpu_async:&s->opencl_to_cpu, err);
 }
 
-static void run_mic(int async, struct data_interface_test_summary *s)
-{
-	int err;
-
-	err = ram_to_mic();
-	set_field(s, &s->cpu_to_mic_async, err);
-	if (err != SUCCESS)
-		return;
-
-	err = mic_to_ram();
-	set_field(s, &s->mic_to_cpu_async, err);
-}
-
 static void ram_to_ram(struct data_interface_test_summary *s)
 {
 	int err;
@@ -465,7 +393,6 @@ static void run_async(struct data_interface_test_summary *s)
 	}
 	run_cuda(1, s);
 	run_opencl(1, s);
-	run_mic(1, s);
 }
 
 static void run_sync(struct data_interface_test_summary *s)
@@ -483,14 +410,11 @@ static void run_sync(struct data_interface_test_summary *s)
 	new_copy_methods.cuda_to_ram_async = NULL;
 	new_copy_methods.ram_to_opencl_async = NULL;
 	new_copy_methods.opencl_to_ram_async = NULL;
-	new_copy_methods.ram_to_mic_async = NULL;
-	new_copy_methods.mic_to_ram_async = NULL;
 
 	handle->ops->copy_methods = &new_copy_methods;
 
 	run_cuda(0, s);
 	run_opencl(0, s);
-	run_mic(0, s);
 
 	handle->ops->copy_methods = old_copy_methods;
 }
@@ -531,7 +455,7 @@ static void to_pointer(struct data_interface_test_summary *s)
 
 			void *data_interface = starpu_data_get_interface_on_node(handle, node);
 			void *ptr = handle->ops->to_pointer(data_interface, node);
-			if (starpu_data_lookup(ptr) != handle)
+			if (ptr != current_config->ptr)
 			{
 				s->to_pointer = FAILURE;
 				break;
@@ -544,44 +468,6 @@ static void to_pointer(struct data_interface_test_summary *s)
 	set_field(s, &s->to_pointer, s->to_pointer);
 }
 
-static void pointer_is_inside(struct data_interface_test_summary *s)
-{
-	starpu_data_handle_t handle;
-
-	s->pointer_is_inside = UNTESTED;
-	handle = *current_config->handle;
-	if (handle->ops->pointer_is_inside && handle->ops->to_pointer)
-	{
-		unsigned int node;
-		unsigned int tests = 0;
-
-		for (node = 0; node < STARPU_MAXNODES; node++)
-		{
-			if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
-				continue;
-			if (!starpu_data_test_if_allocated_on_node(handle, node))
-				continue;
-
-			void *data_interface = starpu_data_get_interface_on_node(handle, node);
-			void *ptr = handle->ops->to_pointer(data_interface, node);
-			if (starpu_data_lookup(ptr) != handle)
-			{
-				s->pointer_is_inside = FAILURE;
-				break;
-			}
-			if (!starpu_data_pointer_is_inside(handle, node, ptr))
-			{
-				s->pointer_is_inside = FAILURE;
-				break;
-			}
-			tests++;
-		}
-		if (tests > 0)
-			s->pointer_is_inside = SUCCESS;
-	}
-	set_field(s, &s->pointer_is_inside, s->pointer_is_inside);
-}
-
 static void pack_unpack(struct data_interface_test_summary *s)
 {
 	starpu_data_handle_t handle;
@@ -600,8 +486,10 @@ static void pack_unpack(struct data_interface_test_summary *s)
 			struct starpu_task *task;
 			void *mem = (void *)starpu_malloc_on_node_flags(STARPU_MAIN_RAM, size, 0);
 
+			starpu_data_acquire(dummy_handle, STARPU_W);
 			starpu_data_unpack(dummy_handle, mem, size);
 			starpu_data_unpack(dummy_handle, ptr, size);
+			starpu_data_release(dummy_handle);
 
 			factor = -factor;
 			err = create_task(&task, STARPU_CPU_WORKER, -1);
@@ -636,9 +524,6 @@ static int load_conf(struct test_config *config)
 #endif
 #ifdef STARPU_USE_OPENCL
 	    !config->opencl_func ||
-#endif
-#ifdef STARPU_USE_MIC
-	    !config->cpu_func_name ||
 #endif
 	    !config->handle)
 	{
@@ -649,13 +534,14 @@ static int load_conf(struct test_config *config)
 	return 0;
 }
 
-void run_tests(struct test_config *conf, struct data_interface_test_summary *s)
+int run_tests(struct test_config *conf, struct data_interface_test_summary *s)
 {
 	summary_init(s);
 
 	if (load_conf(conf) == 1)
 	{
 		FPRINTF(stderr, "Failed to load conf.\n");
+		return 0;
 	}
 
 	run_async(s);
@@ -664,7 +550,8 @@ void run_tests(struct test_config *conf, struct data_interface_test_summary *s)
 	ram_to_ram(s);
 	compare(s);
 	to_pointer(s);
-	pointer_is_inside(s);
 
 	pack_unpack(s);
+
+	return 1;
 }
