@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,9 +23,7 @@ static starpu_data_handle_t variable_handle;
 static starpu_data_handle_t variable2_handle;
 
 /* Codelets */
-#if defined(STARPU_USE_CPU) || defined(STARPU_USE_MIC)
 void test_variable_cpu_func(void *buffers[], void *args);
-#endif
 #ifdef STARPU_USE_CUDA
 extern void test_variable_cuda_func(void *buffers[], void *args);
 #endif
@@ -35,26 +33,22 @@ extern void test_variable_opencl_func(void *buffers[], void *args);
 
 struct test_config variable_config =
 {
-#ifdef STARPU_USE_CPU
-	.cpu_func     = test_variable_cpu_func,
-#endif
+	.cpu_func      = test_variable_cpu_func,
 #ifdef STARPU_USE_CUDA
 	.cuda_func    = test_variable_cuda_func,
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_func  = test_variable_opencl_func,
-#endif
-#ifdef STARPU_USE_MIC
-	.cpu_func_name = "test_variable_cpu_func",
 #endif
 	.handle       = &variable_handle,
+	.ptr          = &variable,
 	.dummy_handle = &variable2_handle,
+	.dummy_ptr    = &variable2,
 	.copy_failed  = SUCCESS,
 	.name         = "variable_interface"
 };
 
-void
-test_variable_cpu_func(void *buffers[], void *args)
+void test_variable_cpu_func(void *buffers[], void *args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -70,9 +64,7 @@ test_variable_cpu_func(void *buffers[], void *args)
 		*val *= -1;
 }
 
-
-static
-void register_data(void)
+static void register_data(void)
 {
 	variable = 42;
 	variable2 = 12;
@@ -83,8 +75,7 @@ void register_data(void)
 				      (uintptr_t) &variable2, sizeof(variable2));
 }
 
-static
-void unregister_data(void)
+static void unregister_data(void)
 {
 	starpu_data_unregister(variable_handle);
 	starpu_data_unregister(variable2_handle);
@@ -100,11 +91,10 @@ int main(int argc, char **argv)
 
 	conf.ncuda = 2;
 	conf.nopencl = 1;
-	conf.nmic = -1;
 
 	ret = starpu_initialize(&conf, &argc, &argv);
-	if (ret == -ENODEV || starpu_cpu_worker_get_count() == 0)
-		return STARPU_TEST_SKIPPED;
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	register_data();
 
