@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2012-2013,2015,2017                      CNRS
- * Copyright (C) 2013                                     Université de Bordeaux
+ * Copyright (C) 2012-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,9 +34,6 @@ struct test_config void_config =
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_func   = fake_func,
-#endif
-#ifdef STARPU_USE_MIC
-	.cpu_func_name = "fake_func",
 #endif
 	.handle        = &void_handle,
 	.dummy_handle  = &void2_handle,
@@ -46,47 +41,39 @@ struct test_config void_config =
 	.name          = "void_interface"
 };
 
-static void
-register_data(void)
+static void register_data(void)
 {
 	starpu_void_data_register(&void_handle);
 	starpu_void_data_register(&void2_handle);
 }
 
-static void
-unregister_data(void)
+static void unregister_data(void)
 {
 	starpu_data_unregister(void_handle);
 	starpu_data_unregister(void2_handle);
 }
 
-int
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
-	data_interface_test_summary *summary;
+	struct data_interface_test_summary summary;
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
 	conf.ncuda = 2;
 	conf.nopencl = 1;
-	conf.nmic = -1;
 
-	if (starpu_initialize(&conf, &argc, &argv) == -ENODEV || starpu_cpu_worker_get_count() == 0)
-		goto enodev;
+	int ret = starpu_initialize(&conf, &argc, &argv);
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	register_data();
 
-	summary = run_tests(&void_config);
-	if (!summary)
-		exit(EXIT_FAILURE);
+	run_tests(&void_config, &summary);
 
 	unregister_data();
 
 	starpu_shutdown();
 
-	data_interface_test_summary_print(stderr, summary);
+	data_interface_test_summary_print(stderr, &summary);
 
-	return data_interface_test_summary_success(summary);
-
-enodev:
-	return STARPU_TEST_SKIPPED;
+	return data_interface_test_summary_success(&summary);
 }
