@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,11 +17,17 @@
 #include <starpu.h>
 #include "../helper.h"
 
+/*
+ * Try mixing readers and writers on the same variable
+ */
+
 static unsigned book = 0;
 static starpu_data_handle_t book_handle;
 
-static void dummy_kernel(void *descr[], void *arg)
+void dummy_kernel(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 }
 
 static struct starpu_codelet r_cl =
@@ -30,6 +35,7 @@ static struct starpu_codelet r_cl =
 	.cuda_funcs = {dummy_kernel},
 	.cpu_funcs = {dummy_kernel},
 	.opencl_funcs = {dummy_kernel},
+	.cpu_funcs_name = {"dummy_kernel"},
 	.nbuffers = 1,
 	.modes = {STARPU_R}
 };
@@ -39,6 +45,7 @@ static struct starpu_codelet w_cl =
 	.cuda_funcs = {dummy_kernel},
 	.cpu_funcs = {dummy_kernel},
 	.opencl_funcs = {dummy_kernel},
+	.cpu_funcs_name = {"dummy_kernel"},
 	.nbuffers = 1,
 	.modes = {STARPU_W}
 };
@@ -47,12 +54,12 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	/* initialize the resource */
-	starpu_vector_data_register(&book_handle, 0, (uintptr_t)&book, 1, sizeof(unsigned));
+	starpu_vector_data_register(&book_handle, STARPU_MAIN_RAM, (uintptr_t)&book, 1, sizeof(unsigned));
 
 #ifdef STARPU_QUICK_CHECK
 	unsigned ntasks = 16;
