@@ -1,8 +1,7 @@
 #!/bin/bash
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2016,2017                                CNRS
-# Copyright (C) 2016,2017,2019                           Université de Bordeaux
+# Copyright (C) 2016-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -15,8 +14,11 @@
 #
 # See the GNU Lesser General Public License in COPYING.LGPL for more details.
 #
+
+if test -n "$STARPU_MICROBENCHS_DISABLED" ; then exit 77 ; fi
+
 source $(dirname $0)/microbench.sh
 
-XSUCCESS="dmda dmdap dmdar dmdas dmdasd pheft"
+XSUCCESS="dmda dmdap dmdar dmdas dmdasd modular-dmda modular-dmdap modular-dmdar modular-dmdas pheft"
 
 test_scheds parallel_independent_heterogeneous_tasks_data
