@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -134,33 +134,33 @@ int main(int argc, char **argv)
 	tasks[ntasks-1].detach = 0;
 
 	start_submit = starpu_timing_now();
-        if (nbuffers)
-        {
-                /* Data dependency, just submit them all */
-                for (i = 0; i < ntasks; i++)
-                {
-                        ret = starpu_task_submit(&tasks[i]);
-                        if (ret == -ENODEV) goto enodev;
-                        STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
-                }
-        }
-        else
-        {
-                /* No data dependency, we have to introduce dependencies by hand */
-                for (i = 1; i < ntasks; i++)
-                {
-                        starpu_tag_declare_deps((starpu_tag_t)i, 1, (starpu_tag_t)(i-1));
+	if (nbuffers)
+	{
+		/* Data dependency, just submit them all */
+		for (i = 0; i < ntasks; i++)
+		{
+			ret = starpu_task_submit(&tasks[i]);
+			if (ret == -ENODEV) goto enodev;
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
+		}
+	}
+	else
+	{
+		/* No data dependency, we have to introduce dependencies by hand */
+		for (i = 1; i < ntasks; i++)
+		{
+			starpu_tag_declare_deps((starpu_tag_t)i, 1, (starpu_tag_t)(i-1));
 
-                        ret = starpu_task_submit(&tasks[i]);
-                        if (ret == -ENODEV) goto enodev;
-                        STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
-                }
+			ret = starpu_task_submit(&tasks[i]);
+			if (ret == -ENODEV) goto enodev;
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
+		}
 
-                /* submit the first task */
-                ret = starpu_task_submit(&tasks[0]);
-                if (ret == -ENODEV) goto enodev;
-                STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
-        }
+		/* submit the first task */
+		ret = starpu_task_submit(&tasks[0]);
+		if (ret == -ENODEV) goto enodev;
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
+	}
 
 	end_submit = starpu_timing_now();
 
@@ -187,61 +187,61 @@ int main(int argc, char **argv)
 	fprintf(stderr, "Total: %f secs\n", (timing_submit+timing_exec)/1000000);
 	fprintf(stderr, "Per task: %f usecs\n", (timing_submit+timing_exec)/ntasks);
 
-        {
-                char *output_dir = getenv("STARPU_BENCH_DIR");
-                char *bench_id = getenv("STARPU_BENCH_ID");
+	{
+		char *output_dir = getenv("STARPU_BENCH_DIR");
+		char *bench_id = getenv("STARPU_BENCH_ID");
 
-                if (output_dir && bench_id)
+		if (output_dir && bench_id)
 		{
-                        char number[1+sizeof(nbuffers)*3+1];
-                        const char *numberp;
-                        char file[1024];
-                        FILE *f;
+			char number[1+sizeof(nbuffers)*3+1];
+			const char *numberp;
+			char file[1024];
+			FILE *f;
 
-                        if (nbuffers)
-                        {
-                                snprintf(number, sizeof(number), "_%u", nbuffers);
-                                numberp = number;
-                        }
-                        else
-                                numberp = "";
+			if (nbuffers)
+			{
+				snprintf(number, sizeof(number), "_%u", nbuffers);
+				numberp = number;
+			}
+			else
+				numberp = "";
 
-                        snprintf(file, sizeof(file), "%s/tasks_overhead_total_submit%s.dat", output_dir, numberp);
-                        f = fopen(file, "a");
-                        fprintf(f, "%s\t%f\n", bench_id, timing_submit/1000000);
-                        fclose(f);
+			snprintf(file, sizeof(file), "%s/tasks_overhead_total_submit%s.dat", output_dir, numberp);
+			f = fopen(file, "a");
+			fprintf(f, "%s\t%f\n", bench_id, timing_submit/1000000);
+			fclose(f);
 
-                        snprintf(file, sizeof(file), "%s/tasks_overhead_per_task_submit%s.dat", output_dir, numberp);
-                        f = fopen(file, "a");
-                        fprintf(f, "%s\t%f\n", bench_id, timing_submit/ntasks);
-                        fclose(f);
+			snprintf(file, sizeof(file), "%s/tasks_overhead_per_task_submit%s.dat", output_dir, numberp);
+			f = fopen(file, "a");
+			fprintf(f, "%s\t%f\n", bench_id, timing_submit/ntasks);
+			fclose(f);
 
-                        snprintf(file, sizeof(file), "%s/tasks_overhead_total_execution%s.dat", output_dir, numberp);
-                        f = fopen(file, "a");
-                        fprintf(f, "%s\t%f\n", bench_id, timing_exec/1000000);
-                        fclose(f);
+			snprintf(file, sizeof(file), "%s/tasks_overhead_total_execution%s.dat", output_dir, numberp);
+			f = fopen(file, "a");
+			fprintf(f, "%s\t%f\n", bench_id, timing_exec/1000000);
+			fclose(f);
 
-                        snprintf(file, sizeof(file), "%s/tasks_overhead_per_task_execution%s.dat", output_dir, numberp);
-                        f = fopen(file, "a");
-                        fprintf(f, "%s\t%f\n", bench_id, timing_exec/ntasks);
-                        fclose(f);
+			snprintf(file, sizeof(file), "%s/tasks_overhead_per_task_execution%s.dat", output_dir, numberp);
+			f = fopen(file, "a");
+			fprintf(f, "%s\t%f\n", bench_id, timing_exec/ntasks);
+			fclose(f);
 
-                        snprintf(file, sizeof(file), "%s/tasks_overhead_total_submit_execution%s.dat", output_dir, numberp);
-                        f = fopen(file, "a");
-                        fprintf(f, "%s\t%f\n", bench_id, (timing_submit+timing_exec)/1000000);
-                        fclose(f);
+			snprintf(file, sizeof(file), "%s/tasks_overhead_total_submit_execution%s.dat", output_dir, numberp);
+			f = fopen(file, "a");
+			fprintf(f, "%s\t%f\n", bench_id, (timing_submit+timing_exec)/1000000);
+			fclose(f);
 
-                        snprintf(file, sizeof(file), "%s/tasks_overhead_per_task_submit_execution%s.dat", output_dir, numberp);
-                        f = fopen(file, "a");
-                        fprintf(f, "%s\t%f\n", bench_id, (timing_submit+timing_exec)/ntasks);
-                        fclose(f);
-                }
-        }
+			snprintf(file, sizeof(file), "%s/tasks_overhead_per_task_submit_execution%s.dat", output_dir, numberp);
+			f = fopen(file, "a");
+			fprintf(f, "%s\t%f\n", bench_id, (timing_submit+timing_exec)/ntasks);
+			fclose(f);
+		}
+	}
 
 	for (buffer = 0; buffer < nbuffers; buffer++)
 	{
 		starpu_data_unregister(data_handles[buffer]);
-		starpu_free((void*)buffers[buffer]);
+		starpu_free_noflag((void*)buffers[buffer], BUFFERSIZE*sizeof(float));
 	}
 
 	starpu_shutdown();
@@ -251,7 +251,7 @@ int main(int argc, char **argv)
 enodev:
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
- 	 * could perform the kernel, so this is not an error from StarPU */
+	 * could perform the kernel, so this is not an error from StarPU */
 	starpu_shutdown();
 	free(tasks);
 	return STARPU_TEST_SKIPPED;
