@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2017-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2017-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,15 +23,6 @@
 #else
 #define _GNU_SOURCE 1
 // Assuming recent simgrid
-#define STARPU_HAVE_SIMGRID_MSG_H
-#define STARPU_HAVE_SIMGRID_SEMAPHORE_H
-#define STARPU_HAVE_SIMGRID_MUTEX_H
-#define STARPU_HAVE_SIMGRID_COND_H
-#define STARPU_HAVE_SIMGRID_BARRIER_H
-#define STARPU_HAVE_XBT_SYNCHRO_H
-#define HAVE_SIMGRID_GET_CLOCK
-#define HAVE_SG_ACTOR_SLEEP_FOR
-#define HAVE_SG_CFG_SET_INT
 #endif
 #include <unistd.h>
 #include <stdlib.h>
@@ -39,45 +30,37 @@
 #include <limits.h>
 #include <common/list.h>
 #include <common/prio_list.h>
-#ifdef STARPU_HAVE_SIMGRID_MSG_H
-#include <simgrid/msg.h>
-#else
-#include <msg/msg.h>
-#endif
+
 #include <simgrid/modelchecker.h>
-#ifdef STARPU_HAVE_XBT_SYNCHRO_H
-#include <xbt/synchro.h>
-#else
-#include <xbt/synchro_core.h>
-#endif
+
+#include <xbt/base.h>
+
+#include <simgrid/actor.h>
+#include <simgrid/host.h>
+#include <simgrid/engine.h>
+#include <xbt/config.h>
+
+#include <simgrid/mutex.h>
+#include <simgrid/cond.h>
 
 #include <common/rbtree.c>
 
 #ifndef NLISTS
 #define NLISTS 1
 #endif
-#ifndef NITERS
-#define NITERS 1
-#endif
 #ifndef NTHREADS
 #define NTHREADS 2
 #endif
 #ifndef NELEMENTS
-#define NELEMENTS 4
+#define NELEMENTS 6
+#endif
+#ifndef NITERS
+#define NITERS 1
 #endif
 
 // MC_ignore
 
-#ifdef STARPU_HAVE_SIMGRID_MUTEX_H
 sg_mutex_t mutex[NLISTS];
-#define mutex_lock(l) sg_mutex_lock(l)
-#define mutex_unlock(l) sg_mutex_unlock(l)
-#else
-xbt_mutex_t mutex[NLISTS];
-#define mutex_lock(l) xbt_mutex_acquire(l)
-#define mutex_unlock(l) xbt_mutex_release(l)
-#endif
-
 
 LIST_TYPE(foo,
 		unsigned prio;
@@ -97,8 +80,8 @@ void check_list_prio(struct foo_prio_list *list)
 	     cur  = foo_prio_list_next(list, cur))
 	{
 		if (cur->prio == lastprio)
-                        /* For same prio, back elements should never get before
-                         * front elements */
+			/* For same prio, back elements should never get before
+			 * front elements */
 			MC_assert(!(back && !cur->back));
 		else
 			MC_assert(lastprio > cur->prio);
@@ -107,7 +90,7 @@ void check_list_prio(struct foo_prio_list *list)
 	}
 }
 
-int worker(int argc, char *argv[])
+void worker(int argc, char *argv[])
 {
 	unsigned myrank = atoi(argv[0]);
 	unsigned i, n, l, iter;
@@ -128,13 +111,13 @@ int worker(int argc, char *argv[])
 			elem->prio = res%10;
 			lrand48_r(&buffer, &res);
 			elem->back = res%2;
-			mutex_lock(mutex[l]);
+			sg_mutex_lock(mutex[l]);
 			if (elem->back)
 				foo_prio_list_push_back(&mylist[l], elem);
 			else
 				foo_prio_list_push_front(&mylist[l], elem);
 			check_list_prio(&mylist[l]);
-			mutex_unlock(mutex[l]);
+			sg_mutex_unlock(mutex[l]);
 		}
 
 		for (i = 0; i < NELEMENTS; i++)
@@ -142,38 +125,45 @@ int worker(int argc, char *argv[])
 			lrand48_r(&buffer, &res);
 			n = res%(NELEMENTS-i);
 
-			mutex_lock(mutex[l]);
+			sg_mutex_lock(mutex[l]);
 			for (elem  = foo_prio_list_begin(&mylist[l]);
 			     n--;
 			     elem  = foo_prio_list_next(&mylist[l], elem))
 				;
 			foo_prio_list_erase(&mylist[l], elem);
 			check_list_prio(&mylist[l]);
-			mutex_unlock(mutex[l]);
+			sg_mutex_unlock(mutex[l]);
 		}
 
 		/* horrible way to wait for list getting empty */
-#ifdef HAVE_SG_ACTOR_SLEEP_FOR
 		sg_actor_sleep_for(1000);
-#else
-		MSG_process_sleep(1000);
-#endif
 	}
-
-	return 0;
 }
 
-int master(int argc, char *argv[])
+void master(int argc, char *argv[])
 {
-	unsigned i, l;
+}
+
+int main(int argc, char *argv[])
+{
+	unsigned l, i;
+
+	if (argc < 3)
+	{
+		fprintf(stderr,"usage: %s platform.xml host\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
+
+	printf("Running with:\n- %d threads\n- %d lists\n- %d elements\n- %d iterations\n", NTHREADS, NLISTS, NELEMENTS, NITERS);
+
+	srand48(0);
+	simgrid_init(&argc, argv);
+	sg_cfg_set_int("contexts/stack-size", 128);
+	simgrid_load_platform(argv[1]);
 
 	for (l = 0; l < NLISTS; l++)
 	{
-#ifdef STARPU_HAVE_SIMGRID_MUTEX_H
 		mutex[l] = sg_mutex_init();
-#else
-		mutex[l] = xbt_mutex_init();
-#endif
 		foo_prio_list_init(&mylist[l]);
 	}
 
@@ -184,31 +174,9 @@ int master(int argc, char *argv[])
 		char **args = malloc(sizeof(char*)*2);
 		args[0] = s;
 		args[1] = NULL;
-		MSG_process_create_with_arguments("test", worker, NULL, MSG_host_self(), 1, args);
+		sg_actor_create("test", sg_host_by_name(argv[2]), worker, 1, args);
 	}
 
-	return 0;
-}
-
-int main(int argc, char *argv[])
-{
-	if (argc < 3)
-	{
-		fprintf(stderr,"usage: %s platform.xml host\n", argv[0]);
-		exit(EXIT_FAILURE);
-	}
-	srand48(0);
-	MSG_init(&argc, argv);
-#ifdef HAVE_SG_CFG_SET_INT
-	sg_cfg_set_int("contexts/stack-size", 128);
-#elif SIMGRID_VERSION_MAJOR < 3 || (SIMGRID_VERSION_MAJOR == 3 && SIMGRID_VERSION_MINOR < 13)
-	extern xbt_cfg_t _sg_cfg_set;
-	xbt_cfg_set_int(_sg_cfg_set, "contexts/stack-size", 128);
-#else
-	xbt_cfg_set_int("contexts/stack-size", 128);
-#endif
-	MSG_create_environment(argv[1]);
-	MSG_process_create("master", master, NULL, MSG_get_host_by_name(argv[2]));
-	MSG_main();
+	simgrid_run();
 	return 0;
 }
