@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2015,2017                           CNRS
- * Copyright (C) 2014                                     Inria
+ * Copyright (C) 2014-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,16 +18,20 @@
 #include "../helper.h"
 #include <stdio.h>
 
+/*
+ * Check the starpu_omp_init/shutdown calls.
+ */
+
 #if !defined(STARPU_OPENMP)
-int main(int argc, char **argv)
+int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
 #else
-int
-main (int argc, char *argv[])
+int main(void)
 {
 	int ret = starpu_omp_init();
+	if (ret == -EINVAL) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_omp_init");
 	starpu_omp_shutdown();
 	return 0;
