@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014,2015,2017,2019                      CNRS
- * Copyright (C) 2014,2016                                Inria
- * Copyright (C) 2017                                     Université de Bordeaux
+ * Copyright (C) 2014-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -56,8 +54,7 @@ void parallel_region_f(void *buffers[], void *args)
 	printf("[tid %p] task thread = %d\n", (void *)tid, worker_id);
 }
 
-int
-main (void)
+int main(void)
 {
 	struct starpu_omp_parallel_region_attr attr;
 	memset(&attr, 0, sizeof(attr));
