@@ -1,8 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2012, 2015-2016  Université de Bordeaux
- * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,12 +15,12 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
 #include <starpu.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include "../helper.h"
 #include <common/thread.h>
 
@@ -47,6 +46,8 @@ static unsigned cnt = NTASKS;
 
 static void callback(void *arg)
 {
+	(void)arg;
+
 	unsigned res = STARPU_ATOMIC_ADD(&cnt, -1);
 	ANNOTATE_HAPPENS_BEFORE(&cnt);
 
@@ -60,11 +61,14 @@ static void callback(void *arg)
 	}
 }
 
-void codelet_sleep(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void codelet_sleep(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
-	usleep(TASKDURATION);
+	starpu_usleep(TASKDURATION);
 }
 
 static struct starpu_perfmodel model =
@@ -77,7 +81,9 @@ static struct starpu_codelet cl =
 {
 	.cpu_funcs = {codelet_sleep},
 	.cuda_funcs = {codelet_sleep},
-        .opencl_funcs = {codelet_sleep},
+#ifndef STARPU_SIMGRID
+	.opencl_funcs = {codelet_sleep},
+#endif
 	.cpu_funcs_name = {"codelet_sleep"},
 	.nbuffers = 1,
 	.modes = {STARPU_R},
@@ -111,7 +117,7 @@ int main(int argc, char **argv)
 
 	starpu_data_partition(handle, &f);
 
-	snprintf(symbolname, 128, "overlap_sleep_%d_%u", VECTORSIZE, TASKDURATION);
+	snprintf(symbolname, sizeof(symbolname), "overlap_sleep_%d_%u", VECTORSIZE, TASKDURATION);
 
 	model.symbol = symbolname;
 
@@ -131,6 +137,7 @@ int main(int argc, char **argv)
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
 
+	starpu_do_schedule();
 	STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 	if (!finished)
 		STARPU_PTHREAD_COND_WAIT(&cond, &mutex);
@@ -138,14 +145,15 @@ int main(int argc, char **argv)
 
 	starpu_data_unpartition(handle, STARPU_MAIN_RAM);
 	starpu_data_unregister(handle);
-	starpu_free(buffer);
+	starpu_free_noflag(buffer, NTASKS*VECTORSIZE*sizeof(char));
 	starpu_shutdown();
 
 	STARPU_RETURN(EXIT_SUCCESS);
 
 enodev:
+	starpu_data_unpartition(handle, STARPU_MAIN_RAM);
 	starpu_data_unregister(handle);
-	starpu_free(buffer);
+	starpu_free_noflag(buffer, NTASKS*VECTORSIZE*sizeof(char));
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
  	 * could perform the kernel, so this is not an error from StarPU */
