@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -46,7 +46,7 @@ void codelet_null(void *descr[], void *_args)
 
 //	FPRINTF(stderr, "WORKERSIZE : %d\n", worker_size);
 
-	usleep(1000/worker_size);
+	starpu_usleep(1000./worker_size);
 #if 0
 	int id = starpu_worker_get_id();
 	int combined_id = starpu_combined_worker_get_id();
@@ -62,7 +62,7 @@ static struct starpu_codelet cl =
 	.cpu_funcs = {codelet_null},
 	.cpu_funcs_name = {"codelet_null"},
 	.cuda_funcs = {codelet_null},
-        .opencl_funcs = {codelet_null},
+	.opencl_funcs = {codelet_null},
 	.nbuffers = 1,
 	.modes = {STARPU_R}
 };
@@ -72,9 +72,9 @@ int main(void)
 {
 	int ret;
 
-        struct starpu_conf conf;
+	struct starpu_conf conf;
 	starpu_conf_init(&conf);
-        conf.sched_policy_name = "peager";
+	conf.sched_policy_name = "peager";
 
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
@@ -102,17 +102,17 @@ int main(void)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_wait_for_all");
 
 	starpu_data_unregister(v_handle);
-	starpu_free(v);
+	starpu_free_noflag(v, VECTORSIZE*sizeof(unsigned));
 	starpu_shutdown();
 
 	return EXIT_SUCCESS;
 
 enodev:
 	starpu_data_unregister(v_handle);
-	starpu_free(v);
+	starpu_free_noflag(v, VECTORSIZE*sizeof(unsigned));
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
- 	 * could perform the kernel, so this is not an error from StarPU */
+	 * could perform the kernel, so this is not an error from StarPU */
 	starpu_shutdown();
 	return STARPU_TEST_SKIPPED;
 }
