@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2020       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,8 +29,9 @@
 #include <starpu.h>
 #include <common/utils.h>
 #include <common/uthash.h>
-#include <core/perfmodel/perfmodel.h> // we need to browse the list associated to history-based models
-                                      // just like in starpu_perfmodel_plot
+// we need to browse the list associated to history-based models
+// just like in starpu_perfmodel_plot
+#include <core/perfmodel/perfmodel.h>
 
 #define STRHEADCMP(s, head) strncmp(s, head, strlen(head))
 
@@ -57,7 +59,8 @@ struct _footprint_list* add_footprint(struct _footprint_list* list, uint32_t foo
 	if(l) return list;
 	else
 	{
-		struct _footprint_list * res = malloc(sizeof(struct _footprint_list));
+		struct _footprint_list *res;
+		_STARPU_MALLOC(res, sizeof(struct _footprint_list));
 		res->footprint = footprint;
 		res->next = list;
 		return res;
@@ -82,7 +85,7 @@ void get_comb_name(int comb, char* name, int name_size)
 void print_archs(FILE* output)
 {
 	int nb_workers = 0;
-	unsigned workerid, node; int comb, old_comb = -1;
+	unsigned workerid, node, src, dst; int comb, old_comb = -1;
 
 	fprintf(output, "%%rec: worker_count\n\n");
 	for (workerid = 0; workerid < starpu_worker_get_count(); workerid++)
@@ -142,6 +145,21 @@ void print_archs(FILE* output)
 			fprintf(output, "\n");
 		fprintf(output, "\n");
 	}
+	fprintf(output, "%%rec: memory_performance\n\n");
+	for (src = 0; src < starpu_memory_nodes_get_count(); src++)
+	{
+		for (dst = 0; dst < starpu_memory_nodes_get_count(); dst++)
+		{
+			if (src != dst)
+			{
+				fprintf(output, "MemoryNodeSrc: %u\n", src);
+				fprintf(output, "MemoryNodeDst: %u\n", dst);
+				fprintf(output, "Bandwidth: %f\n", starpu_transfer_bandwidth(src, dst));
+				fprintf(output, "Latency: %f\n", starpu_transfer_latency(src, dst));
+				fprintf(output, "\n");
+			}
+		}
+	}
 }
 
 /* output file name */
@@ -152,11 +170,11 @@ static void usage()
 {
 	fprintf(stderr, "Dumps perfmodels to a rec file\n\n");
 	fprintf(stderr, "Usage: %s [ input-file ] [ -o output-file ]\n", PROGNAME);
-        fprintf(stderr, "\n");
+	fprintf(stderr, "\n");
 	fprintf(stderr, "If input or output file names are not given, stdin and stdout are used.");
 	fprintf(stderr, "\n");
-        fprintf(stderr, "Report bugs to <"PACKAGE_BUGREPORT">.");
-        fprintf(stderr, "\n");
+	fprintf(stderr, "Report bugs to <"PACKAGE_BUGREPORT">.");
+	fprintf(stderr, "\n");
 }
 
 static void print_entry(const char *name, const char *archname, FILE *output, struct starpu_perfmodel_history_entry *entry)
@@ -189,14 +207,14 @@ static void parse_args(int argc, char **argv)
 	{
 		switch (c)
 		{
-                case 'h': /* display help */
-		  usage();
-		  exit(EXIT_SUCCESS);
-		  break;
+		case 'h': /* display help */
+			usage();
+			exit(EXIT_SUCCESS);
+			break;
 
 		case 'o':
-		  poutput = optarg;
-		  break;
+			poutput = optarg;
+			break;
 		case '?':
 		default:
 			fprintf(stderr, "Unrecognized option: -%c\n", optopt);
@@ -278,7 +296,7 @@ int main(int argc, char **argv)
 					HASH_FIND_STR(models, model_name, model);
 					if (model == NULL)
 					{
-						model = malloc(sizeof(*model));
+						_STARPU_MALLOC(model, sizeof(*model));
 						model->name = model_name;
 						model->footprints = NULL;
 						memset(&model->model, 0, sizeof(model->model));
@@ -330,9 +348,9 @@ int main(int argc, char **argv)
 
 			HASH_ITER(hh, models, model, tmp)
 			{
-				struct _footprint_list* l = model->footprints, *ltmp;
+				struct _footprint_list* lf = model->footprints, *ltmp;
 				int comb;
-				while(l)
+				while(lf)
 				{
 					for(comb = 0; comb < nb_combs; comb++)
 					{
@@ -357,7 +375,7 @@ int main(int argc, char **argv)
 						else while(ptr)
 						     {
 							     struct starpu_perfmodel_history_entry *entry = ptr->entry;
-							     if(entry->footprint == l->footprint)
+							     if(entry->footprint == lf->footprint)
 							     {
 								     print_entry(model->name, archname, output, entry);
 								     break;
@@ -365,13 +383,15 @@ int main(int argc, char **argv)
 							     ptr=ptr->next;
 						     }
 					}
-					ltmp = l->next;
-					free(l);
-					l = ltmp;
+					ltmp = lf->next;
+					free(lf);
+					lf = ltmp;
 				}
 
+				starpu_perfmodel_unload_model(&model->model);
 				free(model->name);
 				HASH_DEL(models, model);
+				free(model);
 			}
 		}
 		fclose(input);
@@ -380,73 +400,80 @@ int main(int argc, char **argv)
 	{
 		fprintf(output, "%%rec: timing\n\n");
 
-		char *path;
+		char **paths;
 		DIR *dp;
 		struct dirent *ep;
+		int i;
 
-		path = _starpu_get_perf_model_dir_codelet();
-		dp = opendir(path);
-		if (dp != NULL)
+		paths = _starpu_get_perf_model_dirs_codelet();
+		for(i=0 ; paths[i] != NULL ; i++)
 		{
-			while ((ep = readdir(dp)))
+			_STARPU_DISP("Processing directory %s\n", paths[i]);
+			dp = opendir(paths[i]);
+			if (dp != NULL)
 			{
-				if (strcmp(ep->d_name, ".") && strcmp(ep->d_name, ".."))
+				while ((ep = readdir(dp)))
 				{
-					int comb, nb_combs;
-					char* symbol = strdup(ep->d_name);
-					char *dot = strrchr(symbol, '.');
-					struct starpu_perfmodel model = {.type = STARPU_PERFMODEL_INVALID };
-
-					if(dot) *dot = '\0';
-					if (starpu_perfmodel_load_symbol(symbol, &model) != 0)
+					if (strcmp(ep->d_name, ".") && strcmp(ep->d_name, ".."))
 					{
-						free(symbol);
-						continue;
-					}
-					if(model.state == NULL)
-					{
-						free(symbol);
-						continue;
-					}
+						int comb, nb_combs;
+						char* symbol = strdup(ep->d_name);
+						char *dot = strrchr(symbol, '.');
+						struct starpu_perfmodel model = {.type = STARPU_PERFMODEL_INVALID };
 
-					_STARPU_DISP("Dumping %s\n", symbol);
-
-					nb_combs = starpu_perfmodel_get_narch_combs();
-					for(comb = 0; comb < nb_combs; ++comb)
-					{
-						char name[32];
-						get_comb_name(comb, name, 32);
-
-						if(!model.state || model.state->nimpls[comb] == 0)
+						if(dot) *dot = '\0';
+						if (starpu_perfmodel_load_symbol(symbol, &model) != 0)
 						{
-							_STARPU_DISP("Symbol %s does not have any implementation on comb %d, not dumping\n", symbol, comb);
-							fprintf(output, "\n");
+							free(symbol);
+							continue;
+						}
+						if(model.state == NULL)
+						{
+							free(symbol);
 							continue;
 						}
 
-						struct starpu_perfmodel_per_arch *arch_model = &model.state->per_arch[comb][0];
-						struct starpu_perfmodel_history_list *ptr;
+						_STARPU_DISP("Dumping %s\n", symbol);
 
-						ptr = arch_model->list;
-						if(!ptr)
-							_STARPU_DISP("Symbol %s for comb %d does not have history based model, not dumping\n", symbol,  comb);
-						else while(ptr)
-						     {
-							     print_entry(symbol, name, output, ptr->entry);
-							     ptr=ptr->next;
-						     }
+						nb_combs = starpu_perfmodel_get_narch_combs();
+						for(comb = 0; comb < nb_combs; ++comb)
+						{
+							char name[32];
+							get_comb_name(comb, name, 32);
+
+							if(!model.state || model.state->nimpls[comb] == 0)
+							{
+								_STARPU_DISP("Symbol %s does not have any implementation on comb %d, not dumping\n", symbol, comb);
+								fprintf(output, "\n");
+								continue;
+							}
+
+							struct starpu_perfmodel_per_arch *arch_model = &model.state->per_arch[comb][0];
+							struct starpu_perfmodel_history_list *ptr;
+
+							ptr = arch_model->list;
+							if(!ptr)
+								_STARPU_DISP("Symbol %s for comb %d does not have history based model, not dumping\n", symbol,  comb);
+							else
+							{
+								while(ptr)
+								{
+									print_entry(symbol, name, output, ptr->entry);
+									ptr=ptr->next;
+								}
+							}
+						}
+						starpu_perfmodel_unload_model(&model);
+						free(symbol);
 					}
-					starpu_perfmodel_unload_model(&model);
-					free(symbol);
 				}
+				closedir(dp);
+			}
+			else
+			{
+				_STARPU_DISP("Could not open the perfmodel directory <%s>: %s\n", paths[i], strerror(errno));
 			}
-			closedir (dp);
 		}
-		else
-		{
-			_STARPU_DISP("Could not open the perfmodel directory <%s>: %s\n", path, strerror(errno));
-		}
-
 		print_archs(output);
 	}
 
