@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2016-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2017       Erwan Leria
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -97,14 +97,13 @@ static struct task
 	struct starpu_rbtree_node node;
 	UT_hash_handle hh;
 	jobid_t jobid;
-        int iteration;
+	int iteration;
 	long submit_order;
 	jobid_t *deps;
 	size_t ndependson;
 	struct starpu_task task;
 	enum task_type type;
 	int reg_signal;
-
 } *tasks;
 
 /* Record handles */
@@ -124,6 +123,161 @@ static struct perfmodel
 	char * model_name;
 } * model_hash;
 
+
+
+/*
+ * Replay data interface
+ * We don't care about many things anyway, essentially only sizes.
+ */
+
+struct replay_interface
+{
+	enum starpu_data_interface_id id;
+	starpu_data_handle_t orig_handle;
+	size_t size;
+	size_t alloc_size;
+	size_t max_size;
+};
+
+static struct starpu_data_interface_ops replay_interface_ops;
+static void register_replay(starpu_data_handle_t handle, int home_node, void *data_interface)
+{
+	(void) home_node;
+	struct replay_interface *replay_interface = data_interface;
+	unsigned node;
+	for (node = 0; node < STARPU_MAXNODES; node++)
+	{
+		struct replay_interface *local_interface =
+			starpu_data_get_interface_on_node(handle, node);
+
+		local_interface->id = replay_interface->id;
+		local_interface->orig_handle = replay_interface->orig_handle;
+		local_interface->size = replay_interface->size;
+		local_interface->alloc_size = replay_interface->alloc_size;
+		local_interface->max_size = replay_interface->max_size;
+	}
+}
+
+static void replay_data_register(starpu_data_handle_t *handleptr, starpu_data_handle_t orig_handle, int home_node, size_t size, size_t alloc_size, size_t max_size)
+{
+	struct replay_interface interface =
+	{
+		.id = replay_interface_ops.interfaceid,
+		.orig_handle = orig_handle,
+		.size = size,
+		.alloc_size = alloc_size,
+		.max_size = max_size,
+	};
+
+	starpu_data_register(handleptr, home_node, &interface, &replay_interface_ops);
+}
+
+static size_t replay_get_size(starpu_data_handle_t handle)
+{
+	struct replay_interface *interface =
+		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+	return interface->size;
+}
+
+static size_t replay_get_alloc_size(starpu_data_handle_t handle)
+{
+	struct replay_interface *interface =
+		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+	return interface->alloc_size;
+}
+
+static size_t replay_get_max_size(starpu_data_handle_t handle)
+{
+	struct replay_interface *interface =
+		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+	return interface->max_size;
+}
+
+static uint32_t replay_footprint(starpu_data_handle_t handle)
+{
+	return starpu_hash_crc32c_be(replay_get_size(handle), 0);
+}
+
+static int replay_compare(void *data_interface_a, void *data_interface_b)
+{
+	struct replay_interface *replay_a = data_interface_a;
+	struct replay_interface *replay_b = data_interface_b;
+
+	/* Two variables are considered compatible if they have the same size */
+	return replay_a->size == replay_b->size;
+}
+
+static void display_replay(starpu_data_handle_t handle, FILE *f)
+{
+	struct replay_interface *replay_interface =
+		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+
+	fprintf(f, "%lu/%lu/%lu\t",
+			(unsigned long) replay_interface->size,
+			(unsigned long) replay_interface->alloc_size,
+			(unsigned long) replay_interface->max_size);
+}
+
+static starpu_ssize_t describe_replay(void *data_interface, char *buf, size_t size)
+{
+	struct replay_interface *replay_interface = data_interface;
+	return snprintf(buf, size, "r%lu/%lu/%lu\t",
+			(unsigned long) replay_interface->size,
+			(unsigned long) replay_interface->alloc_size,
+			(unsigned long) replay_interface->max_size);
+}
+
+static starpu_ssize_t allocate_replay_on_node(void *data_interface, unsigned dst_node)
+{
+	struct replay_interface *replay_interface = data_interface;
+	starpu_memory_allocate(dst_node, replay_interface->alloc_size, STARPU_MEMORY_OVERFLOW);
+	return 0;
+}
+
+static void free_replay_on_node(void *data_interface, unsigned dst_node)
+{
+	struct replay_interface *replay_interface = data_interface;
+	starpu_memory_deallocate(dst_node, replay_interface->alloc_size);
+}
+
+static int replay_copy(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data)
+{
+	(void) dst_interface;
+	struct replay_interface *src = src_interface;
+
+	/* We don't care about pointers */
+	return starpu_interface_copy(1, 0, src_node, 1, 0, dst_node, src->size, async_data);
+}
+
+static const struct starpu_data_copy_methods replay_copy_data_methods =
+{
+	.any_to_any = replay_copy,
+};
+
+static struct starpu_data_interface_ops replay_interface_ops =
+{
+	.register_data_handle = register_replay,
+	.allocate_data_on_node = allocate_replay_on_node,
+	.free_data_on_node = free_replay_on_node,
+	.copy_methods = &replay_copy_data_methods,
+	.get_size = replay_get_size,
+	.get_alloc_size = replay_get_alloc_size,
+	.get_max_size = replay_get_max_size,
+	.footprint = replay_footprint,
+	.compare = replay_compare,
+	.interfaceid = STARPU_UNKNOWN_INTERFACE_ID,
+	.interface_size = sizeof(struct replay_interface),
+	.display = display_replay,
+	.pack_data = NULL,
+	.peek_data = NULL,
+	.unpack_data = NULL,
+	.describe = describe_replay,
+
+	/* We want to observe actual allocations/deallocations */
+	.dontcache = 1,
+};
+
+
 /* [SUBMITORDER] The tree of the submit order */
 
 static struct starpu_rbtree tree = STARPU_RBTREE_INITIALIZER;
@@ -273,11 +427,25 @@ static void arrays_managing(int mode)
 	{
 		_STARPU_MALLOC(handles_ptr, sizeof(*handles_ptr) * nb_parameters);
 		_STARPU_MALLOC(modes_ptr, sizeof(*modes_ptr) * nb_parameters);
-		_STARPU_CALLOC(reg_signal, nb_parameters, sizeof(char *));
+		_STARPU_CALLOC(reg_signal, nb_parameters, sizeof(char));
 
 	}
 }
 
+static unsigned count_number_tokens(const char* buffer, const char* delim)
+{
+	char* dup = strdup(buffer);
+	int result = 0;
+	char* token = strtok(dup, delim);
+	while(token != NULL)
+	{
+		++result;
+		token = strtok(NULL, delim);
+	}
+	free(dup);
+	return result;
+}
+
 /* Check if a handle hasn't been registered yet */
 static void variable_data_register_check(size_t * array_of_size, int nb_handles)
 {
@@ -309,7 +477,9 @@ static void variable_data_register_check(size_t * array_of_size, int nb_handles)
 
 				handles_cell->handle = handles_ptr[h]; /* Get the hidden key (initial handle from the file) to store it as a key*/
 
-				starpu_variable_data_register(handles_ptr+h, STARPU_MAIN_RAM, (uintptr_t) 1, array_of_size[h]);
+				replay_data_register(handles_ptr+h, handles_ptr[h],
+						modes_ptr[h] & STARPU_R ? STARPU_MAIN_RAM : -1,
+						array_of_size[h], array_of_size[h], array_of_size[h]);
 
 				handles_cell->mem_ptr = handles_ptr[h]; /* Store the new value of the handle into the hash table */
 
@@ -415,6 +585,7 @@ int submit_tasks(void)
 					struct starpu_task *task = starpu_task_create();
 					int ret;
 					task->cl = NULL;
+					task->name = "fake task for submit order";
 					ret = starpu_task_submit(task);
 					STARPU_ASSERT(ret == 0);
 					last_submitorder++;
@@ -447,6 +618,8 @@ int submit_tasks(void)
 				starpu_iteration_push(currentTask->iteration);
 
 			applySchedRec(&currentTask->task, currentTask->submit_order);
+			if (currentTask->submit_order == -1)
+				currentTask->task.no_submitorder = 1;
 			int ret_val = starpu_task_submit(&currentTask->task);
 
 			if (!(currentTask->iteration == -1))
@@ -475,12 +648,11 @@ int submit_tasks(void)
 		else
 		{
 			fix_wontuse_handle(currentTask); /* Add the handle in the wontuse task */
-                        /* FIXME: can not actually work properly since we have
-                         * disabled sequential consistency, so we don't have any
-                         * easy way to make this wait for the last task that
-                         * wrote to the handle. */
 			if (currentTask->task.handles[0])
+			{
 				starpu_data_wont_use(currentTask->task.handles[0]);
+				last_submitorder++;
+			}
 		}
 
 		currentNode = starpu_rbtree_next(currentNode);
@@ -504,8 +676,6 @@ static void usage(const char *program)
 
 int main(int argc, char **argv)
 {
-	starpu_data_set_default_sequential_consistency_flag(0);
-
 	FILE *rec;
 	char *s;
 	const char *tasks_rec = NULL;
@@ -515,6 +685,8 @@ int main(int argc, char **argv)
 
 	unsigned long nread_tasks = 0;
 
+	/* FIXME: we do not support data with sequential consistency disabled */
+
 	_STARPU_MALLOC(s, s_allocated);
 	dependson_size = REPLAY_NMAX_DEPENDENCIES; /* Change the value of REPLAY_NMAX_DEPENCIES to modify the number of dependencies */
 	_STARPU_MALLOC(dependson, dependson_size * sizeof (* dependson));
@@ -721,7 +893,7 @@ int main(int argc, char **argv)
 						int one = 0;
 						for (i = 0; i < narch ; i++)
 						{
-							struct starpu_perfmodel_arch *arch = starpu_perfmodel_arch_comb_fetch(i);
+							arch = starpu_perfmodel_arch_comb_fetch(i);
 							perfTime[i] = starpu_perfmodel_history_based_expected_perf(&realmodel->perfmodel, arch, footprint);
 							if (!(perfTime[i] == 0 || isnan(perfTime[i])))
 								one = 1;
@@ -829,31 +1001,24 @@ int main(int argc, char **argv)
 		}
 		else if (TEST("Parameters"))
 		{
-			/* Parameters line format is PARAM1 PARAM2 (...)PARAMi (...)PARAMn */
-			char * param_str = s + 12;
-			int count = 0;
-
-			for (i = 0 ; param_str[i] != '\n'; i++)
-			{
-				if (param_str[i] == ' ') /* Checking the number of ' ' (space), assuming that the file is not corrupted */
-				{
-					count++;
-				}
-			}
-
-			nb_parameters = count + 1; /* There is one space per paramater except for the last one, that's why we have to add +1 (dirty programming) */
-
-			/* This part of the algorithm will determine if it needs static or dynamic arrays */
-			alloc_mode = set_alloc_mode(nb_parameters);
-			arrays_managing(alloc_mode);
-
+			/* Nothing to do */
 		}
 		else if (TEST("Handles"))
 		{
+			*ln = 0;
 			char *buffer = s + 9;
 			const char *delim = " ";
-			char *token = strtok(buffer, delim);
+			unsigned nb_parameters_line = count_number_tokens(buffer, delim);
 
+			if(nb_parameters == 0)
+			{
+				nb_parameters = nb_parameters_line;
+				arrays_managing(set_alloc_mode(nb_parameters));
+			}
+			else
+				STARPU_ASSERT(nb_parameters == nb_parameters_line);
+
+			char* token = strtok(buffer, delim);
 			for (i = 0 ; i < nb_parameters ; i++)
 			{
 				STARPU_ASSERT(token);
@@ -880,10 +1045,21 @@ int main(int argc, char **argv)
 		}
 		else if (TEST("Modes"))
 		{
+			*ln = 0;
 			char * buffer = s + 7;
-			int mode_i = 0;
+			unsigned mode_i = 0;
 			const char * delim = " ";
-			char * token = strtok(buffer, delim);
+			unsigned nb_parameters_line = count_number_tokens(buffer, delim);
+
+			if(nb_parameters == 0)
+			{
+				nb_parameters = nb_parameters_line;
+				arrays_managing(set_alloc_mode(nb_parameters));
+			}
+			else
+				STARPU_ASSERT(nb_parameters == nb_parameters_line);
+
+			char* token = strtok(buffer, delim);
 
 			while (token != NULL && mode_i < nb_parameters)
 			{
@@ -913,13 +1089,23 @@ int main(int argc, char **argv)
 		}
 		else if (TEST("Sizes"))
 		{
+			*ln = 0;
 			char *  buffer = s + 7;
 			const char * delim = " ";
-			char * token = strtok(buffer, delim);
-			int k = 0;
+			unsigned nb_parameters_line = count_number_tokens(buffer, delim);
+			unsigned k = 0;
+
+			if(nb_parameters == 0)
+			{
+				nb_parameters = nb_parameters_line;
+				arrays_managing(set_alloc_mode(nb_parameters));
+			}
+			else
+				STARPU_ASSERT(nb_parameters == nb_parameters_line);
 
 			_STARPU_MALLOC(sizes_set, nb_parameters * sizeof(size_t));
 
+			char * token = strtok(buffer, delim);
 			while (token != NULL && k < nb_parameters)
 			{
 				sizes_set[k] = strtol(token, NULL, 10);
@@ -973,7 +1159,7 @@ eof:
 		starpu_data_unregister(handle->mem_ptr);
 		HASH_DEL(handles_hash, handle);
 		free(handle);
-        }
+	}
 
 	struct perfmodel *model_s=NULL, *modeltmp=NULL;
 	HASH_ITER(hh, model_hash, model_s, modeltmp)
@@ -982,7 +1168,7 @@ eof:
 		HASH_DEL(model_hash, model_s);
 		free(model_s->model_name);
 		free(model_s);
-        }
+	}
 
 	struct task *task=NULL, *tasktmp=NULL;
 	HASH_ITER(hh, tasks, task, tasktmp)
@@ -1001,7 +1187,7 @@ eof:
 		free(task->deps);
 		starpu_rbtree_remove(&tree, &task->node);
 		free(task);
-        }
+	}
 
 	starpu_shutdown();
 	return 0;
